

### Gibb et al, 'Land use has global and systematic effects on local zoonotic host communities"
# Script 00: General functions for analysis

# ------------ dependencies ------------- 

library(lme4)
library(ggplot2)
library(nlme)
library(plyr)
library(dismo)
library(geosphere)
library(spdep)
library(rgdal)
library(INLA)
library(viridis)
library(raster)
library(KernSmooth)
library(dplyr)



# ---------------- 1. FUNCTIONS FOR PROCESSING OF PREDICTS AND HOSTS DATASET ----------------


# ---- Print summary details of PREDICTS df ----
#' @param df PREDICTS dataframe

printDetails = function(df){
  print(paste( "Num studies:", length(unique(df$SS)), 
               "Num sites:", length(unique(df$SSS)), 
               "Total species:", length(unique(df$bgb)),
               "Num hosts:", length(unique( df$bgb[ df$HumanInfective_Any==1 ] )),
               "Num non-hosts:", length(unique( df$bgb[ df$HumanInfective_Any==0 ] )),
               "Num zoonotic strict hosts:", length(unique( df$bgb[ df$IsZoonotic_Strict == 1 ] )),
               "Num non-zoonotic strict hosts:", length(unique( df$bgb[ df$IsZoonotic_Strict == 0 ] )) ))
}


# ---- Remove domestic species -----
#' @param df PREDICTS dataframe

removeDomestics = function(df){
  return( df[ df$IsDomestic == 0, ] )
}

# ---- Remove absence points ----
#' @param df PREDICTS dataframe

removeAbsences = function(df){ 
  return( df[ df$Effort_corrected_measurement > 0 , ] )
}

# ---- Extract spatial data from raster (e.g. HPD, climate) ----
#' @param df PREDICTS dataframe
#' @param raster spatial data raster
#' @param variable_name name to call extracted data in returned df

# arguments: df with latlon points to extract; raster of spatial variable; name for variable in updated df
extractRasterData = function(df, raster, variable_name){
  
  # create column for storing results
  df[ , variable_name ] = 0
  
  # create spdf
  spdf = df[ , c("Longitude", "Latitude")]
  spdf$row_num = 1:nrow(spdf)
  coordinates(spdf) = ~Longitude+Latitude
  
  # extract from raster
  proj4string(spdf) = CRS(projection(raster))
  spdf$extracted = extract(raster, spdf)
  
  # match to df
  df[ spdf$row_num, variable_name ] = spdf$extracted
  return(df)
}


# ---- Aggregate land use categories (updated 16/02/18) ----
#' @param df PREDICTS dataframe

prepareLandUse = function(df){
  
  # land use mappings
  plu = data.frame(Predominant_land_use = c("Primary vegetation","Young secondary vegetation","Intermediate secondary vegetation","Mature secondary vegetation", "Secondary vegetation (indeterminate age)", "Plantation forest","Pasture","Cropland","Urban"),
                   PLU2 = c("1_Primary", "Secondary","Secondary","Secondary", "Secondary", "Plantation forest", "Pasture", "Cropland", "Urban"),
                   LandUseType = c("1_Primary","Secondary","Secondary","Secondary","Secondary","Managed","Managed","Managed","Urban"),
                   stringsAsFactors = F)
  
  # merge and reshuffle columns
  df$foo = 1:nrow(df)
  df1 = merge(df, plu, by="Predominant_land_use", all.x=T)
  df1 = df1[ , c(2:ncol(df1), 1) ]
  df1 = df1[ order(df1$foo), ]
  
  # remove 'use' from intensity
  df1$Use_intensity = gsub(" use","", as.vector(df1$Use_intensity) )
  
  # combine use intensities into three states (minimal, substantial, cannot decide)
  df1$UseIntensity = ifelse(df1$Use_intensity == "Minimal", "Minimal", "Substantial")
  df1$UseIntensity[ df1$Use_intensity == "Cannot decide" ] = "Cannot decide"
  
  # combine with land use type for composite variable for modelling
  df1$LU_UI = paste(df1$LandUseType, df1$UseIntensity, sep="_")
  df1$LU_UI[ df1$LU_UI == "1_Primary_Substantial" ] = "Primary_Substantial"
  
  return( as.data.frame(df1, stringsAsFactors=F) )
}


# ------- rename columns depending on which data subset being analysed -------

sortColumns = function(df, analysis){
  
  # rename 'cargo-carrier' columns
  if(analysis == "cargocarrier"){
    to_rename = names(df)[ grep("CargoCarrier", names(df)) ]
    renamed = unlist(lapply(lapply(strsplit(to_rename, "_"), function(x) x[1:length(x)-1]), paste, collapse="_"))
    renamed[which(renamed=="bgb")] = "bgb1"
    names(df)[ grep("CargoCarrier", names(df)) ] = renamed 
  }
  
  # rename 'mammal hosts' columns
  if(analysis == "mammalhosts"){
    to_rename = names(df)[ grep("MammalHosts", names(df)) ]
    renamed = unlist(lapply(lapply(strsplit(to_rename, "_"), function(x) x[1:length(x)-1]), paste, collapse="_"))
    renamed[which(renamed=="bgb")] = "bgb1"
    names(df)[ grep("MammalHosts", names(df)) ] = renamed 
  }
  return(df)
}


# ---- Calculate host richness and abundance ----
#' @param df PREDICTS dataframe
# this function nested within aggregateStudy and aggregateSite functions

addHostMetrics = function(df){
  
  # total richness and host richness (1 for each species record with non-absence record associated)
  # calculate for host status, strict host status, and weighted for effort
  df$SR = ifelse(df$Effort_corrected_measurement > 0, 1, 0)
  df$HostSR = df$SR * df$HumanInfective_Any
  df$HostSR_Zoonotic = df$SR * df$IsZoonotic_Strict
  
  # effort weighted
  df$SR_EffWeight = df$SR * df$ProbDetection
  df$HostSR_EffWeight = df$HostSR * df$ProbDetection
  df$HostSR_Zoonotic_EffWeight = df$HostSR_Zoonotic * df$ProbDetection

  # abundance and host abundance (i.e. 0 for nonhosts)
  df$Abundance = df$Effort_corrected_measurement
  df$HostAbundance = df$Abundance * df$HumanInfective_Any
  df$HostAbundance_Zoonotic = df$Abundance * df$IsZoonotic_Strict
  
  # effort weighted
  df$Abundance_EffWeight = df$Abundance * df$ProbDetection
  df$HostAbundance_EffWeight = df$HostAbundance * df$ProbDetection
  df$HostAbundance_Zoonotic_EffWeight = df$HostAbundance_Zoonotic * df$ProbDetection

  # set abundance related metrics to NA for occurrence only studies
  df[ df$Diversity_metric_type == "Occurrence", c("Abundance", "HostAbundance", "HostAbundance_Zoonotic",
                                                  "Abundance_EffWeight", "HostAbundance_EffWeight", "HostAbundance_Zoonotic_EffWeight") ] = NA
  return(df)
}


# ---- Create summary data per study (SS) ----
#' @param df PREDICTS dataframe

aggregateStudy = function(df){
  
  # add host metrics
  df1 = addHostMetrics(df)
  
  # summarise
  df2 = ddply(df1, .(SS), plyr::summarize,
              NumSites = length(unique(SSS)),
              Latitude = median(Latitude, na.rm=T),
              Longitude = median(Longitude, na.rm=T),
              Country = head(Country, 1), 
              UN_subregion = head(UN_subregion, 1),
              
              # numer of records
              TotalRecs = length(SSS), 
              HostRecs = sum(HostSR, na.rm=T), 
              
              # host metrics
              HostSR = sum( !is.na( unique(HostBinom))),
              TotalSR = sum( !is.na( unique(bgb))),
              HostAbundance = sum(HostAbundance, na.rm=T), 
              TotalAbundance = sum(Abundance, na.rm=T),
              
              # other summary metrics
              LUSampled = length(unique(Predominant_land_use)),
              LUDiv = paste(unique(Predominant_land_use), collapse = ","),
              ClassSampled = length(unique(Class)),
              ClassDiv = paste(unique(Class), collapse=","))
  return(df2)
}


# ----------- Community weighted mean function: for calculating weighted trait values ------------

# applied within aggregateSite function
# if all abundances are equal, then commWeightMean(trait_values) == mean(trait_values)
#' @param traits vector of species trait values
#' @param abundances vector of species abundances

commWeightMean = function(trait_values, abundances){
  if(all(is.na(abundances))) return(NA)
  abundances_weighted = abundances / sum(abundances) # weight abundances
  sum(trait_values*abundances_weighted, na.rm=T) # multiply by trait values and sum
}



# ------------- Calculate PREDICTS data to site level (SSS) prior to community modelling ----------

#' @param df PREDICTS dataframe

aggregateSite = function(df){
  
  # remove absence records
  #df = removeAbsences(df)
  
  # calculate host metrics to summarise
  df1 = addHostMetrics(df)
  
  # summarise diversity metrics by site
  df2 = ddply(df1, .(SSS), plyr::summarize,
              
              # biodiversity and host metrics
              Diversity_metric_type = head(Diversity_metric_type, 1),
              
              # Abundance
              TotalAbundance = sum(Abundance, na.rm=T),
              HostAbundance = sum(HostAbundance, na.rm=T),
              HostAbundance_Zoonotic = sum(HostAbundance_Zoonotic, na.rm=T),
              
              # Abundance effort weighted
              TotalAbundance_EW = sum(Abundance_EffWeight, na.rm=T),
              HostAbundance_EW = sum(HostAbundance_EffWeight, na.rm=T),
              HostAbundance_Zoonotic_EW = sum(HostAbundance_Zoonotic_EffWeight, na.rm=T),
              
              # Species richness
              TotalSR = sum(SR, na.rm=T),
              HostSR = sum(HostSR, na.rm=T),
              HostSR_Zoonotic = sum(HostSR_Zoonotic, na.rm=T),
              
              # Species richness effort weighted
              TotalSR_EW = sum(SR_EffWeight, na.rm=T),
              HostSR_EW = sum(HostSR_EffWeight, na.rm=T),
              HostSR_Zoonotic_EW = sum(HostSR_Zoonotic_EffWeight, na.rm=T),
              
              # publications / study effort
              LogMedianPubs = log( median(NumPubs*SR, na.rm=T) + 1 ), # because absence points included, multiply by SR
              MeanDetectionProb = mean(ProbDetection*SR, na.rm=T),
              CWMDetectionProb = commWeightMean(ProbDetection, Abundance)
              )
  
  # calculate nonhost abundance
  df2$NonHostAbundance = df2$TotalAbundance - df2$HostAbundance
  df2$NonHostAbundance_EW = df2$TotalAbundance_EW - df2$HostAbundance_EW
  df2$NonHostSR = df2$TotalSR - df2$HostSR
  df2$NonHostSR_EW = df2$TotalSR_EW - df2$HostSR_EW
  
  # combine with other relevant columns
  dfx = df1[ !duplicated(df1$SSS), c("SSS", "SS", "SSB", "SSBS", 
                                     "Latitude", "Longitude", 
                                     "Country", "UN_subregion", "Biome", "Realm", "PLU2", "LandUseType", 
                                     "Use_intensity", "UseIntensity", "LU_UI", 
                                     "HumanDensity", "MeanAnnualTemp", "MeanAnnualPrecip") ]
  df2 = full_join(dfx, df2, by="SSS")
  df2$LogHumanDensity = log(df2$HumanDensity+1)
  
  # get taxonomic range sampled per study
  df3 = ddply(df1, .(SS), plyr::summarize, TaxaSampled = paste(unique(Class), collapse=", "))
  df2 = full_join(df2, df3, by="SS")
  
  # add pathogen names (for human diseases)
  df_zoohosts = df1[ df1$HumanInfective_Any == 1, ]
  pathogendiv = ddply(df_zoohosts, .(SS), plyr::summarize,
                      Pathogens_Zoonotic_Strict = paste(unique(unlist(strsplit(Pathogens_Zoonotic_Strict, ", "))), collapse=", "),
                      PathogenDiv = paste(unique(unlist(strsplit(PathogenDiv, ", "))), collapse=", "))
  df2 = full_join(df2, pathogendiv, by="SS")
  
  # set abundance-related metrics to NA for occurrence studies
  df2[ df2$Diversity_metric_type == "Occurrence", c("TotalAbundance", "HostAbundance", "HostAbundance_Zoonotic", 
                                                    "TotalAbundance_EW", "HostAbundance_EW", "HostAbundance_Zoonotic_EW", 
                                                    "NonHostAbundance", "NonHostAbundance_EW", "CWMDetectionProb") ] = NA
  
  # reorder columns
  df2 = df2[ , c("SS", "SSS", "SSB", "SSBS", "Latitude", "Longitude", "Country", "UN_subregion", "Biome", "Realm", # broad info
                 "PLU2", "LandUseType", "Use_intensity", "UseIntensity", "LU_UI", # land use
                 "MeanAnnualTemp", "MeanAnnualPrecip", "HumanDensity", "LogHumanDensity", # other environmental conditions
                 "TaxaSampled", "Diversity_metric_type", # study parameters
                 "TotalSR", "NonHostSR", "HostSR", "HostSR_Zoonotic", # species richness
                 "TotalSR_EW", "NonHostSR_EW", "HostSR_EW", "HostSR_Zoonotic_EW", # species richness effort weighted
                 "TotalAbundance", "NonHostAbundance", "HostAbundance", "HostAbundance_Zoonotic", # abundance effort weighted
                 "TotalAbundance_EW", "NonHostAbundance_EW", "HostAbundance_EW", "HostAbundance_Zoonotic_EW", # abundance effort weighted
                 "LogMedianPubs", "MeanDetectionProb", "CWMDetectionProb", # publication effort
                 "PathogenDiv", "Pathogens_Zoonotic_Strict") ] # parasite information
  
  # remove factors
  df2 = as.data.frame(df2, stringsAsFactors=F)
  df2$SS = as.vector(df2$SS)
  df2$SSS = as.vector(df2$SSS)
  return(df2)
}



# ---- Extract residuals from INLA models ----
#' @param model fitted INLA model
#' @param df dataset used for model fitting
#' @param model_metric name of model response variable
#' @param model_name char; name to give to model in resulting df

getResidINLA = function(model, df, model_metric, model_name){
  
  # extract fitted values
  fitted = as.data.frame(model$summary.fitted.values)
  
  # check length is the same as df
  if(nrow(fitted) != nrow(df)) return("Error: different dataset sizes")
  
  # create df of observed and fitted values
  resid = df[ , c("SSS", "SS", "Latitude", "Longitude", "LandUseType", "UseIntensity", "LU_UI") ]
  resid = cbind(resid, df[ , which( names(df) == model_metric) ])
  names(resid)[8] = "observed"
  resid$model = model_name
  
  # calculate residuals
  resid$fitted = fitted$mean
  resid$resid = log(resid$observed+1) - resid$fitted
  
  # exclude any rows where observed value was NA (i.e. non-abundance records in occurrence data)
  resid = resid[ !is.na(resid$observed), ]
  return(resid)
}


# ---- Moran's I test for spatial autocorrelation ----
# run Moran's I on full dataset or subsets thereof and return results
#' @param coords matrix of spatial coordinates (lon-lat)
#' @param variable vector of response variable to test for spatial autocorrelation in (e.g. residuals)
#' @param method either "moran.test" or "moran.mc" for two-sided under randomisation or one-sided using monte carlo sims
#' @param subset numeric; number of points to randomly select if running MI on a subset of full data (mainly for function testing)
#' @alternative specify alternative hypothesis if method == "moran.mc", default greater

runMoranTest = function(coords, variable, method = "moran.test", subset = NA, alternative="greater"){
  
  # remove records where is.na(variable)
  coords1 = coords[ !is.na(variable), ]
  variable1 = variable[ !is.na(variable) ]
  
  # return error message if subset > nrow(coords)
  if( !is.na(subset) & subset > nrow(coords) ) { return("Error: subset must be shorter than nrow(coords)") }
  if( !is.na(subset) & !is.numeric(subset) ){ return("Error: subset must be a number") }
  
  # subsample total data (saves time) if !is.na(subset)
  if( !is.na(subset) ){ 
    sub = sample(1:nrow(coords1), subset)
    variable1 = variable1[ sub ]
    coords1 = coords1[ sub, ]
  } 
  
  # calculate inverse distances matrix for coordinates
  dists.mat = distm( as.matrix(coords1), fun=distHaversine )
  dists.inv = 1/dists.mat
  dists.inv[ dists.inv == "Inf"] = 0 # sets 0 for instances where coordinates of two sites are identical
  diag(dists.inv) = 0
  #dists.inv[1:5, 1:5]
  
  # calculate weights matrix for moran's i test
  lw = mat2listw(dists.inv) 
  lww = nb2listw(lw$neighbours, glist=lw$weights, style="W")
  
  # calculate moran's i with significance test
  # either two sided under randomisation, or using monte carlo sims
  if(method == "moran.test"){ return( moran.test(variable1, lww, alternative="two.sided") ) }
  if(method == "moran.mc"){ return( moran.mc(variable1, lww, nsim=99, alternative=alternative) ) }
}


# ---- Run Moran's I tests separately for each study in the model ----
#' @param resid dataframe of residuals; produced using getResidINLA function
#' @param model_name char; specify name of model to run tests on (specified as model_name in getResidINLA)
 
runMoranStudy = function(resid, model_name){
  
  # dataframe and error
  moran_results = data.frame()
  e = simpleError("test error")
  r = resid[ resid$model == model_name, ]
  
  # run for specified model
  for(i in 1:length(unique(r$SS))){
    
    # subset to only points within a study and run autocorrelation test
    d = r[ r$SS == unique(r$SS)[i], ]
    m = tryCatch( runMoranTest(coords=cbind(d$Longitude, d$Latitude), variable=d$resid, method="moran.mc"), error=function(e) e )
    
    # return test estimates or error (for moran.test)
    if( class(m)[1] == "simpleError" ){ mdf = data.frame(SS = d$SS[1], Nsites = nrow(d), Estimate = NA, p.val = NA) 
    } else{ 
      mdf = data.frame(SS = d$SS[1],
                       Nsites = nrow(d), 
                       Estimate = m$statistic,
                       p.val = m$p.value)
    }
    # combine
    moran_results = rbind(moran_results, mdf)
    print(i)
  }
  moran_results$model = model_name
  return(moran_results)
}

# ---- Run global Moran's I test across all sites within a study ----
#' @param resid dataframe of residuals; produced using getResidINLA function
#' @param model_name char; specify name of model to run tests on (specified as model_name in getResidINLA)
#' @param subset numeric; specify subset of points to run tests on if needed (see runMoranTest function)

globalMoran = function(resid, model_name, subset){
  r = resid[ resid$model == model_name, ]
  moran = runMoranTest(coords=cbind(r$Longitude, r$Latitude), variable=r$resid, method="moran.mc", subset=NA)
  res = data.frame(Model = model_name,
                   Nsites = nrow(r),
                   Estimate = moran$statistic,
                   p.val = moran$p.value)
  return(res)
}



# -------- 3. FUNCTIONS FOR PLOTTING AND DIAGNOSTICS OF INLA MODELS ---------


# ---- Plot INLA diagnostics to visualise ----
#' @param mod fitted INLA model with cpo and waic calculated

plotCPOdiag = function(mod){
  
  # extract diagnostics
  cpo = mod$cpo$cpo[ !is.na(mod$cpo$cpo) ]
  pit = mod$cpo$pit[ !is.na(mod$cpo$pit) ]
  n_obs = length( cpo )
  n_fails = length( which(mod$cpo$failure > 0) )
  
  # plot
  par(mfrow=c(2,2), oma=c(0,0,2,0))
  hist( cpo, 20, main="cpo histogram" )
  plot(1:n_obs, cpo, ylab="CPO", pch=21, main= "cpo points per obs", xlab="obs")
  hist( pit, 20, main="pit histogram" )
  uniquant = (1:n_obs)/(n_obs+1)
  plot(uniquant, sort(pit), xlab="uniform quantiles", ylab="Sorted PIT values", ylim=c(0, 1), main="PIT quantiles")
  abline(0,1)
}



# ---- Extract INLA fixed effects ----
#' @param mod fitted INLA model with cpo and waic calculated
#' @param modelname string; name of model to label returned df

getFixed = function(mod, modelname = NA){
  
  fixed = as.data.frame(mod$summary.fixed)
  fixed$param = rownames(fixed)
  fixed = reshape:::rename(fixed, c("0.025quant"="quant025", "0.5quant" = "quant05", "0.975quant"="quant975"))
  if( !is.na(modelname) ){ fixed$model = modelname }
  return(fixed)
  
}



# ---- Plot fixed effects to visualise ----
#' @param fixed df of fixed effects extracted using getFixed; can be rbind from multiple models
#' @param col points colour
#' @param hline line at y intercept
#' @param multimodel logical; does plot include fixed effects from multiple models?

plotFixed = function(fixed, col="darkred", hline=0, multimodel=F, header="", xlab="", ylab=""){
  
  if(multimodel == T){
    # for multiple models
    print( ggplot(fixed, aes(param, quant05, col=model)) + 
             geom_point(cex=2.5, position=position_dodge(width=0.3)) + geom_hline(yintercept=hline, lty=2) +
             geom_linerange(aes(ymin=quant025, ymax=quant975), lwd=0.7, position=position_dodge(width=0.3)) + theme_bw() +
             scale_color_manual(values=c("host" = "darkred", "nonhost"= "darkblue")) + ggtitle(header) + xlab(xlab) + ylab(ylab) ) 
    
    
  } else{
    # for a single model
    print( ggplot( fixed, aes(x=param, y=quant05) ) + geom_point(col=col, cex=2.5) + geom_hline(yintercept=hline, lty=2) +
             geom_linerange(aes(ymin=quant025, ymax=quant975), col=col, lwd=0.7) + theme_bw() +
             theme(axis.text.x=element_text(angle=90,hjust=1)) + ggtitle(header) + xlab(xlab) + ylab(ylab) )
  }
}



# ---- Rescale fixed effects from log scale (exponentiate) ----
#' @param fixed fixed effects
#' @param nat_scale logical; whether to rescale to natural scale with intercept centred at 0
 
rescaleFixed = function(fixed, nat_scale=F){
  
  f1 = fixed
  
  # rescale to proportion scale (i.e. parameters are the ratio of the geometric mean of the intercept (exp(intercept)) to the geometric mean of the factor )
  f1[, c("quant025", "quant05", "quant975")] = exp(f1[, c("quant025", "quant05", "quant975")])
  
  # if specified rescale to natural scale (i.e. calculate differences in geometric means with error bars from intercept)
  # and centre intercept at 0
  if(nat_scale) {
    f1[ 2:nrow(f1), c("quant025", "quant05", "quant975")] = f1[ 2:nrow(f1), c("quant025", "quant05", "quant975")] * f1[ 1, "quant05"]
    f1[ , c("quant025", "quant05", "quant975")] = f1[ , c("quant025", "quant05", "quant975")] - f1[ 1, "quant05"]
    
  } else{
    # otherwise centre intercept around 1 for plotting of proportional change
    f1[ f1$param == "(Intercept)", 3:5] = 1
  }
  
  # rename params
  f1$param[ f1$param == "LU_UIPrimary_Substantial"] = "Primary_Substantial"
  f1$param[ f1$param == "LU_UISecondary_Minimal"] = "Secondary_Minimal"
  f1$param[ f1$param == "LU_UISecondary_Substantial"] = "Secondary_Substantial"
  f1$param[ f1$param == "LU_UIManaged_Minimal"] = "Managed_Minimal"
  f1$param[ f1$param == "LU_UIManaged_Substantial"] = "Managed_Substantial"
  f1$param[ f1$param == "LU_UIUrban_Minimal"] = "Urban_Minimal"
  f1$param[ f1$param == "LU_UIUrban_Substantial"] = "Urban_Substantial"
  
  f1$param[ f1$param == "LandUseTypeSecondary"] = "Secondary"
  f1$param[ f1$param == "LandUseTypeManaged"] = "Managed"
  f1$param[ f1$param == "LandUseTypeUrban"] = "Urban"
  
  return(f1)
}


# ---- Rescale fixed effects from log scale (exponentiate) 2: does not set intercept to 1 ----
#' @param fixed fixed effects
#' @param nat_scale logical; whether to rescale to natural scale with intercept centred at 0

rescaleFixed2 = function(fixed, nat_scale=F){
  
  f1 = fixed
  
  # rescale to proportion scale (i.e. parameters are the ratio of the geometric mean of the intercept (exp(intercept)) to the geometric mean of the factor )
  f1[, c("quant025", "quant05", "quant975")] = exp(f1[, c("quant025", "quant05", "quant975")])
  
  # if specified rescale to natural scale (i.e. calculate differences in geometric means with error bars from intercept)
  # and centre intercept at 0
  if(nat_scale) {
    f1[ 2:nrow(f1), c("quant025", "quant05", "quant975")] = f1[ 2:nrow(f1), c("quant025", "quant05", "quant975")] * f1[ 1, "quant05"]
    f1[ , c("quant025", "quant05", "quant975")] = f1[ , c("quant025", "quant05", "quant975")] - f1[ 1, "quant05"]
    
  } else{
    # otherwise centre intercept around 1 for plotting of proportional change
    #f1[ f1$param == "(Intercept)", 3:5] = 1
  }
  
  # rename params
  f1$param[ f1$param == "LU_UIPrimary_Substantial"] = "Primary_Substantial"
  f1$param[ f1$param == "LU_UISecondary_Minimal"] = "Secondary_Minimal"
  f1$param[ f1$param == "LU_UISecondary_Substantial"] = "Secondary_Substantial"
  f1$param[ f1$param == "LU_UIManaged_Minimal"] = "Managed_Minimal"
  f1$param[ f1$param == "LU_UIManaged_Substantial"] = "Managed_Substantial"
  f1$param[ f1$param == "LU_UIUrban_Minimal"] = "Urban_Minimal"
  f1$param[ f1$param == "LU_UIUrban_Substantial"] = "Urban_Substantial"
  
  f1$param[ f1$param == "LandUseTypeSecondary"] = "Secondary"
  f1$param[ f1$param == "LandUseTypeManaged"] = "Managed"
  f1$param[ f1$param == "LandUseTypeUrban"] = "Urban"
  
  return(f1)
}




# ---- Plot posterior marginal distribution of fixed effects as ridgeline plot ----
#' @param mod fitted INLA model

plotPosteriorFixed = function(mod){
  
  # dependencies
  require(ggridges)
  
  # extract posterior fixed effects
  posteriors = data.frame()
  for(i in 1:length(mod$marginals.fixed)){
    dat = as.data.frame(mod$marginals.fixed[[i]])
    dat$param = names(mod$marginals.fixed)[i]
    posteriors = rbind(posteriors, dat)
  }
  
  # rename and order params
  if( "LU_UIPrimary_Substantial" %in% posteriors$param ){
    posteriors$param[ posteriors$param == "(Intercept)"] = "Intercept"
    posteriors$param[ posteriors$param == "LU_UIPrimary_Substantial"] = "Primary_Substantial"
    posteriors$param[ posteriors$param == "LU_UISecondary_Minimal"] = "Secondary_Minimal"
    posteriors$param[ posteriors$param == "LU_UISecondary_Substantial"] = "Secondary_Substantial"
    posteriors$param[ posteriors$param == "LU_UIManaged_Minimal"] = "Managed_Minimal"
    posteriors$param[ posteriors$param == "LU_UIManaged_Substantial"] = "Managed_Substantial"
    posteriors$param[ posteriors$param == "LU_UIUrban_Minimal"] = "Urban_Minimal"
    posteriors$param[ posteriors$param == "LU_UIUrban_Substantial"] = "Urban_Substantial"
    posteriors$param = factor(posteriors$param, levels=rev(c("Intercept", "Primary_Substantial",
                                                             "Secondary_Minimal", "Secondary_Substantial", 
                                                             "Managed_Minimal", "Managed_Substantial",
                                                             "Urban_Minimal", "Urban_Substantial",
                                                             "LogMedianPubs", "LogHumanDensity")), ordered=T)
  } else{
    posteriors$param[ posteriors$param == "(Intercept)"] = "Intercept"
    posteriors$param[ posteriors$param == "LandUseTypeSecondary"] = "Secondary"
    posteriors$param[ posteriors$param == "LandUseTypeManaged"] = "Managed"
    posteriors$param[ posteriors$param == "LandUseTypeUrban"] = "Urban"
    posteriors$param = factor(posteriors$param, levels=rev(c("Intercept", "Secondary", "Managed", "Urban",
                                                             "LogMedianPubs", "LogHumanDensity")), ordered=T)
  }
  
  # plot
  p = ggplot(posteriors, aes(x, param, height = y, group = param)) + 
    geom_density_ridges2(stat = "identity", scale = 1.3, rel_min_height = 0.000001, fill="skyblue4", col="black") +
    #geom_vline(xintercept=0, lty=2, lwd=0.5) + 
    xlab(expression(beta)) + 
    ylab("parameter")
  
  return(p)
}


# ---- Plot CPO diagnostics alone ----
#' @param mod fitted INLA model with cpo and waic calculated
#' @param title plot name

plotCPO = function(mod, title){
  cpo = mod$cpo$cpo[ !is.na(mod$cpo$cpo) ]
  par(mfrow=c(1, 2))
  hist( cpo, 60, xlab="CPO", main=title, ylab="Frequency")
  plot(1:length(cpo), cpo, ylab="CPO", pch=19, main="", cex=0.5, col="black", xlab="observation")
}



### ---- Plot posterior marginal distribution of hyperparameters as ridgeline plot ----
#' @param mod fitted INLA model with cpo and waic calculated
#' @param axis.range specify xlim range as c(x, y)
#' @param scale param for ggridges
#' @param rel_min_height param for ggridges

plotPosteriorHyper = function(mod, axis.range=NA, scale=1.1, rel_min_height = 0.000001){
  
  require(ggridges)
  
  # extract posterior fixed effects
  posteriors = data.frame()
  for(i in 1:length(mod$marginals.hyperpar)){
    dat = as.data.frame(mod$marginals.hyperpar[[i]])
    dat$param = names(mod$marginals.hyperpar)[i]
    posteriors = rbind(posteriors, dat)
  }
  
  posteriors$param[ posteriors$param == "Precision for the Gaussian observations"] = "Precision Gaussian obs"
  posteriors$param[ posteriors$param == "Precision for SS"] = "Precision Study"
  posteriors$param[ posteriors$param == "Precision for SSS"] = "Precision Site"
  posteriors$param[ posteriors$param == "Precision for TotalSR"] = "Precision TotalSR"
  posteriors$param[ posteriors$param == "Precision for TotalAbundance"] = "Precision TotalAbund"
  
  if(!is.na(axis.range[1])){
    p = ggplot(posteriors, aes(x, param, height = y, group = param)) + 
      geom_density_ridges2(stat = "identity", scale = scale, rel_min_height = rel_min_height, fill="skyblue4", col="black") +
      #geom_vline(xintercept=0, lty=2, lwd=0.5) + 
      xlab(expression(theta)) + 
      ylab("hyperparameter") + xlim(axis.range[1], axis.range[2])
  } else {
  
  # plot
  p = ggplot(posteriors, aes(x, param, height = y, group = param)) + 
    geom_density_ridges2(stat = "identity", scale = scale, rel_min_height = rel_min_height, fill="skyblue4", col="black") +
    #geom_vline(xintercept=0, lty=2, lwd=0.5) + 
    xlab(expression(theta)) + 
    ylab("hyperparameter")
  }
  
  return(p)
}


### get posterior distributions from model
getPosteriorFixed = function(mod, modelname=NA){
  
  # extract
  posteriors = data.frame()
  for(i in 1:length(mod$marginals.fixed)){
    dat = as.data.frame(mod$marginals.fixed[[i]])
    dat$param = names(mod$marginals.fixed)[i]
    posteriors = rbind(posteriors, dat)
  }
  
  # set model name
  if( !is.na(modelname) ) posteriors$model = modelname
  
  return(posteriors)
}

### get marginal quantiles from each model
getMarginalQuantiles = function(mod, modelname=NA){
  
  # extract
  quantiles = data.frame()
  for(i in 1:length(mod$marginals.fixed)){
    
    dat = mod$marginals.fixed[[i]]
    # quantile = data.frame(quantile=c(0.025, 0.25, 0.5, 0.75, 0.095), 
    #                       val=inla.qmarginal(c(0.025, 0.25, 0.5, 0.75, 0.095), dat),
    #                       param = names(mod$marginals.fixed)[i])
    quantile = as.data.frame(t(inla.qmarginal(c(0.025, 0.25, 0.5, 0.75, 0.975), dat)))
    quantile[ , 1:5] = as.numeric( as.vector(quantile))
    quantile$param = names(mod$marginals.fixed)[i]
    quantiles = rbind(quantiles, quantile)
    
  }
  
  # names
  names(quantiles)[1:5] = c("min", "low", "mid", "top", "max")
  
  # set model name
  if( !is.na(modelname) ){ quantiles$model = modelname }
  
  return(quantiles)
}


### rename parameters
editParamNames = function(posteriors){
  
  # create group
  posteriors$Group = NA
  
  # for cargo-carrier dataset 
  if( "LU_UIPrimary_Substantial" %in% posteriors$param ){
    posteriors$param[ posteriors$param == "(Intercept)"] = "Intercept"
    posteriors$param[ posteriors$param == "LU_UIPrimary_Substantial"] = "Primary_Substantial"
    posteriors$param[ posteriors$param == "LU_UISecondary_Minimal"] = "Secondary_Minimal"
    posteriors$param[ posteriors$param == "LU_UISecondary_Substantial"] = "Secondary_Substantial"
    posteriors$param[ posteriors$param == "LU_UIManaged_Minimal"] = "Managed_Minimal"
    posteriors$param[ posteriors$param == "LU_UIManaged_Substantial"] = "Managed_Substantial"
    posteriors$param[ posteriors$param == "LU_UIUrban_Minimal"] = "Urban_Minimal"
    posteriors$param[ posteriors$param == "LU_UIUrban_Substantial"] = "Urban_Substantial"
    posteriors$param = factor(posteriors$param, levels=rev(c("Intercept", "Primary_Substantial",
                                                             "Secondary_Minimal", "Secondary_Substantial", 
                                                             "Managed_Minimal", "Managed_Substantial",
                                                             "Urban_Minimal", "Urban_Substantial",
                                                             "LogMedianPubs", "LogHumanDensity", 
                                                             "log(TotalSR + 1)", "log(TotalAbundance + 1)")), ordered=T)
  } else{
    
    # for mammals only models
    posteriors$param[ posteriors$param == "(Intercept)"] = "Intercept"
    posteriors$param[ posteriors$param == "LandUseTypeSecondary"] = "Secondary"
    posteriors$param[ posteriors$param == "LandUseTypeManaged"] = "Managed"
    posteriors$param[ posteriors$param == "LandUseTypeUrban"] = "Urban"
    posteriors$param = factor(posteriors$param, levels=rev(c("Intercept", "Secondary", "Managed", "Urban",
                                                             "LogMedianPubs", "LogHumanDensity",
                                                             "log(TotalSR + 1)", "log(TotalAbundance + 1)")), ordered=T)
  }
  
  # set groupings
  posteriors$Group[ posteriors$param %in% c("Intercept", "Primary_Substantial", "Secondary_Minimal", "Secondary_Substantial", "Managed_Minimal", "Managed_Substantial", 
                                            "Urban_Minimal", "Urban_Substantial", "Secondary", "Managed", "Urban") ] = "LandUse"
  posteriors$Group[ posteriors$param == "LogMedianPubs" ] = "Publications"
  posteriors$Group[ posteriors$param == "LogHumanDensity" ] = "HumanDensity"
  posteriors$Group[ posteriors$param %in% c("log(TotalSR + 1)", "log(TotalAbundance + 1)") ] = "TotalDiversityMetric"
  
  return(posteriors)
}

# wrapper for full marginal distributions
getMarginalDists = function(mod, modelname){
  return(editParamNames(getPosteriorFixed(mod, modelname=modelname)))
}

# wrapper for quantiles
getMarginalQuants = function(mod, modelname){
  return(editParamNames(getMarginalQuantiles(mod, modelname=modelname)))
}


# plot three different CPO diagnostic figures: histogram, geographical distribution, boxplots by land use
plotCPO = function(mod, df, plot_type, plot_title){
  
  # extract diagnostics and pair with data
  df$cpo = mod$cpo$cpo
  
  # map
  if(plot_type == "hist"){
    # CPO histogram
    final = ggplot(df, aes(cpo)) + geom_histogram(stat="bin", bins=60, col="black", fill="grey60") + 
      theme_bw() + theme(axis.text=element_text(size=12), axis.title = element_text(size=14), plot.title=element_text(size=20)) +
      xlab("CPO") + ylab("Count") + ggtitle(plot_title)
  }
  
  if(plot_type == "map"){
    
    # CPO histogram
    cpo_hist = ggplot(df, aes(cpo)) + geom_histogram(stat="bin", bins=60, col="black", fill="grey60") + 
      theme_bw() + theme(axis.text=element_text(size=12), axis.title = element_text(size=14)) +
      xlab("CPO") + ylab("Count")
    
    # plot geographical distribution
    cont = readOGR("./data/continents_shp/continent.shp", "continent")
    cont1 = fortify(cont[ cont$CONTINENT != "Antarctica", ])
    cpo_geo = ggplot() + geom_polygon(data=cont1, aes(x=long, y=lat, group=group), fill="white", col="grey25", lwd=0.2) +
      coord_fixed() + geom_point(data=df, aes(Longitude, Latitude, col=cpo), size=2) + ggtitle(plot_title) +
      scale_color_viridis(option="D", name="CPO") + theme(axis.text=element_blank(), 
                                                          plot.title=element_text(size=20),
                                                          axis.title=element_blank(), 
                                                          axis.ticks=element_blank(), 
                                                          panel.background = element_rect(fill="grey90"),
                                                          panel.border=element_rect(fill=NA),
                                                          legend.position=c(0.97, 0.5),
                                                          legend.background = element_rect(fill="white", color="black", size=0.3))
    
    # combine into CPO map
    final = cpo_geo +
      annotation_custom(grob = ggplotGrob(cpo_hist + theme(plot.background = element_blank())),
                        xmin = -198,
                        xmax = -100,
                        ymin = -62,
                        ymax = 10)
  }
  
  if(plot_type == "landuse_intensity"){
    # CPO by land use type
    df$LU_UI[ df$LU_UI == "1_Primary_Minimal" ] = "Primary_Minimal"
    df$LU_UI = factor(df$LU_UI, ordered=TRUE, levels=c("Primary_Minimal", "Primary_Substantial", "Secondary_Minimal", "Secondary_Substantial",
                                                       "Managed_Minimal", "Managed_Substantial", "Urban_Minimal", "Urban_Substantial"))
    final = ggplot(df, aes(LU_UI, cpo, fill=LandUseType)) + geom_boxplot() +
      theme_bw() + theme(axis.text=element_text(size=12), 
                         axis.title = element_text(size=14),
                         axis.text.x=element_text(angle=90),
                         plot.title=element_text(size=20),
                         legend.position = "none") + 
      scale_fill_manual(values=c("1_Primary"="forestgreen",
                                 "Secondary"="darkolivegreen3",
                                 "Managed"="navajowhite",
                                 "Urban"="lightsteelblue")) +
      xlab("Land use") + ylab("CPO per-site") + ggtitle(plot_title)
  }
  
  if(plot_type == "landuse"){
    # CPO by land use type
    df$LandUseType[ df$LandUseType == "1_Primary" ] = "Primary"
    if("Urban" %in% df$LandUseType){ df$LandUseType = factor(df$LandUseType, ordered=TRUE, levels=c("Primary", "Secondary", "Managed", "Urban")) 
    } else{ df$LandUseType = factor(df$LandUseType, ordered=TRUE, levels=c("Primary", "Secondary", "Managed")) }
    final = ggplot(df, aes(LandUseType, cpo, fill=LandUseType)) + geom_boxplot() +
      theme_bw() + theme(axis.text=element_text(size=12), 
                         axis.title = element_text(size=14),
                         axis.text.x=element_text(angle=90),
                         plot.title=element_text(size=20),
                         legend.position = "none") + 
      scale_fill_manual(values=c("Primary"="forestgreen",
                                 "Secondary"="darkolivegreen3",
                                 "Managed"="navajowhite",
                                 "Urban"="lightsteelblue")) +
      xlab("Land use") + ylab("CPO per-site") + ggtitle(plot_title)
  }
  
  return(final)
}



# ---- Logistic function: convert logit to probability scale ----

logistic = function (x) {
  p = 1/(1 + exp(-x))
  p = ifelse(x == Inf, 1, p)
  p
}


# ---------- calculate dataset summaries for order models --------------

# calculate dataset summaries
calcSSS = function(df){ return(data.frame(NumStudies = length(unique(df$SS)),
                                          NumSites = length(unique(df$SSS)),
                                          NumHosts = length(unique(df$bgb[df$IsHost==1])),
                                          NumNonHosts = length(unique(df$bgb[df$IsHost==0])),
                                          NumPrimary = sum(df$LandUseType[ !duplicated(df$SSS) ] == "1_Primary"),
                                          NumSecondary = sum(df$LandUseType[ !duplicated(df$SSS) ] == "Secondary"),
                                          NumManaged = sum(df$LandUseType[ !duplicated(df$SSS) ] == "Managed") ))}

