
### Gibb et al, 'Land use has global and systematic effects on local zoonotic host communities"
# Script 1: Pre-processing of PREDICTS database
# Reads predicts db, corrects diversity metrics for sampling effort, merge close sites (following Newbold et al 2015)
# remove taxa that are not relevant for zoonotic transmission (plants, etc), extract environmental covariates

# -------- package install -------
#install.packages(c("Hmisc, "cat", "taxize", "rgbif", "countrycode"))
#require(c(ape, geiger, wordcloud, Hmisc, raster, rgdal, geosphere))
#install.packages("./data/r_packages/yarg_0.1-14.tar.gz", repos=NULL, type="source")

# dependencies
library(yarg)
library(cat)
library(Hmisc)
library(taxize)
library(rgbif)
library(plyr)
library(ggplot2)
library(rgdal)
library(ggmap)
library(countrycode)

# load functions
source("00_functions_general.R")



# ------ load PREDICTS database

# predicts
dd = readRDS("./data/predicts/predicts_database.Rds")

# correct for sampling effort
# rescales sampling effort within studies (SS) relative to maximum (each value = se/max(se)), then corrects diversity metric for effort
cat('Correcting for sampling effort\n')
dd = CorrectSamplingEffort(dd)

# merge sites that were sampled close together (e.g. transects) (n = 343410 records dropped, leaving 2906994 records)
# public = TRUE, as this is public data
cat('Merging sites\n')
dd = MergeSites(dd, public=TRUE)

# nrow=2906994
# remove taxa (e.g. plants) that are not relevant (n = 1889781 records dropped, retained n = 1017213 records)
dd$Best_guess_binomial = as.vector(dd$Best_guess_binomial)
dd1 = dd[ dd$Best_guess_binomial != "" & dd$Phylum %in% c("Chordata", "Arthropoda", "Gastropoda", "Mollusca"), ]

# standardise species names to facilitate taxonomic matching (creates binomial and genus columns)
dd1$bgb = tolower(dd1$Best_guess_binomial)
dd1$bgg = unlist(lapply(strsplit(dd1$bgb, " "), function(x) return(x[1])))

# code countries as ISO2c 
dd1$Country_ISO2C = countrycode(as.vector(dd1$Country), origin="country.name", destination="iso2c")

# save
#save(dd1, file="./output/predicts/predicts_mergedsites.R")



# --------- examine species richness across studies ----------

# calculate species richness of each study
SR1 = ddply(dd1, .(SS), plyr::summarize, SS_SpeciesRichness=length(unique(as.vector(bgb))), 
            n_records = length(bgb),
            n_sites = length(unique(SSS)),
            genus_div = length(unique(Genus)), 
            order_div = length(unique(Order)))

# label PREDICTS database with total species richness
SR2 = SR1[ , c("SS", "SS_SpeciesRichness")]
names(SR2)[2] = "Study_TotalSR"
dd1 = merge(dd1, SR2, by="SS")




### -------------------- Scrape PubMed for species-level publication effort ----------------- 

# list of species
spp_list = unique(dd$bgb)

# run pubmed scrape on all species (>3000): runs for approx 1hr
source("./00_function_pubmed.R")
pubmed_results = sapply(spp_list, FUN=searchPubMed)
pm_scrape = data.frame(bgb = names(pubmed_results), NumPubs = as.vector(pubmed_results))

# save pubmed scrape
write.csv(pm_scrape, "./output/pubmed/pubmed_PREDICTSall_20180715.csv", row.names=F)
#pm_scrape = read.csv("./output/pubmed/pubmed_PREDICTSall_20180715.csv", stringsAsFactors=F)

# set NAs (for axis axis and indicator indicator: massive hits due to generic name)
pm_scrape$NumPubs = as.numeric(as.vector(pm_scrape$NumPubs))
pm_scrape$NumPubs[ is.na(pm_scrape$NumPubs) ] = NA
pm_scrape$NumPubs[ pm_scrape$bgb %in% c("axis axis", "indicator indicator") ] = NA

# combine with PREDICTS
dd1$id1 = 1:nrow(dd1)
dd1 = merge(dd1, pm_scrape, by="bgb", all.x=T)
dd1 = dd1[ order(dd1$id1), ]




### --------- Remove all records with no geographical coordinates ---------

# remove records with no coordinates (n=866)
nrow(dd1[ is.na(dd1$Longitude) | is.na(dd1$Latitude), ])
dd1 = dd1[ !is.na(dd1$Longitude) & !is.na(dd1$Latitude), ]




### --------- Extract covariate data: human population density, local climatic variables ---------

# Human population density
# GPW v4 (gridded population of the world) 2005, persons per square kilometre
gpw = raster("./data/population_GPW_v4/2005_un-adjusted/gpw-v4-population-density-adjusted-to-2015-unwpp-country-totals_2005.tif")
dd1 = extractRasterData(dd1, gpw, "HumanDensity")

# Worldclim mean annual temperature
glob_meantemp = raster("./data/worldclim2.0_climate/mean_globaltemp.tif")
dd1 = extractRasterData(dd1, glob_meantemp, "MeanAnnualTemp")

# Worldclim mean annual precipitation
glob_meanprecip = raster("./data/worldclim2.0_climate/mean_globalprecip.tif")
dd1 = extractRasterData(dd1, glob_meanprecip, "MeanAnnualPrecip")




# --------- save processed PREDICTS database for matching to host datasets -------------

save(dd1, file="./output/predicts/predicts_cleaned.R")
