### Gibb et al, 'Land use has global and systematic effects on local zoonotic host communities"
# Script 3: Combine PREDICTS with host-pathogen databases

# dependencies
library(sp)
library(raster)
library(rgdal)
library(ggplot2)
library(nlme)
library(dismo)
library(ggmap)
library(Hmisc)
library(plyr)
library(dplyr)

# load functions
source("00_functions_general.R")


# ------------ PREDICTS data --------------

# load predicts db
load("./output/predicts_processed/predicts_cleaned.R")
dd = dd1

# label and exclude domestic species records
load(file="./data/domestics_list/domestic_species.R")
domestic = c(domestic, "canis familiaris", "apis mellifera", "bos frontalis")
dd$IsDomestic = ifelse( dd$bgb %in% domestic, 1, 0)
dd = dd[ dd$IsDomestic == 0, ]


# ------------ Full hosts dataset ------------

# host data and create unique identifier
hosts = read.csv("./output/host_parasite_processed/hostdata_alldatasets_harmonised_20180916.csv", stringsAsFactors = F)
hosts = hosts[ hosts$IsDomestic == 0, ]
hosts$id = 1:nrow(hosts)

# function to split synonyms
splitSyns = function(x){
  orig = hosts$HostBinomial[ hosts$id == x ]
  syns = unlist(strsplit(hosts$HostSynonyms[ hosts$id == x ], ", "))
  if(all(is.na(syns))) syns = c()
  df = data.frame(id = x, Host = c(orig, syns), name_type = c("orig", rep("syn", length(syns))))
  df
}
syns = do.call(rbind.data.frame, lapply(hosts$id, splitSyns))

# subset to only species in PREDICTS (n=650 matches, no duplicates)
syns = syns[ syns$Host %in% dd$bgb, ]

# link ID from syns to PREDICTS
dd$id = syns$id[ match(dd$bgb, syns$Host) ]

# subset host data to only hosts with PREDICTS matches and save both all and human-shared only
hosts_preds = hosts[ hosts$id %in% syns$id, ]
write.csv(hosts_preds, "./output/summary_tables/HostSpecies_withPREDICTSmatches.csv", row.names=F)
write.csv(hosts_preds[hosts_preds$HumanInfective_Any == 1, ], "./output/summary_tables/HostSpecies_humaninfective_withPREDICTSmatches.csv", row.names=F)

# subset down to important columns and merge with PREDICTS
hosts_preds2 = hosts_preds[ , -which(names(hosts_preds) %in% c("HostClass", "HostOrder", "HostFamily", "HostSynonyms", "IsDomestic")) ]
dd1 = full_join(dd, hosts_preds2)

# sort columns
dd1$HumanInfective_Any[ is.na(dd1$HumanInfective_Any) ] = 0
dd1$DiseaseAgent_Strict[ is.na(dd1$DiseaseAgent_Strict) ] = 0
dd1$IsZoonotic_Strict[ is.na(dd1$IsZoonotic_Strict) ] = 0
dd1$PathRich_HumanInfective[ is.na(dd1$PathRich_HumanInfective) ] = 0
dd1$PathRich_NonHuman[ is.na(dd1$PathRich_NonHuman) ] = 0
dd1$PathRich_NonHumanDom[ is.na(dd1$PathRich_NonHumanDom) ] = 0
dd1$PathogenDiv[ is.na(dd1$PathogenDiv) ] = ""
dd1$Pathogens_Human[ is.na(dd1$Pathogens_Human) ] = ""
dd1$Pathogens_Zoonotic_Strict[ is.na(dd1$Pathogens_Zoonotic_Strict) ] = ""
dd1$Pathogens_NonHuman[ is.na(dd1$Pathogens_NonHuman) ] = ""
dd1$Sero_HumanInfective_Any[ is.na(dd1$Sero_HumanInfective_Any) ] = 0
dd1$Sero_DiseaseAgent_Strict[ is.na(dd1$Sero_DiseaseAgent_Strict) ] = 0
dd1$Sero_IsZoonotic_Strict[ is.na(dd1$Sero_IsZoonotic_Strict) ] = 0
dd1$Sero_PathRich_HumanInfective[ is.na(dd1$Sero_PathRich_HumanInfective) ] = 0
dd1$Sero_PathRich_NonHuman[ is.na(dd1$Sero_PathRich_NonHuman) ] = 0
dd1$Sero_PathRich_NonHumanDom[ is.na(dd1$Sero_PathRich_NonHumanDom) ] = 0
dd1$Isolation_HumanInfective_Any[ is.na(dd1$Isolation_HumanInfective_Any) ] = 0
dd1$Isolation_DiseaseAgent_Strict[ is.na(dd1$Isolation_DiseaseAgent_Strict) ] = 0
dd1$Isolation_IsZoonotic_Strict[ is.na(dd1$Isolation_IsZoonotic_Strict) ] = 0
dd1$Isolation_PathRich_HumanInfective[ is.na(dd1$Isolation_PathRich_HumanInfective) ] = 0
dd1$Isolation_PathRich_NonHuman[ is.na(dd1$Isolation_PathRich_NonHuman) ] = 0
dd1$Isolation_PathRich_NonHumanDom[ is.na(dd1$Isolation_PathRich_NonHumanDom) ] = 0
dd1$Sources[ is.na(dd1$Sources) ] = ""



### ------------- Label studies with no matches to host-parasite database  ------------

labelStudies = function(x, host_type="HumanInfective_Any", class="all"){
  if(class == "all"){ 
    dat = dd1[ dd1$SS == unique(dd1$SS)[x], which(names(dd1) == host_type) ]
  } else{ dat = dd1[ dd1$SS == unique(dd1$SS)[x] & dd1$Class == class, which(names(dd1) == host_type) ] }
  return(ifelse(sum(dat)>0, 1, 0))
}

# apply function to label studies based on either 'host' or 'strict', and cargocarrier or not
stud = data.frame(SS = as.vector(unique(dd1$SS)))
stud$SS_Cargocarrier = unlist(lapply(1:nrow(stud), labelStudies)) 
stud$SS_Cargocarrier_Mammalia = unlist(lapply(1:nrow(stud), labelStudies, class="Mammalia")) 
stud$SS_Cargocarrier_Aves = unlist(lapply(1:nrow(stud), labelStudies, class="Aves")) 
stud$SS_Zoonotic_Cargocarrier = unlist(lapply(1:nrow(stud), labelStudies, host_type="IsZoonotic_Strict"))
stud$SS_Mammalsero = unlist(lapply(1:nrow(stud), labelStudies, host_type="Sero_HumanInfective_Any"))
stud$SS_Mammalisolation = unlist(lapply(1:nrow(stud), labelStudies, host_type="Isolation_HumanInfective_Any"))

# combine
dd1 = full_join(dd1, stud, by="SS")

# also calculate more broadly: hosts plus any mammals, any birds
dd1$HIAMB = ifelse(dd1$SS_Cargocarrier == 1 | dd1$Class == "Mammalia" | dd1$Class == "Aves", 1, 0)
stud$SS_HIAMB = unlist(lapply(1:nrow(stud), labelStudies, host_type="HIAMB"))
dd1 = dd1[ , -which(names(dd1) == "HIAMB") ]
dd1 = full_join(dd1, stud[ , c("SS", "SS_HIAMB")])




# -------------------- corrections and save dataset --------------------

# correct taxonomy and remove sites with land use classification of 'cannot decide'
dd1$Order = as.vector(dd1$Order)
dd1$Order[ dd1$Order == "Artiodactyla" ] = "Cetartiodactyla"
dd1 = dd1[ dd1$Predominant_land_use != "Cannot decide", ]

# number of studies and sites with host matches: 
# 177 studies, 7893 sites (cargocarrier all)
# 61 studies, 1976 sites(cargocarrier mammals)
# 80 studies, 4616 sites (cargocarrier birds)
# 155 studies, 7098 sites (cargocarrier zoonotic)
# 58 studies, 1930 sites (mammal reservoir)
print(paste( "Num studies:", length(unique(dd1$SS[dd1$SS_Cargocarrier==1])), "Num sites:", length(unique(dd1$SSS[dd1$SS_Cargocarrier==1])) ))
print(paste( "Num studies:", length(unique(dd1$SS[dd1$SS_Cargocarrier_Mammalia==1])), "Num sites:", length(unique(dd1$SSS[dd1$SS_Cargocarrier_Mammalia==1])) ))
print(paste( "Num studies:", length(unique(dd1$SS[dd1$SS_Cargocarrier_Aves==1])), "Num sites:", length(unique(dd1$SSS[dd1$SS_Cargocarrier_Aves==1])) ))
print(paste( "Num studies:", length(unique(dd1$SS[dd1$SS_Zoonotic_Cargocarrier==1])), "Num sites:", length(unique(dd1$SSS[dd1$SS_Zoonotic_Cargocarrier==1])) ))
print(paste( "Num studies:", length(unique(dd1$SS[dd1$SS_Mammalsero==1])), "Num sites:", length(unique(dd1$SSS[dd1$SS_Mammalsero==1])) ))
print(paste( "Num studies:", length(unique(dd1$SS[dd1$SS_Mammalisolation==1])), "Num sites:", length(unique(dd1$SSS[dd1$SS_Mammalisolation==1])) ))
print(paste( "Num studies:", length(unique(dd1$SS[dd1$SS_HIAMB==1])), "Num sites:", length(unique(dd1$SSS[dd1$SS_HIAMB==1])) ))

# save
save(dd1, file="./output/predicts_processed/predicts_pathogens.R")

