
### Gibb et al, 'Land use has global and systematic effects on local zoonotic host communities"
# Script 7: Model selection on community (richness and abundance) models for Tier 2 (mammal zoonotic reservoirs)
# Compares models using WAIC and selects best random/fixed effects structure (prior to effort bootstrap)

# dependencies
library(lme4)
library(lmerTest)
library(ggplot2)
library(nlme)
library(plyr)
library(dismo)
library(geosphere)
library(spdep)
library(rgdal)
library(INLA)
library(INLAutils)
library(viridis)
library(raster)
library(RColorBrewer)
library(KernSmooth)
library(dplyr)

# functions
source("00_functions_general.R")

# colours for plotting
nonhostcol = "#018571"
hostcol = "#8c510a"




# ------------ prepare and summarise data --------

# load PREDICTS data
load(file="./data/predicts_processed/predicts_pathogens_researcheffort.R")

# keep only mammal studies (n=66)
xx = ddply(dd1, .(SS), plyr::summarise, include_mammalia=ifelse(any(Class == "Mammalia"), 1, 0))
dd1 = dd1[ dd1$SS %in% xx$SS[ xx$include_mammalia == 1], ]
#dd1 = dd1[ dd1$Study_TotalSR > 3, ]

# prepare land use categories
dd1 = prepareLandUse(dd1)

# tier 2 strictness: mammals pathogen isolation/direct detection
# and set any mammal non-hosts to 0
dd1$HumanInfective_Any[ dd1$Class == "Mammalia" ] = dd1$Isolation_HumanInfective_Any[ dd1$Class == "Mammalia" ]
dd1$IsZoonotic_Strict[ dd1$Class == "Mammalia" ] = dd1$Isolation_IsZoonotic_Strict[ dd1$Class == "Mammalia" ]
dd1$HumanInfective_Any[ dd1$Class != "Mammalia" ] = 0
dd1$IsZoonotic_Strict[ dd1$Class != "Mammalia" ] = 0

# details: 147 mammal hosts, 700 non hosts, 2066 sites, 66 studies 
print(paste("Hosts:", length(unique(dd1$bgb[ which(dd1$Class == "Mammalia" & dd1$HumanInfective_Any == 1)])), "mammals, ", 
            length(unique(dd1$bgb[ which(dd1$Class == "Aves" & dd1$HumanInfective_Any == 1)])), "birds, ", 
            length(unique(dd1$bgb[ which(dd1$Class == "Insecta" & dd1$HumanInfective_Any == 1)])), "insects", sep=" "))
printDetails(dd1)

# summarise
dd1_site = aggregateSite(dd1)
#write.csv(dd1_site, "./output/predicts_sites_formodelling/Tier2_mammals_predicts_sites_SR0_20190514.csv", row.names=F)
#dd1_site = read.csv("./output/predicts_sites_formodelling/Tier2_mammals_predicts_sites_SR0_20190514.csv", stringsAsFactors=F)

# exclude sites with use intensity of 'cannot decide' (n=40)
# leaving 2026 sites from 63 studies
sum(dd1_site$UseIntensity == "Cannot decide")
models_df = dd1_site[ dd1_site$UseIntensity != "Cannot decide", ]

# combine all managed and all urban sites together: improve evenness of sampling
table(models_df$LU_UI)
models_df$LU_UI[ grep("Managed", models_df$LU_UI) ] = "Managed"
models_df$LU_UI[ grep("Urban", models_df$LU_UI) ] = "Urban"

# calculate log total richness/abundance
models_df$logTotalSR = log(models_df$TotalSR+1)
models_df$logTotalAbund = log(models_df$TotalAbundance+1)




# ---------- priors -----------

# precision prior for iid effect (log-gamma, 0.1/0.1)
hyper1.iid = list(prec = list(prior='loggamma', param=c(0.1,0.1)))

# uninformative priors for mu and theta on fixed effects (mean 0, sd 100)
control.fixed1 = list(mean.intercept=0, mean = 0,
                      prec = 0.01, prec.intercept = 0.01)

# prior precision for the gaussian obs (loggamma, 0.1/0.1)
control.family.gauss = list(hyper=list(prec=list(prior='loggamma', param=c(0.1, 0.1))))




# ---------- (1) Model selection: host richness and abundance models ----------


# (a) selection for random effects structure (compare using WAIC and sum-CPO)

# land use effects type (intensityfixed or intensityrandom)
type = "intensityfixed"


# model components
response_fixed_hostsr = "HostSR ~ 1 + LU_UI + LogHumanDensity + f(SS, model='iid', hyper=hyper1.iid)"
response_fixed_hostabund = "log(HostAbundance + 1) ~ 1 + LU_UI + LogHumanDensity + f(SS, model='iid', hyper=hyper1.iid)"
geog = "+ f(Biome, model='iid', hyper=hyper1.iid)"
ssb = "+ f(SSB, model='iid', hyper=hyper1.iid)"
sss = "+ f(SSBS, model='iid', hyper=hyper1.iid)"

# combine into formulae
formulae = list(c(response_fixed_hostsr, "HostSR"),
                c(paste(response_fixed_hostsr, ssb, sss, geog), "HostSR"),
                c(paste(response_fixed_hostsr, ssb, sss), "HostSR"),
                c(paste(response_fixed_hostsr, ssb, geog), "HostSR"),
                c(paste(response_fixed_hostsr, ssb), "HostSR"),
                c(response_fixed_hostabund, "HostAbundance"),
                c(paste(response_fixed_hostabund, ssb, sss, geog), "HostAbundance"),
                c(paste(response_fixed_hostabund, ssb, sss), "HostAbundance"),
                c(paste(response_fixed_hostabund, ssb, geog), "HostAbundance"),
                c(paste(response_fixed_hostabund, ssb), "HostAbundance"))

# fit models
for(x in 1:length(formulae)){
  
  # model id
  modid = sample(1:100000, 1)
  form = formula(formulae[[x]][1])
  resp = formulae[[x]][2]
  
  # hostSR model
  if(resp == "HostSR"){
    host_mod = inla(form, data=models_df,
                    family="poisson", E=1,
                    control.fixed = control.fixed1,
                    control.predictor=list(compute=TRUE, link=1), # link scale pred
                    control.compute=list(cpo=TRUE, waic=TRUE, config=TRUE))
  }
  
  # abundance model
  if(resp == "HostAbundance"){
    host_mod =  inla(form, data=models_df,
                     family="gaussian", 
                     control.predictor=list(compute=T),
                     control.compute=list(cpo=T, waic=T),
                     control.fixed=control.fixed1,
                     control.family=control.family.gauss)
  }
  
  # create diagnostic dataframe
  dat = data.frame(modid = modid,
                   response = formulae[[x]][2],
                   formula = formulae[[x]][1],
                   waic = host_mod$waic$waic,
                   n_eff_rep = as.numeric(host_mod$neffp[3, 1]), # number of effective replicates
                   n_obs = length(host_mod$cpo$cpo), # number of datapoints used to fit model
                   n_cpo_fails = length( which(host_mod$cpo$failure > 0) ),
                   sum_cpo = sum(host_mod$cpo$cpo, na.rm=T), # if cpo = 1 model predicts point perfectly; so perfect fit would be n_obs
                   meanlogcpo = -mean( log( host_mod$cpo$cpo ), na.rm=T) # if every point predicted perfectly, -mean(log(cpo)) == 0, so larger vals indicate greater deviance
  )
  
  if(x == 1){ dat1 = dat } else{ dat1 = rbind(dat1, dat)}
  print(x)
}

# save diagnostics
write.csv(dat1, file=paste("./output/model_outputs/community_tier2_mammals/model_selection/host_", type, "_rsel_diagnostics.csv", sep=""), row.names=F)





# (b) fixed effects selection using best random effects structure ----------

# land use effects type (intensityfixed or intensityrandom)
type = "intensityfixed"

response_hostsr = "HostSR ~ 1"
response_hostabund = "log(HostAbundance + 1) ~ 1"
lu = "+ LU_UI"
hpd = "+ LogHumanDensity"
rand_hostsr = "+ f(SS, model='iid', hyper=hyper1.iid) + f(SSB, model='iid', hyper=hyper1.iid)"
rand_hostabund = "+ f(SS, model='iid', hyper=hyper1.iid) + f(SSB, model='iid', hyper=hyper1.iid) + f(SSBS, model='iid', hyper=hyper1.iid) + f(Biome, model='iid', hyper=hyper1.iid)"

# formulae (with best fitting random effects structure)
formulae1 = list( c(paste(response_hostsr, lu, rand_hostsr), "HostSR"),
                  c(paste(response_hostsr, lu, hpd, rand_hostsr), "HostSR"),
                  c(paste(response_hostabund, lu, rand_hostabund), "HostAbundance"),
                  c(paste(response_hostabund, lu, hpd, rand_hostabund), "HostAbundance") )

# fit models
for(x in 1:length(formulae1)){
  
  # model id
  modid = sample(1:100000, 1)
  form = formula(formulae1[[x]][1])
  resp = formulae1[[x]][2]
  
  # hostSR model
  if(resp == "HostSR"){
    host_mod = inla(form, data=models_df,
                    family="poisson", E=1,
                    control.fixed = control.fixed1,
                    control.predictor=list(compute=TRUE, link=1), # link scale pred
                    control.compute=list(cpo=TRUE, waic=TRUE, config=TRUE))
  }
  
  # abundance model
  if(resp == "HostAbundance"){
    host_mod =  inla(form, data=models_df,
                     family="gaussian", 
                     control.predictor=list(compute=T),
                     control.compute=list(cpo=T, waic=T),
                     control.fixed=control.fixed1,
                     control.family=control.family.gauss)
  }
  
  # create diagnostic dataframe
  dat = data.frame(modid = modid,
                   response = formulae1[[x]][2],
                   formula = formulae1[[x]][1],
                   waic = host_mod$waic$waic,
                   n_eff_rep = as.numeric(host_mod$neffp[3, 1]), # number of effective replicates
                   n_obs = length(host_mod$cpo$cpo), # number of datapoints used to fit model
                   n_cpo_fails = length( which(host_mod$cpo$failure > 0) ),
                   sum_cpo = sum(host_mod$cpo$cpo, na.rm=T), # if cpo = 1 model predicts point perfectly; so perfect fit would be n_obs
                   meanlogcpo = -mean( log( host_mod$cpo$cpo ), na.rm=T) # if every point predicted perfectly, -mean(log(cpo)) == 0, so larger vals indicate greater deviance
  )
  
  if(x == 1){ dat1 = dat } else{ dat1 = rbind(dat1, dat)}
  print(x)
}

# save diagnostics
write.csv(dat1, file=paste("./output/model_outputs/community_tier2_mammals/model_selection/host_", type, "_fsel_diagnostics.csv", sep=""), row.names=F)







# ---------- 2) Model selection: non-host richness and abundance models ----------

# (a) selection for random effects structure (compare using WAIC and sum-CPO)

# land use effects type (intensityfixed or intensityrandom)
type = "intensityfixed"

# model components
response_fixed_hostsr = "NonHostSR ~ 1 + LU_UI + LogHumanDensity + f(SS, model='iid', hyper=hyper1.iid)"
response_fixed_hostabund = "log(NonHostAbundance + 1) ~ 1 + LU_UI + LogHumanDensity + f(SS, model='iid', hyper=hyper1.iid)"
geog = "+ f(Biome, model='iid', hyper=list(prec = list(prior='loggamma',param=c(0.1,0.1))))"
ssb = "+ f(SSB, model='iid', hyper=list(prec = list(prior='loggamma',param=c(0.1,0.1))))"
sss = "+ f(SSBS, model='iid', hyper=list(prec = list(prior='loggamma',param=c(0.1,0.1))))"

# combine into formulae
formulae = list(c(response_fixed_hostsr, "NonHostSR"),
                c(paste(response_fixed_hostsr, ssb, sss, geog), "NonHostSR"),
                c(paste(response_fixed_hostsr, ssb, sss), "NonHostSR"),
                c(paste(response_fixed_hostsr, ssb, geog), "NonHostSR"),
                c(paste(response_fixed_hostsr, ssb), "NonHostSR"),
                c(response_fixed_hostabund, "NonHostAbundance"),
                c(paste(response_fixed_hostabund, ssb, sss, geog), "NonHostAbundance"),
                c(paste(response_fixed_hostabund, ssb, sss), "NonHostAbundance"),
                c(paste(response_fixed_hostabund, ssb, geog), "NonHostAbundance"),
                c(paste(response_fixed_hostabund, ssb), "NonHostAbundance"))

# fit models
for(x in 1:length(formulae)){
  
  # model id
  modid = sample(1:100000, 1)
  form = formula(formulae[[x]][1])
  resp = formulae[[x]][2]
  
  # nonhostSR model
  if(resp == "NonHostSR"){
    host_mod = inla(form, data=models_df,
                    family="poisson", E=1,
                    control.fixed = control.fixed1,
                    control.predictor=list(compute=TRUE, link=1), # link scale pred
                    control.compute=list(cpo=TRUE, waic=TRUE, config=TRUE))
  }
  
  # nonhost abundance model
  if(resp == "NonHostAbundance"){
    host_mod =  inla(form, data=models_df,
                     family="gaussian", 
                     control.predictor=list(compute=T),
                     control.compute=list(cpo=T, waic=T),
                     control.fixed=control.fixed1,
                     control.family=control.family.gauss)
  }
  
  # create diagnostic dataframe
  dat = data.frame(modid = modid,
                   response = formulae[[x]][2],
                   formula = formulae[[x]][1],
                   waic = host_mod$waic$waic,
                   n_eff_rep = as.numeric(host_mod$neffp[3, 1]), # number of effective replicates
                   n_obs = length(host_mod$cpo$cpo), # number of datapoints used to fit model
                   n_cpo_fails = length( which(host_mod$cpo$failure > 0) ),
                   sum_cpo = sum(host_mod$cpo$cpo, na.rm=T), # if cpo = 1 model predicts point perfectly; so perfect fit would be n_obs
                   meanlogcpo = -mean( log( host_mod$cpo$cpo ), na.rm=T) # if every point predicted perfectly, -mean(log(cpo)) == 0, so larger vals indicate greater deviance
  )
  
  if(x == 1){ dat1 = dat } else{ dat1 = rbind(dat1, dat)}
  print(x)
}

# save diagnostics
write.csv(dat1, file=paste("./output/model_outputs/community_tier2_mammals/model_selection/nonhost_", type, "_rsel_diagnostics.csv", sep=""), row.names=F)






# (b) fixed effects selection using best random effects structure ----------

response_hostsr = "NonHostSR ~ 1"
response_hostabund = "log(NonHostAbundance + 1) ~ 1"
lu = "+ LU_UI"
hpd = "+ LogHumanDensity"
rand_hostsr = "+ f(SS, model='iid', hyper=hyper1.iid) + f(SSB, model='iid', hyper=hyper1.iid) + f(Biome, model='iid', hyper=hyper1.iid)"
rand_hostabund = "+ f(SS, model='iid', hyper=hyper1.iid) + f(SSB, model='iid', hyper=hyper1.iid) + f(SSBS, model='iid', hyper=hyper1.iid)"

# formulae (with best fitting random effects structure)
formulae1 = list( c(paste(response_hostsr, lu, rand_hostsr), "NonHostSR"),
                  c(paste(response_hostsr, lu, hpd, rand_hostsr), "NonHostSR"),
                  c(paste(response_hostabund, lu, rand_hostabund), "NonHostAbundance"),
                  c(paste(response_hostabund, lu, hpd, rand_hostabund), "NonHostAbundance") )

# fit models
for(x in 1:length(formulae1)){
  
  # model id
  modid = sample(1:100000, 1)
  form = formula(formulae1[[x]][1])
  resp = formulae1[[x]][2]
  
  # nonhostSR model
  if(resp == "NonHostSR"){
    host_mod = inla(form, data=models_df,
                    family="poisson", E=1,
                    control.fixed = control.fixed1,
                    control.predictor=list(compute=TRUE, link=1), # link scale pred
                    control.compute=list(cpo=TRUE, waic=TRUE, config=TRUE))
  }
  
  # nonhost abundance model
  if(resp == "NonHostAbundance"){
    host_mod =  inla(form, data=models_df,
                     family="gaussian", 
                     control.predictor=list(compute=T),
                     control.compute=list(cpo=T, waic=T),
                     control.fixed=control.fixed1,
                     control.family=control.family.gauss)
  }
  
  # create diagnostic dataframe
  dat = data.frame(modid = modid,
                   response = formulae1[[x]][2],
                   formula = formulae1[[x]][1],
                   waic = host_mod$waic$waic,
                   n_eff_rep = as.numeric(host_mod$neffp[3, 1]), # number of effective replicates
                   n_obs = length(host_mod$cpo$cpo), # number of datapoints used to fit model
                   n_cpo_fails = length( which(host_mod$cpo$failure > 0) ),
                   sum_cpo = sum(host_mod$cpo$cpo, na.rm=T), # if cpo = 1 model predicts point perfectly; so perfect fit would be n_obs
                   meanlogcpo = -mean( log( host_mod$cpo$cpo ), na.rm=T) # if every point predicted perfectly, -mean(log(cpo)) == 0, so larger vals indicate greater deviance
  )
  
  if(x == 1){ dat1 = dat } else{ dat1 = rbind(dat1, dat)}
  print(x)
}

# save diagnostics
write.csv(dat1, file=paste("./output/model_outputs/community_tier2_mammals/model_selection/nonhost_", type, "_fsel_diagnostics.csv", sep=""), row.names=F)





# ---------- (3) Model selection: proportion host richness and abundance models ----------

# (a) selection for random effects structure (compare using WAIC and sum-CPO)

# land use effects type (intensityfixed or intensityrandom)
type = "intensityfixed"

# model components
response_fixed_hostsr = "HostSR ~ 1 + LU_UI + LogHumanDensity + logTotalSR + f(SS, model='iid', hyper=hyper1.iid)"
response_fixed_hostabund = "log(HostAbundance + 1) ~ 1 + LU_UI + LogHumanDensity + logTotalAbund + f(SS, model='iid', hyper=hyper1.iid)"
geog = "+ f(Biome, model='iid', hyper=hyper1.iid)"
ssb = "+ f(SSB, model='iid', hyper=hyper1.iid)"
sss = "+ f(SSBS, model='iid', hyper=hyper1.iid)"

# combine into formulae
formulae = list(c(response_fixed_hostsr, "PropHostSR"),
                c(paste(response_fixed_hostsr, ssb, sss, geog), "PropHostSR"),
                c(paste(response_fixed_hostsr, ssb, sss), "PropHostSR"),
                c(paste(response_fixed_hostsr, ssb, geog), "PropHostSR"),
                c(paste(response_fixed_hostsr, ssb), "PropHostSR"),
                c(response_fixed_hostsr, "PropHostAbundance"),
                c(paste(response_fixed_hostabund, ssb, sss, geog), "PropHostAbundance"),
                c(paste(response_fixed_hostabund, ssb, sss), "PropHostAbundance"),
                c(paste(response_fixed_hostabund, ssb, geog), "PropHostAbundance"),
                c(paste(response_fixed_hostabund, ssb), "PropHostAbundance"))

# fit models
for(x in 1:length(formulae)){
  
  # model id
  modid = sample(1:100000, 1)
  form = formula(formulae[[x]][1])
  resp = formulae[[x]][2]
  
  # hostSR model
  if(resp == "PropHostSR"){
    host_mod = inla(form, data=models_df,
                    family="poisson", E=1,
                    control.fixed = control.fixed1,
                    control.predictor=list(compute=TRUE, link=1), # link scale pred
                    control.compute=list(cpo=TRUE, waic=TRUE, config=TRUE))
  }
  
  # abundance model
  if(resp == "PropHostAbundance"){
    host_mod =  inla(form, data=models_df,
                     family="gaussian", 
                     control.predictor=list(compute=T),
                     control.compute=list(cpo=T, waic=T),
                     control.fixed=control.fixed1,
                     control.family=control.family.gauss)
  }
  
  # save fixedeffects
  # tiff(filename=paste("./output/model_outputs/cargocarrier/model_selection/", formulae[[x]][2], "_", modid, "_", type, "_fixed.tif", sep=""), res=300, width=8, height=8, units="in")
  # plotFixed( getFixed(host_mod), col="darkblue")
  # dev.off()
  
  # create diagnostic dataframe
  dat = data.frame(modid = modid,
                   response = formulae[[x]][2],
                   formula = formulae[[x]][1],
                   waic = host_mod$waic$waic,
                   n_eff_rep = as.numeric(host_mod$neffp[3, 1]), # number of effective replicates
                   n_obs = length(host_mod$cpo$cpo), # number of datapoints used to fit model
                   n_cpo_fails = length( which(host_mod$cpo$failure > 0) ),
                   sum_cpo = sum(host_mod$cpo$cpo, na.rm=T), # if cpo = 1 model predicts point perfectly; so perfect fit would be n_obs
                   meanlogcpo = -mean( log( host_mod$cpo$cpo ), na.rm=T) # if every point predicted perfectly, -mean(log(cpo)) == 0, so larger vals indicate greater deviance
  )
  
  if(x == 1){ dat1 = dat } else{ dat1 = rbind(dat1, dat)}
  print(x)
}

# save diagnostics
write.csv(dat1, file=paste("./output/model_outputs/community_tier2_mammals/model_selection/prophost_", type, "_rsel_diagnostics.csv", sep=""), row.names=F)





# (b) fixed effects selection using best random effects structure ----------

response_hostsr = "HostSR ~ 1 + logTotalSR"
response_hostabund = "log(HostAbundance+1) ~ 1 + logTotalAbundance"
lu = "+ LU_UI"
hpd = "+ LogHumanDensity"
rand_hostsr = "+ f(SS, model='iid', hyper=hyper1.iid)"
rand_hostabund = "+ f(SS, model='iid', hyper=hyper1.iid) + f(SSB, model='iid', hyper=hyper1.iid) + f(SSBS, model='iid', hyper=hyper1.iid) + f(Biome, model='iid', hyper=hyper1.iid)"

# formulae (with best fitting random effects structure)
formulae1 = list( c(paste(response_hostsr, lu, rand_hostsr), "PropHostSR"),
                  c(paste(response_hostsr, lu, hpd, rand_hostsr), "PropHostSR"),
                  c(paste(response_hostabund, lu, rand_hostabund), "PropHostAbundance"),
                  c(paste(response_hostabund, lu, hpd, rand_hostabund), "PropHostAbundance") )

# fit models
for(x in 1:length(formulae1)){
  
  # model id
  modid = sample(1:100000, 1)
  form = formula(formulae1[[x]][1])
  resp = formulae1[[x]][2]
  
  # nonhostSR model
  if(resp == "PropHostSR"){
    host_mod = inla(form, data=models_df,
                    family="poisson", E=1,
                    control.fixed = control.fixed1,
                    control.predictor=list(compute=TRUE, link=1), # link scale pred
                    control.compute=list(cpo=TRUE, waic=TRUE, config=TRUE))
  }
  
  # nonhost abundance model
  if(resp == "PropHostAbundance"){
    host_mod =  inla(form, data=models_df,
                     family="gaussian", 
                     control.predictor=list(compute=T),
                     control.compute=list(cpo=T, waic=T),
                     control.fixed=control.fixed1,
                     control.family=control.family.gauss)
  }
  
  # save fixedeffects
  # tiff(filename=paste("./output/model_outputs/cargocarrier/model_selection/", formulae1[[x]][2], "_", modid, "_", type, "_fsel_fixed.tif", sep=""), res=300, width=8, height=8, units="in")
  # plotFixed( getFixed(host_mod), col="darkblue")
  # dev.off()
  
  # create diagnostic dataframe
  dat = data.frame(modid = modid,
                   response = formulae1[[x]][2],
                   formula = formulae1[[x]][1],
                   waic = host_mod$waic$waic,
                   n_eff_rep = as.numeric(host_mod$neffp[3, 1]), # number of effective replicates
                   n_obs = length(host_mod$cpo$cpo), # number of datapoints used to fit model
                   n_cpo_fails = length( which(host_mod$cpo$failure > 0) ),
                   sum_cpo = sum(host_mod$cpo$cpo, na.rm=T), # if cpo = 1 model predicts point perfectly; so perfect fit would be n_obs
                   meanlogcpo = -mean( log( host_mod$cpo$cpo ), na.rm=T) # if every point predicted perfectly, -mean(log(cpo)) == 0, so larger vals indicate greater deviance
  )
  
  if(x == 1){ dat1 = dat } else{ dat1 = rbind(dat1, dat)}
  print(x)
}

# save diagnostics
write.csv(dat1, file=paste("./output/model_outputs/community_tier2_mammals/model_selection/prophost_", type, "_fsel_diagnostics.csv", sep=""), row.names=F)




# -------------- combine model selection data -----------------

# final model selection
fsel1 = read.csv("./output/model_outputs/community_tier2_mammals/model_selection/host_intensityfixed_fsel_diagnostics.csv")
fsel2 = read.csv("./output/model_outputs/community_tier2_mammals/model_selection/nonhost_intensityfixed_fsel_diagnostics.csv")
fsel3 = read.csv("./output/model_outputs/community_tier2_mammals/model_selection/prophost_intensityfixed_fsel_diagnostics.csv")

# get best models (minimising WAIC / maximising sum-CPO)
best_models = rbind(fsel1[c(2, 6),],
                    fsel2[c(2, 4),],
                    fsel3[c(4, 6),])

# save
write.csv(best_models, "./output/model_outputs/community_tier2_mammals/model_selection/Tier2_mammals_communitymodels_bestmodels.csv", row.names=F)
