
### Gibb et al, 'Land use has global and systematic effects on local zoonotic host communities"
# Script 9: Fit best-fitting community models (mammal zoonotic reservoirs subset) using mean false negative prob as a linear covariate to control for research effort
# Extract observed vs. fitted values and residuals for checking model fit and spatial autocorrelaion


# dependencies
library(lme4)
library(ggplot2)
library(nlme)
library(plyr)
library(dismo)
library(geosphere)
library(spdep)
library(rgdal)
library(INLA)
library(viridis)
library(raster)
library(RColorBrewer)
library(KernSmooth)
library(dplyr)

# colours for plotting
nonhostcol = "#018571"
hostcol = "#8c510a"

# directory for saving
#save_dir = "./output/model_outputs/dec_2019/fullmodels_meaneffort/"
save_dir = "./full_models/"



# ------------ functions for model fitting and bootstrap -----------

# ---- Aggregate land use categories (updated 16/02/18) ----
#' @param df PREDICTS dataframe

prepareLandUse = function(df){
  
  # land use mappings
  plu = data.frame(Predominant_land_use = c("Primary vegetation","Young secondary vegetation","Intermediate secondary vegetation","Mature secondary vegetation", "Secondary vegetation (indeterminate age)", "Plantation forest","Pasture","Cropland","Urban"),
                   PLU2 = c("1_Primary", "Secondary","Secondary","Secondary", "Secondary", "Plantation forest", "Pasture", "Cropland", "Urban"),
                   LandUseType = c("1_Primary","Secondary","Secondary","Secondary","Secondary","Managed","Managed","Managed","Urban"),
                   stringsAsFactors = F)
  
  # merge and reshuffle columns
  df$foo = 1:nrow(df)
  df1 = merge(df, plu, by="Predominant_land_use", all.x=T)
  df1 = df1[ , c(2:ncol(df1), 1) ]
  df1 = df1[ order(df1$foo), ]
  
  # remove 'use' from intensity
  df1$Use_intensity = gsub(" use","", as.vector(df1$Use_intensity) )
  
  # combine use intensities into three states (minimal, substantial, cannot decide)
  df1$UseIntensity = ifelse(df1$Use_intensity == "Minimal", "Minimal", "Substantial")
  df1$UseIntensity[ df1$Use_intensity == "Cannot decide" ] = "Cannot decide"
  
  # combine with land use type for composite variable for modelling
  df1$LU_UI = paste(df1$LandUseType, df1$UseIntensity, sep="_")
  df1$LU_UI[ df1$LU_UI == "1_Primary_Substantial" ] = "Primary_Substantial"
  
  return( as.data.frame(df1, stringsAsFactors=F) )
}


# ---- Calculate host richness and abundance ----
#' @param df PREDICTS dataframe
# this function nested within aggregateStudy and aggregateSite functions

addHostMetrics = function(df){
  
  # total richness and host richness (1 for each species record with non-absence record associated)
  # calculate for host status, strict host status, and weighted for effort
  df$SR = ifelse(df$Effort_corrected_measurement > 0, 1, 0)
  df$HostSR = df$SR * df$HumanInfective_Any
  
  # abundance and host abundance (i.e. 0 for nonhosts)
  df$Abundance = df$Effort_corrected_measurement
  df$HostAbundance = df$Abundance * df$HumanInfective_Any
  
  # set abundance related metrics to NA for occurrence only studies
  df[ df$Diversity_metric_type == "Occurrence", c("Abundance", "HostAbundance") ] = NA
  return(df)
}


# ----------- Community weighted mean function: for calculating weighted trait values ------------

# applied within aggregateSite function
# if all abundances are equal, then commWeightMean(trait_values) == mean(trait_values)
#' @param traits vector of species trait values
#' @param abundances vector of species abundances

commWeightMean = function(trait_values, abundances){
  if(all(is.na(abundances))) return(NA)
  abundances_weighted = abundances / sum(abundances) # weight abundances
  sum(trait_values*abundances_weighted, na.rm=T) # multiply by trait values and sum
}



# ------------ prepare dataset --------

# load PREDICTS data
load(file="./data/predicts_processed/predicts_pathogens_researcheffort.R")

# keep only mammal studies (n=66), prepare land use categories
xx = ddply(dd1, .(SS), plyr::summarise, include_mammalia=ifelse(any(Class == "Mammalia"), 1, 0))
dd1 = dd1[ dd1$SS %in% xx$SS[ xx$include_mammalia == 1], ]
dd1 = prepareLandUse(dd1)

# tier 2 strictness: mammals pathogen isolation/direct detection, and set any mammal non-hosts to 0
dd1$HumanInfective_Any[ dd1$Class == "Mammalia" ] = dd1$Isolation_HumanInfective_Any[ dd1$Class == "Mammalia" ]
dd1$IsZoonotic_Strict[ dd1$Class == "Mammalia" ] = dd1$Isolation_IsZoonotic_Strict[ dd1$Class == "Mammalia" ]
dd1$HumanInfective_Any[ dd1$Class != "Mammalia" ] = 0
dd1$IsZoonotic_Strict[ dd1$Class != "Mammalia" ] = 0

# set 'human infective' to 'zoonotic' for zoonotic agents subset
dd1$HumanInfective_Any = dd1$IsZoonotic_Strict



# ---------- formulae for model ----------

# 1 = HostSR
# 2 = HostAbundance
# 3 = NonHostSR
# 4 = NonHostAbundance
# 5 = PropHostSR
# 6 = PropHostAbundance

# best model formulae (from fitting in script 6)
# include mean / community weighted mean effort measure
formulae = data.frame(
  response = c("HostSR", "HostAbundance", "NonHostSR", "NonHostAbundance", "PropHostSR", "PropHostAbundance"),
  formula = c(
    "HostSR ~ 1 + LU_UI + MeanEffort + f(SS, model='iid', hyper=hyper1.iid) + f(SSB, model='iid', hyper=hyper1.iid)",
    "log(HostAbundance+1) ~ LU_UI + CWMeanEffort + f(SS, model='iid', hyper=hyper1.iid) + f(SSB, model='iid', hyper=hyper1.iid) + f(SSBS, model='iid', hyper=hyper1.iid)",
    "NonHostSR ~ 1 + LU_UI + f(SS, model='iid', hyper=hyper1.iid) + f(SSB, model='iid', hyper=hyper1.iid)",
    "log(NonHostAbundance+1) ~ LU_UI + f(SS, model='iid', hyper=hyper1.iid) + f(SSB, model='iid', hyper=hyper1.iid) + f(SSBS, model='iid', hyper=hyper1.iid)",
    "HostSR ~ 1 + LU_UI + MeanEffort +f(SS, model='iid', hyper=hyper1.iid) + f(SSB, model='iid', hyper=hyper1.iid)",
    "log(HostAbundance+1) ~ LU_UI + CWMeanEffort + logTotalAbund + f(SS, model='iid', hyper=hyper1.iid) + f(SSB, model='iid', hyper=hyper1.iid) + f(SSBS, model='iid', hyper=hyper1.iid)"
  )
)


# ----------- set up  ----------

#### prepare dataset for modelling

# calculate host metrics
ddi = addHostMetrics(dd1)

# for any hosts, set FNRi to 0 (i.e. false negative prob of 0 assuming all true hosts)
ddi$FNRi = ddi$FNR
ddi$FNRi[ ddi$HumanInfective_Any == 1 ] = 0

# summarise by site (removing absences)
ddi_site = ddply(ddi, .(SSS), plyr::summarize,
                 Diversity_metric_type = head(as.vector(Diversity_metric_type), 1),
                 TotalSR = sum(SR),
                 HostSR = sum(HostSR),
                 TotalAbundance = sum(Abundance),
                 HostAbundance = sum(HostAbundance),
                 MeanEffort = mean(FNRi * SR, na.rm=T), # mean or cw false negative prob
                 CWMeanEffort = commWeightMean(FNRi, Abundance)
)

# calculate nonhost metrics
ddi_site$NonHostSR = ddi_site$TotalSR - ddi_site$HostSR
ddi_site$NonHostAbundance = ddi_site$TotalAbundance - ddi_site$HostAbundance

# combine with relevant columns
dfx = ddi[ !duplicated(ddi$SSS), c("SSS", "SS", "SSB", "SSBS", 
                                   "Latitude", "Longitude", 
                                   "Country", "UN_subregion", "Biome", "Realm", "PLU2", "LandUseType", 
                                   "Use_intensity", "UseIntensity", "LU_UI", 
                                   "HumanDensity", "MeanAnnualTemp", "MeanAnnualPrecip") ]
ddi_site = left_join(ddi_site, dfx, by="SSS")
ddi_site$LogHumanDensity = log(ddi_site$HumanDensity+1)

# calculate log total abundance/richness as offset
ddi_site$logTotalSR = log(ddi_site$TotalSR+1)
ddi_site$logTotalAbund = log(ddi_site$TotalAbundance+1)

# remove 'cannot decide' use intensity
ddi_site = ddi_site[ ddi_site$UseIntensity != "Cannot decide", ]

# combine all managed and all urban sites together: improve evenness of sampling
#table(ddi_site$LU_UI)
ddi_site$LU_UI[ grep("Managed", ddi_site$LU_UI) ] = "Managed"
ddi_site$LU_UI[ grep("Urban", ddi_site$LU_UI) ] = "Urban"



# ----------- run models ----------

# loop to run all models
for(i in 1:6){
  
  #### tracker if running locally
  print(i)
  
  # get model specifications for run
  formula = as.vector( formulae$formula[ i ] )
  response = as.vector( formulae$response[ i ] )
  
  #### setup and run model inference
  fitCommunityModel = function(df, response, formula){
    
    # run to work around server GLIBC issue
    INLA:::inla.dynload.workaround()
    
    # get model formula
    err = simpleError("Error!")
    form = formula(formula)
    
    ### priors
    hyper1.iid = list(prec = list(prior='loggamma', param=c(0.1,0.1)))
    control.fixed1 = list(mean.intercept=0, mean = 0,
                          prec = 0.01, prec.intercept = 0.01)
    control.family.gauss = list(hyper=list(prec=list(prior='loggamma', param=c(0.1, 0.1))))
    
    # gaussian models for abundance
    if(response %in% c("HostAbundance", "NonHostAbundance", "PropHostAbundance")){
      model = tryCatch(inla(form, data=df,
                            family="gaussian", 
                            control.predictor=list(compute=T),
                            control.compute=list(cpo=T, waic=T, config=TRUE),
                            control.fixed=control.fixed1,
                            control.family=control.family.gauss), error=function(err) err)
    }
    
    # poisson for SR
    if(response %in% c("HostSR", "NonHostSR")){
      model = tryCatch(inla(form, data=df,
                            family="poisson", E=1,
                            control.fixed = control.fixed1,
                            control.predictor=list(compute=TRUE, link=1), # link scale pred
                            control.compute=list(cpo=TRUE, waic=TRUE, config=TRUE)), error=function(err) err)
    }
    
    # poisson with offset for prop SR
    if(response %in% c("PropHostSR")){
      model = tryCatch(inla(form, data=df,
                            family="poisson", 
                            offset=logTotalSR,
                            control.fixed = control.fixed1,
                            control.predictor=list(compute=TRUE, link=1), # link scale pred
                            control.compute=list(cpo=TRUE, waic=TRUE, config=TRUE)), error=function(err) err)
    }
    
    # return error if problem with model
    if(class(model)[1]=="simpleError"){ return("Error in model fitting") }
    return(model)
  }
  
  # run model
  mi = fitCommunityModel(df=ddi_site, response=response, formula=formula)
  if(class(mi) != "inla"){ # skip to next iteration if model breaks
    result[[i]] = "Model fit error"
    next 
  } 
  
  # save model
  unique_id = sample(1:10^8, 1)
  filename = paste("tier2mammals", response, unique_id, "fullmodel_meaneffort.R", sep="_")
  save(mi, file=paste(save_dir, filename, sep=""))
}



### ------------ Get fitted values and residuals for testing spatial autocorrelation and visualising obs/fit ---------------

### load models and extract fitted values and residuals
mm = list.files(paste(save_dir, sep=""), pattern='.R')
mm = mm[ grep("tier2", mm) ]

# create results df without response variable
resx1 = ddi_site[ , c("SS", "SSS", "SSB", "Latitude", "Longitude", "Biome", "LandUseType", "UseIntensity", "LU_UI", "TotalSR", "TotalAbundance")]

# full result
resid = data.frame()

# for each model
for(mod in mm){
  
  # response
  resp = strsplit(mod, "_")[[1]][2]
  print(resp)
  
  # new model
  resx2 = resx1
  resx2$ResponseName = resp
  
  # load model
  load(paste(save_dir, mod, sep=""))
  
  # create response var column
  if(resp == "HostAbundance"){ resx2$ResponseValue = log(ddi_site$HostAbundance+1) }
  if(resp == "HostSR"){ resx2$ResponseValue = ddi_site$HostSR }
  if(resp == "NonHostAbundance"){ resx2$ResponseValue = log(ddi_site$NonHostAbundance+1) }
  if(resp == "NonHostSR"){ resx2$ResponseValue = ddi_site$NonHostSR }
  if(resp == "PropHostAbundance"){ resx2$ResponseValue = log(ddi_site$HostAbundance+1) }
  if(resp == "PropHostSR"){ resx2$ResponseValue = ddi_site$HostSR }
  
  # get fitted values
  ff = mi$summary.fitted.values[ , c(1, 4)]
  names(ff) = c("FittedMean", "FittedMedian")
  resx2 = cbind(resx2, ff)
  
  # calculate residual and append to data frame
  resx2$Residual = resx2$FittedMedian - resx2$ResponseValue
  resid = rbind(resid, resx2)
}

# save residuals
write.csv(resid, paste(save_dir, "tier2_fullmodelresiduals_20200103.csv", sep=""))






