
### Gibb et al, 'Land use has global and systematic effects on local zoonotic host communities"
# Script 10: Runs research effort bootstrap for community models (all PREDICTS)
# For each iteration, randomly transitions n 'non-host' species into the 'host' compartment, with probability FNR 
# i.e. species with less confidence in non-host status will transition at highest rates
# Then recalculates community (species richness/abundance) response variables and fits models to these
# For each iteration extracts key model outputs and 2000 samples from the approximated posterior, and saves to a list
# This script is designed to run on cluster with each run fitting n models (currently 250)


#setwd("/Users/Rory/Dropbox/PhD/chapter_predicts/predicts_analysis_201901/")
setwd("/scratch/scratch/ucbtgib/predicts")

# dependencies
library(lme4)
library(ggplot2)
library(nlme)
library(plyr)
library(dismo)
library(geosphere)
library(spdep)
library(rgdal)
library(INLA)
library(viridis)
library(raster)
library(RColorBrewer)
library(KernSmooth)
library(dplyr)

# colours for plotting
nonhostcol = "#018571"
hostcol = "#8c510a"

# directory and unique identifier for saving
#save_dir = "./output/model_outputs/community_tier1/effort_bootstrap/"
save_dir = "./bootstrap_output/"
unique_id = sample(1:10^8, 1)



# ------------ functions for model fitting and bootstrap -----------

# ---- Aggregate land use categories (updated 16/02/18) ----
#' @param df PREDICTS dataframe

prepareLandUse = function(df){
  
  # land use mappings
  plu = data.frame(Predominant_land_use = c("Primary vegetation","Young secondary vegetation","Intermediate secondary vegetation","Mature secondary vegetation", "Secondary vegetation (indeterminate age)", "Plantation forest","Pasture","Cropland","Urban"),
                   PLU2 = c("1_Primary", "Secondary","Secondary","Secondary", "Secondary", "Plantation forest", "Pasture", "Cropland", "Urban"),
                   LandUseType = c("1_Primary","Secondary","Secondary","Secondary","Secondary","Managed","Managed","Managed","Urban"),
                   stringsAsFactors = F)
  
  # merge and reshuffle columns
  df$foo = 1:nrow(df)
  df1 = merge(df, plu, by="Predominant_land_use", all.x=T)
  df1 = df1[ , c(2:ncol(df1), 1) ]
  df1 = df1[ order(df1$foo), ]
  
  # remove 'use' from intensity
  df1$Use_intensity = gsub(" use","", as.vector(df1$Use_intensity) )
  
  # combine use intensities into three states (minimal, substantial, cannot decide)
  df1$UseIntensity = ifelse(df1$Use_intensity == "Minimal", "Minimal", "Substantial")
  df1$UseIntensity[ df1$Use_intensity == "Cannot decide" ] = "Cannot decide"
  
  # combine with land use type for composite variable for modelling
  df1$LU_UI = paste(df1$LandUseType, df1$UseIntensity, sep="_")
  df1$LU_UI[ df1$LU_UI == "1_Primary_Substantial" ] = "Primary_Substantial"
  
  return( as.data.frame(df1, stringsAsFactors=F) )
}


# ---- Calculate host richness and abundance ----
#' @param df PREDICTS dataframe
# this function nested within aggregateStudy and aggregateSite functions

addHostMetrics = function(df){
  
  # total richness and host richness (1 for each species record with non-absence record associated)
  # calculate for host status, strict host status, and weighted for effort
  df$SR = ifelse(df$Effort_corrected_measurement > 0, 1, 0)
  df$HostSR = df$SR * df$HumanInfective_Any

  # abundance and host abundance (i.e. 0 for nonhosts)
  df$Abundance = df$Effort_corrected_measurement
  df$HostAbundance = df$Abundance * df$HumanInfective_Any

  # set abundance related metrics to NA for occurrence only studies
  df[ df$Diversity_metric_type == "Occurrence", c("Abundance", "HostAbundance") ] = NA
  return(df)
}



# ------------ prepare dataset --------

# load PREDICTS data
#load(file="./output/predicts_processed/predicts_pathogens_researcheffort.R")
load(file="./predicts_pathogens_researcheffort.R")

# remove studies with no host matches (all cargo-carrier hosts), prepare land use categories
dd1 = dd1[ dd1$SS_HIAMB == 1, ]
dd1 = prepareLandUse(dd1)

# tier 1 strictness: cargo carrier for non-mammals, mammals serological and any human infectivity
dd1$HumanInfective_Any[ dd1$Class == "Mammalia" ] = dd1$Sero_HumanInfective_Any[ dd1$Class == "Mammalia" ]
dd1$IsZoonotic_Strict[ dd1$Class == "Mammalia" ] = dd1$Sero_IsZoonotic_Strict[ dd1$Class == "Mammalia" ]



# ---------- each bootstrap runs n iterations of a randomly selected model (1:6) -----------

# sample randomly for model type
num_iterations = 250
model_type = sample(1:6, 1)
n_samples = 2500

# 1 = HostSR
# 2 = HostAbundance
# 3 = NonHostSR
# 4 = NonHostAbundance
# 5 = PropHostSR
# 6 = PropHostAbundance

# best model formulae (from fitting in script 6)
formulae = data.frame(
  response = c("HostSR", "HostAbundance", "NonHostSR", "NonHostAbundance", "PropHostSR", "PropHostAbundance"),
  formula = c(
    "HostSR ~ 1 + LU_UI + f(SS, model='iid', hyper=hyper1.iid) + f(SSB, model='iid', hyper=hyper1.iid)",
    "log(HostAbundance+1) ~ LU_UI + LogHumanDensity + f(SS, model='iid', hyper=hyper1.iid) + f(SSB, model='iid', hyper=hyper1.iid) + f(SSBS, model='iid', hyper=hyper1.iid)",
    "NonHostSR ~ 1 + LU_UI + f(SS, model='iid', hyper=hyper1.iid) + f(SSB, model='iid', hyper=hyper1.iid)",
    "log(NonHostAbundance+1) ~ LU_UI + LogHumanDensity + f(SS, model='iid', hyper=hyper1.iid) + f(SSB, model='iid', hyper=hyper1.iid) + f(SSBS, model='iid', hyper=hyper1.iid) + f(Biome, model='iid', hyper=hyper1.iid)",
    "HostSR ~ 1 + LU_UI + f(SS, model='iid', hyper=hyper1.iid) + f(SSB, model='iid', hyper=hyper1.iid)",
    "log(HostAbundance+1) ~ LU_UI + logTotalAbund + LogHumanDensity + f(SS, model='iid', hyper=hyper1.iid) + f(SSB, model='iid', hyper=hyper1.iid) + f(SSBS, model='iid', hyper=hyper1.iid) + f(Biome, model='iid', hyper=hyper1.iid)"
  )
)

# get model specifications for run
formula = as.vector( formulae$formula[ model_type ] )
response = as.vector( formulae$response[ model_type ] )

# save tracking output
write.csv(c(1, 2, 3), paste("./output_tracker/", unique_id, "tier1_runningmodels.csv", sep="_"))



 # ---------- run iterations and at each iteration, impute non-hosts ----------

# list to save results
result = vector("list", num_iterations)

# bootstrap loop
for(i in 1:num_iterations){
  
  #### tracker if running locally
  print(i)
  
  #### implement non-host -> host transition
  
  # impute n non-host species at rate = FNR (approx false negative prob)
  spp = dd1[ !duplicated(dd1$bgb), c("bgb", "Order", "FNR", "HumanInfective_Any") ]
  spp$ImputeRate = spp$FNR
  spp$ImputeRate[ is.na(spp$ImputeRate) ] = 0.001
  ran_impute = function(n){ rbinom(1, 1, prob=spp$ImputeRate[n]) }
  spp$IsHost_imp = unlist(lapply(1:nrow(spp), ran_impute))
  number_species_imputed = sum(spp$IsHost_imp[ spp$HumanInfective_Any == 0 ]) # calculate number imputed for saving
  spp$IsHost_imp[ spp$HumanInfective_Any == 1 ] = 1
  
  # combine with PREDICTS for ith iteration
  ddi = left_join(dd1, spp)
  ddi$HumanInfective_Any = ddi$IsHost_imp
  
  
  #### prepare dataset for modelling
  
  # calculate host metrics
  ddi = addHostMetrics(ddi)
  
  # summarise by site
  ddi_site = ddply(ddi, .(SSS), plyr::summarize,
                   Diversity_metric_type = head(as.vector(Diversity_metric_type), 1),
                   TotalSR = sum(SR),
                   HostSR = sum(HostSR),
                   TotalAbundance = sum(Abundance),
                   HostAbundance = sum(HostAbundance)
                   )
  
  # calculate nonhost metrics
  ddi_site$NonHostSR = ddi_site$TotalSR - ddi_site$HostSR
  ddi_site$NonHostAbundance = ddi_site$TotalAbundance - ddi_site$HostAbundance

  # set abundance to NA for occurrence models
  ddi_site[ ddi_site$Diversity_metric_type == "Occurrence" , grep("Abundance", names(ddi_site))] = NA
    
  # combine with relevant columns
  dfx = ddi[ !duplicated(ddi$SSS), c("SSS", "SS", "SSB", "SSBS", 
                                     "Latitude", "Longitude", 
                                     "Country", "UN_subregion", "Biome", "Realm", "PLU2", "LandUseType", 
                                     "Use_intensity", "UseIntensity", "LU_UI", 
                                     "HumanDensity", "MeanAnnualTemp", "MeanAnnualPrecip") ]
  ddi_site = full_join(dfx, ddi_site, by="SSS")
  ddi_site$LogHumanDensity = log(ddi_site$HumanDensity+1)
  
  # calculate log total abundance/richness as offset
  ddi_site$logTotalSR = log(ddi_site$TotalSR+1)
  ddi_site$logTotalAbund = log(ddi_site$TotalAbundance+1)
  
  # remove 'cannot decide' use intensity
  ddi_site = ddi_site[ ddi_site$UseIntensity != "Cannot decide", ]
  
  
  #### setup and run model inference
  fitBootstrapModel = function(df, response, formula){
    
    # run to work around server GLIBC issue
    INLA:::inla.dynload.workaround()
    
    # get model formula
    err = simpleError("Error!")
    form = formula(formula)
    
    ### priors
    hyper1.iid = list(prec = list(prior='loggamma', param=c(0.1,0.1)))
    control.fixed1 = list(mean.intercept=0, mean = 0,
                          prec = 0.01, prec.intercept = 0.01)
    control.family.gauss = list(hyper=list(prec=list(prior='loggamma', param=c(0.1, 0.1))))
    
    # gaussian models for abundance
    if(response %in% c("HostAbundance", "NonHostAbundance", "PropHostAbundance")){
      model = tryCatch(inla(form, data=df,
                            family="gaussian", 
                            control.predictor=list(compute=T),
                            control.compute=list(cpo=T, waic=T, config=TRUE),
                            control.fixed=control.fixed1,
                            control.family=control.family.gauss), error=function(err) err)
    }
    
    # poisson for SR
    if(response %in% c("HostSR", "NonHostSR")){
      model = tryCatch(inla(form, data=df,
                            family="poisson", E=1,
                            control.fixed = control.fixed1,
                            control.predictor=list(compute=TRUE, link=1), # link scale pred
                            control.compute=list(cpo=TRUE, waic=TRUE, config=TRUE)), error=function(err) err)
    }
    
    # poisson with offset for prop SR
    if(response %in% c("PropHostSR")){
      model = tryCatch(inla(form, data=df,
                            family="poisson", 
                            offset=logTotalSR,
                            control.fixed = control.fixed1,
                            control.predictor=list(compute=TRUE, link=1), # link scale pred
                            control.compute=list(cpo=TRUE, waic=TRUE, config=TRUE)), error=function(err) err)
    }
    
    # return error if problem with model
    if(class(model)[1]=="simpleError"){ return("Error in model fitting") }
    return(model)
  }
    
  # run model
  mi = fitBootstrapModel(df=ddi_site, response=response, formula=formula)
  if(class(mi) != "inla"){ # skip to next iteration if model breaks
    result[[i]] = "Model fit error"
    next 
    } 
  
  # create summary object
  resi = list()
  resi$response = response
  resi$species_imputed = number_species_imputed
  resi$fixed = mi$summary.fixed
  resi$hyper = mi$summary.hyperpar
  
  # extract posterior samples (specified above)
  samp = inla.posterior.sample(n_samples, result=mi)
  
  # extract from samples
  getSampFixed = function(x){ 
    ff = as.data.frame(t(tail(samp[[x]]$latent, 12)))
    }
  fixed_samples = do.call(rbind.data.frame, lapply(1:length(samp), getSampFixed))
  row.names(fixed_samples) = 1:nrow(fixed_samples)
  resi$samples_fixed = fixed_samples
  
  # get observed and fitted for calculating residuals
  of = data.frame(
    SSS = ddi_site$SSS,
    HostSR = ddi_site$HostSR,
    NonHostSR = ddi_site$NonHostSR,
    HostAbundance = ddi_site$HostAbundance,
    NonHostAbundance = ddi_site$NonHostAbundance,
    Fitted_val = mi$summary.fitted.values$mean
  )
  if(response == "HostSR"){ of$Observed = of$HostSR }
  if(response == "NonHostSR"){ of$Observed = of$NonHostSR }
  if(response == "PropHostSR"){ of$Observed = of$HostSR }
  if(response == "HostAbundance"){ of$Observed = log(of$HostAbundance+1) }
  if(response == "NonHostAbundance"){ of$Observed = log(of$NonHostAbundance+1) }
  if(response == "PropHostAbundance"){ of$Observed = log(of$HostAbundance+1) }
  of$Resid = of$Fitted_val - of$Observed
  resi$obs_fit = of[ , c("SSS", "Observed", "Fitted_val", "Resid")]
  
  # save to list
  result[[i]] = resi
}

# save result
filename = paste("tier1all", response, unique_id, num_iterations, "bootstrapx1.R", sep="_")
save(result, file=paste(save_dir, filename, sep=""))
#save(result, file=filename)



