
### Gibb et al, 'Land use has global and systematic effects on local zoonotic host communities"
# Script 14: Visualises and analyses residuals from community models (both all PREDICTS and mammal zoonotic reservoirs) to check for model fit / spatial auto.
# Plots observed versus fitted for each model (here, using mean FNR as a covariate to control for research effort, rather than full bootstrap)
# Runs Moran's I tests for spatial autocorrelation on all sites within each study to test how affected results may be by spatial autocorrelation
# Creates summary plot for suppmat

#setwd("/Volumes/ritd-ag-project-rd00nx-rjgib11/chapter2_predicts/predicts_analysis_201901/")
#setwd("/Users/rorygibb/Dropbox/PhD/chapter_predicts/predicts_analysis_201901/")
#setwd("Y:/chapter2_predicts/predicts_analysis_201901/")

# dependencies
library(lme4)
library(ggplot2)
library(nlme)
library(plyr)
library(dismo)
library(geosphere)
library(spdep)
library(rgdal)
library(INLA)
library(viridis)
library(raster)
library(RColorBrewer)
library(KernSmooth)
library(dplyr)
library(ggpubr)

# functions
source("./00_functions_general.R")

# colours for plotting
nonhostcol = "#018571"
hostcol = "#8c510a"

# directory where model outputs are stored
dat_dir = "./output/model_outputs/dec_2019/fullmodels_meaneffort/"


### ---------- consistent axis text/title sizes ---------

axtx = 10
axty = 10
axtitle = 12
strtitle = 12


### ---------- plot observed versus fitted -------------

# load model output data (observed, fitted, residuals)
files = list.files(dat_dir, pattern=".csv", full.names=TRUE)
read_mark = function(x){
  foo = read.csv(files[x], stringsAsFactors = FALSE)
  foo$tier = x
  foo
}
df = do.call(rbind.data.frame, lapply(1:2, read_mark))

# set up for plot
df$name = ifelse(df$tier == 1, "\n (all)", "\n (mammals)")
#df$name = ifelse(df$tier == 1, " (all)", " (mammals)")
df$Tier_Model = paste(df$ResponseName, df$name, sep="")

# set up factors
# xx = paste(c("HostSR", "NonHostSR", "PropHostSR", "HostAbundance", "NonHostAbundance", "PropHostAbundance"), " (all)", sep="")
# yy = paste(c("HostSR", "NonHostSR", "PropHostSR", "HostAbundance", "NonHostAbundance", "PropHostAbundance"), " (mammals)", sep="")
xx = paste(c("HostSR", "NonHostSR", "PropHostSR", "HostAbundance", "NonHostAbundance", "PropHostAbundance"), "\n (all)", sep="")
yy = paste(c("HostSR", "NonHostSR", "PropHostSR", "HostAbundance", "NonHostAbundance", "PropHostAbundance"), "\n (mammals)", sep="")
df$Tier_Model = factor(df$Tier_Model, levels=c(xx, yy), ordered=T)

# plot of observed versus fitted
ofp = ggplot(df[ !is.na(df$ResponseValue), ], aes(ResponseValue, FittedMedian)) + 
  geom_point(size=0.1) + 
  geom_abline(intercept=0, slope=1, col="darkred") +
  facet_wrap(~Tier_Model, scale="free", nrow=4, ncol=3) + 
  theme_classic() + xlab("Observed") + ylab("Fitted") +
  theme(strip.background = element_blank(),
        axis.title=element_text(size=axtitle)) 




### ----------- test for spatial autocorrelation in residuals ---------

# models
mods = as.vector(unique(df$Tier_Model))

# function to run moran within studies
getMoranStudy = function(mm){
  
  # get model
  print(mm)
  ss = df[ as.vector(df$Tier_Model) == mm, ]
  ss = ss[ !is.na(ss$Residual), ]

  # run separate moran's tests on residuals for each study
  studyMoran = function(x){
    dfx = ss[ ss$SS == unique(ss$SS)[x], ]
    err = simpleError("test error")
    m = tryCatch( runMoranTest(coords=cbind(dfx$Longitude, dfx$Latitude), variable=dfx$Residual, method="moran.mc"), error=function(err) err )
    if( class(m)[1] == "simpleError" ){ return(data.frame(SS = dfx$SS[1], Nsites = nrow(dfx), Estimate = NA, p.val = NA, Tier_Model = mm)) } 
    return(data.frame(SS = dfx$SS[1], Nsites = nrow(dfx), Estimate = m$statistic, p.val = m$p.value, Tier_Model = mm))
  }
  mtest_results = do.call(rbind.data.frame, lapply(1:length(unique(ss$SS)), studyMoran))
  row.names(mtest_results) = c()
  
  # calculate proportion significant
  #calcSig = function(x) signif( (sum( x$p.val < 0.05, na.rm=T ) / nrow(x)), 3)*100
  calcSig = function(x) signif( (sum( x$p.val < 0.05, na.rm=T ) / sum(!is.na(x$p.val))), 3)*100
  sig = calcSig(mtest_results)
  mtest_results$sig = sig
  mtest_results
}

# run across all models
moran_result = do.call(rbind.data.frame, lapply(mods, getMoranStudy))

# tier model and order titles
mr = moran_result
mr$Tier_Model = as.vector(mr$Tier_Model)
mr$Tier_Model2 = paste( substr(mr$Tier_Model, 1, nchar(mr$Tier_Model)-1), "; ", mr$sig, "%)", sep="")
fac_order = data.frame(a = c(xx, yy))
fac_order$b = mr$Tier_Model2[ match(fac_order$a, mr$Tier_Model) ]
mr$Tier_Model2 = factor(mr$Tier_Model2, levels=fac_order$b, ordered=T)

# moran tests plot
mtp = ggplot(mr, aes(p.val)) + 
  geom_histogram(bins=50, col="black", fill="grey70", lwd=0.4) +
  theme_classic() + 
  geom_vline(xintercept=0.05, col="darkred") + 
  ylab("Frequency") + xlab("p-value") +
  facet_wrap(~Tier_Model2, scales="free_y", nrow=4, ncol=3) +
  theme(strip.background = element_blank(),
        axis.title=element_text(size=axtitle))





### ----------- combine into single 24-panel diagnostics plot for Extended Data ---------

# combine and save
dp = gridExtra::arrangeGrob(ofp, mtp, ncol=2, nrow=1)
dp = ggpubr::as_ggplot(dp) +
  cowplot::draw_plot_label(label = c("A", "B"), fontface = "bold", size = 18, x = c(0.02, 0.52), y = c(0.99, 0.99))
ggsave(dp, filename="./output/figures/nature_ms3_jan2019/ExtendedData_CommunityModelDiagnostics.png",
       device="png", width=11.5, height=6.5, units="in", dpi=600)
