
### Gibb et al, 'Land use has global and systematic effects on local zoonotic host communities"
# Script 15: Reads and summarises output from models using mean research effort (FNR) as a linear covariate
# Allows for comparison with full bootstrapped ensemble: 
# effect sizes in managed and secondary land are larger and narrower CIs than in full bootstrapped ensemble

#setwd("/Users/rorygibb/Desktop")
# setwd("/Volumes/ritd-ag-project-rd00nx-rjgib11/chapter2_predicts/predicts_analysis_201901/")
# setwd("Y:/chapter2_predicts/predicts_analysis_201901/")
# setwd("/scratch/scratch/ucbtgib/predicts")


## --- dependencies
library(ggplot2)


## --- load models and extract fixed effects and other parameter estimates ---

# directory where model outputs are stored
dat_dir = "./output/model_outputs/dec_2019/fullmodels_meaneffort/"
#dat_dir = "./full_models/"

### load models and extract fitted values and residuals
#mm = list.files(paste(dat_dir, "models/", sep=""), pattern='.R')
mm = list.files(paste(dat_dir, sep=""), pattern='.R')

# extracts key outputs from each model
getModelOutput = function(m){
  #load(paste(dat_dir, "models/", m, sep=""))
  load(paste(dat_dir, m, sep=""))
  response = strsplit(m, "_")[[1]][2]
  tier = strsplit(m, "_")[[1]][1]
  waic = mi$waic$waic
  ff = mi$summary.fixed
  hh = mi$summary.hyperpar
  return(list(
    response = response,
    tier = tier,
    waic = waic,
    fixed = ff,
    hyper = hh
  ))
}

# run across all models and save
results = lapply(mm, getModelOutput)
save(results, file=paste(dat_dir, "communitymodels_fixedhyper_meaneffortcovar.R", sep=""))



### ---- 2. Combine and summarise outputs for fixed effects -----

# get info 
load(file=paste(dat_dir, "communitymodels_fixedhyper_meaneffortcovar.R", sep=""))
extractFixed = function(x){
  ff = as.data.frame(x$fixed)
  ff$param = row.names(ff)
  ff$param[ ff$param == "(Intercept)" ] = "Primary_Minimal"
  row.names(ff) = c()
  ff$model = x$response
  ff$tier = substr(x$tier, 5, 5)
  ff
}
df = do.call(rbind.data.frame, lapply(results, extractFixed))



# ------ 3. Visualise results ---------

## data
df = df[ ! df$param %in% c("MeanEffort", "CWMeanEffort", "LogHumanDensity", "logTotalAbund"), ]
df$param[ df$param != "Primary_Minimal" ] = substr(df$param[ df$param != "Primary_Minimal" ], 6, 100)
df[ df$param == "Primary_Minimal", 1:5] = 0 # make intercept 0
df[ , 1:5 ] = 100 * (exp(df[ 1:5]) - 1) # rescale, centre around 0, make %
names(df)[3:5] = c("lower_95", "median", "upper_95")

# sort param and model names
df$div_metric = ifelse(grepl("Abund", df$model), "Abundance", "SR")
df$model[ grep("Prop", df$model) ] = "Prop"
df$model[ grep("Host", df$model) ] = ifelse(grepl("Non", df$model[ grep("Host", df$model) ]), "Non-host", "Host")
dm2 = data.frame(model = c("Host", "Host", "Non-host", "Non-host", "Prop", "Prop"), 
                 div_metric=rep(c("SR", "Abundance"),3), 
                 div_metric2 = c(rep(c("Species richness", "Total abundance"), 2), c("Host proportion of richness", "Host proportion of total abundance")))
df = left_join(df, dm2)
df$div_metric2 = factor(df$div_metric2, levels=dm2$div_metric2[c(1, 2, 5, 6)], ordered=T)

# sort param names
df$param[ df$param %in% c("Managed", "Urban")] = paste(df$param[ df$param %in% c("Managed", "Urban")], "_Both", sep="")
df$LU = unlist(lapply(strsplit(df$param, "_"), "[[", 1))
df$Intensity = unlist(lapply(strsplit(df$param, "_"), "[[", 2))

# order factors
df$LU = factor(df$LU, levels=c("Primary", "Secondary", "Managed", "Urban"), ordered=T)
df$foo = paste(df$Intensity, df$model, sep="_")

# colours for plotting
nonhostcol = "#018571"
hostcol = "#8c510a"

# legend text size
legend_text_size = 7
point_size = 2



# ------- Plot for Tier 1 (all hosts) -------

# 1. Host-non host
dfx = df[ df$tier == 1 & df$model %in% c("Host", "Non-host"), ]
p1 = ggplot(dfx, aes(x=LU, y=median, split=Intensity, col=model)) + 
  geom_point(position=position_dodge(width=0.6), aes(pch=Intensity), size=point_size) +
  geom_errorbar(aes(ymin=lower_95, ymax=upper_95), position=position_dodge(width=0.6), width=0.2) + 
  theme_bw() + 
  xlab("") + 
  ylab("% difference from primary minimal") +
  geom_hline(yintercept=0, lty=2) +
  theme(legend.background = element_rect(size=0.2, linetype="solid", colour=NA, fill=NA),
        # axis.text = element_text(size=15),
        panel.grid.minor = element_line(colour="grey93", size=0.2), 
        panel.grid.major = element_line(colour="grey88", size=0.3),
        # axis.title.y = element_text(size=15),
        legend.position=c(0.4, 0.95),
        legend.text = element_text(size=legend_text_size),
        legend.title=element_blank(),
        legend.box="horizontal",
        strip.background = element_rect(fill="grey90")) + 
  scale_color_manual(values=c("Host"=hostcol, 
                              "Non-host"=nonhostcol),
                     name="Model") + 
  scale_shape_manual(name="Use intensity", values=c("Minimal"=16, "Substantial"=17), labels=c("Minimal use", "Substantial use")) +
  facet_wrap(~div_metric2, nrow=2, scales="free_y")

# proportion
dfy = df[ df$tier == 1 & df$model == "Prop", ]
p2 = ggplot(dfy, aes(x=LU, y=median, split=Intensity)) + 
  geom_point(position=position_dodge(width=0.6), aes(pch=Intensity), size=point_size, col=hostcol) +
  geom_errorbar(aes(ymin=lower_95, ymax=upper_95), position=position_dodge(width=0.6), width=0.15, col=hostcol) + 
  theme_bw() + 
  xlab("") + 
  ylab("% difference from primary minimal") +
  geom_hline(yintercept=0, lty=2) +
  theme(legend.background = element_rect(size=0.2, linetype="solid", colour=NA, fill=NA),
        # axis.text = element_text(size=15),
        panel.grid.minor = element_line(colour="grey93", size=0.2), 
        panel.grid.major = element_line(colour="grey88", size=0.3),
        # axis.title.y = element_text(size=15),
        #legend.position=c(0.42, 0.91),
        legend.position="none",
        # legend.text = element_text(size=12),
        legend.title=element_blank(),
        legend.box="horizontal",
        axis.title.y = element_blank(),
        strip.background = element_rect(fill="grey90")) + 
  scale_shape_manual(name="Use intensity", values=c("Minimal"=16, "Substantial"=17), labels=c("Minimal use", "Substantial use")) +
  facet_wrap(~div_metric2, nrow=2, scales="free_y")

# combine
pp = gridExtra::arrangeGrob(p1, p2, ncol=2, widths=c(1.1, 1))
pp = ggpubr::as_ggplot(pp) +
  cowplot::draw_plot_label(label = c("A", "B", "C", "D"), fontface = "bold", size = 18, x = c(0.035, 0.035, 0.53, 0.53), y = c(1, 0.535, 1, 0.535))
ggsave(pp, filename="./output/figures/nature_ms3_jan2019/Output_CommunityTier1_meaneffortmodels.png",
       device="png", width=7, height=7, units="in", dpi=1200, scale=0.9)



# ------- Plot for Tier 2 (mammal reservoirs) -------

# 1. Host-non host
dfx = df[ df$tier == 2 & df$model %in% c("Host", "Non-host"), ]
dfx$Intensity = factor(dfx$Intensity, levels=c("Minimal", "Substantial", "Both"), ordered=T)
p1 = ggplot(dfx, aes(x=LU, y=median, split=Intensity, col=model)) + 
  geom_point(position=position_dodge(width=0.6), aes(pch=Intensity), size=point_size, stroke=1.2) +
  geom_errorbar(aes(ymin=lower_95, ymax=upper_95), position=position_dodge(width=0.6), width=0.2) + 
  theme_bw() + 
  xlab("") + 
  ylab("% difference from primary minimal") +
  geom_hline(yintercept=0, lty=2) +
  theme(legend.background = element_rect(size=0.2, linetype="solid", colour=NA, fill=NA),
        # axis.text = element_text(size=15),
        panel.grid.minor = element_line(colour="grey93", size=0.2), 
        panel.grid.major = element_line(colour="grey88", size=0.3),
        # axis.title.y = element_text(size=15),
        legend.position=c(0.4, 0.93),
        legend.text = element_text(size=legend_text_size),
        legend.title=element_blank(),
        legend.box="horizontal",
        strip.background = element_rect(fill="grey90")) + 
  scale_color_manual(values=c("Host"=hostcol, 
                              "Non-host"=nonhostcol),
                     name="Model") + 
  scale_shape_manual(name="Use intensity", values=c("Minimal"=16, "Substantial"=17, "Both"=4), labels=c("Minimal use", "Substantial use", "Both")) +
  facet_wrap(~div_metric2, nrow=2, scales="free_y")

# proportion
dfy = df[ df$tier == 2 & df$model == "Prop", ]
dfy$Intensity = factor(dfy$Intensity, levels=c("Minimal", "Substantial", "Both"), ordered=T)
p2 = ggplot(dfy, aes(x=LU, y=median, split=Intensity)) + 
  geom_point(position=position_dodge(width=0.6), aes(pch=Intensity), size=point_size, col=hostcol, stroke=1.2) +
  geom_errorbar(aes(ymin=lower_95, ymax=upper_95), position=position_dodge(width=0.6), width=0.15, col=hostcol) + 
  theme_bw() + 
  xlab("") + 
  ylab("% difference from primary minimal") +
  geom_hline(yintercept=0, lty=2) +
  theme(legend.background = element_rect(size=0.2, linetype="solid", colour=NA, fill=NA),
        # axis.text = element_text(size=15),
        panel.grid.minor = element_line(colour="grey93", size=0.2), 
        panel.grid.major = element_line(colour="grey88", size=0.3),
        # axis.title.y = element_text(size=15),
        #legend.position=c(0.42, 0.91),
        legend.position="none",
        # legend.text = element_text(size=12),
        legend.title=element_blank(),
        legend.box="horizontal",
        axis.title.y = element_blank(),
        strip.background = element_rect(fill="grey90")) + 
  scale_shape_manual(name="Use intensity", values=c("Minimal"=16, "Substantial"=17, "Both"=4), labels=c("Minimal use", "Substantial use", "Both")) +
  facet_wrap(~div_metric2, nrow=2, scales="free_y")

# combine
pp = gridExtra::arrangeGrob(p1, p2, ncol=2, widths=c(1.1, 1))
pp = ggpubr::as_ggplot(pp) +
  cowplot::draw_plot_label(label = c("A", "B", "C", "D"), fontface = "bold", size = 18, x = c(0.035, 0.035, 0.53, 0.53), y = c(1, 0.535, 1, 0.535))
ggsave(pp, filename="./output/figures/nature_ms3_jan2019/Output_CommunityTier2Mammals_meaneffortmodels.png",
       device="png", width=7, height=7, units="in", dpi=1200, scale=0.9)









