
### Gibb et al, 'Land use has global and systematic effects on local zoonotic host communities"
# Script 16: Reads and summarises output from research effort bootstrap of community models (both all PREDICTS and mammal zoonotic reservoirs)
# For n models within each bootstrap, extracts posterior samples and calculates summary statistics of overall shared posterior density
# Shows that 500 models is sufficient to approximate CIs
# Saves results, then plots combined figures of parameter estimates and quantiles for manuscript

#setwd("/Users/rorygibb/Desktop")
# setwd("/Volumes/ritd-ag-project-rd00nx-rjgib11/chapter2_predicts/predicts_analysis_201901/")
#setwd("Y:/chapter2_predicts/predicts_analysis_201901/")


## --- dependencies
library(ggplot2)
library(dplyr)


## --- housekeeping

# directory where bootstrap outputs are stored 
files_dir = "./output/model_outputs/dec_2019/effort_bootstrap/bootstrap_output/"
#files_dir = "./output/model_outputs/dec_2019/new_re2/bootstrap_output/"
ff = list.files(files_dir, pattern=".R")

# directory for saving
save_dir = "./output/model_outputs/dec_2019/effort_bootstrap/"
#save_dir = "./output/model_outputs/dec_2019/new_re2/"



### ---- 1. Process and summarise bootstrap outputs ----

# get info about bootstraps
ff_doc = do.call(rbind.data.frame, strsplit(ff, "_"))
names(ff_doc) = c("tier", "model", "id", "num_it", "ff")
ff_doc$tier_model = paste(ifelse(ff_doc$tier == "tier1all", 1, 2), ff_doc$model, sep="_")
ff_doc$loc = paste(files_dir, ff, sep="")

# function to get all bootstrapped samples from all models
getSamplesBootstrap = function(model){
  
  # subset data frame
  print(paste("Summarising bootstrap for model:", model, sep=" "))
  mods = ff_doc[ ff_doc$tier_model == model, ]
  mods = mods[ sample(1:nrow(mods), 4), ] #1000 models - could be more
  
  # get samples
  print("Getting samples...")
  samples = data.frame()
  for(i in mods$loc){
    load(i)
    getSamp = function(x){ 
      resx = result[[x]]$samples_fixed 
      resx$id = sample(1:10^9, 1)
      resx$spp_imputed = result[[x]]$species_imputed
      resx
      }
    ss = do.call(rbind.data.frame, lapply(1:length(result), getSamp))
    samples = rbind(ss, samples)
  }
  
  # rename and remove params
  names(samples)[ grep("Intercept", names(samples)) ] = "Intercept"
  names(samples)[ grep("Primary_Substantial", names(samples)) ] = "Primary_Substantial"
  names(samples)[ grep("Secondary_Minimal", names(samples)) ] = "Secondary_Minimal"
  names(samples)[ grep("Secondary_Substantial", names(samples)) ] = "Secondary_Substantial"
  names(samples)[ grep("Managed_Minimal", names(samples)) ] = "Managed_Minimal"
  names(samples)[ grep("Managed_Substantial", names(samples)) ] = "Managed_Substantial"
  names(samples)[ grep("Managed:", names(samples)) ] = "Managed"
  names(samples)[ grep("Urban_Minimal", names(samples)) ] = "Urban_Minimal"
  names(samples)[ grep("Urban_Substantial", names(samples)) ] = "Urban_Substantial"
  names(samples)[ grep("Urban:", names(samples)) ] = "Urban"
  names(samples)[ grep("logTotalSR", names(samples)) ] = "logTotalSR"
  names(samples)[ grep("logTotalAbund", names(samples)) ] = "logTotalAbund"
  names(samples)[ grep("LogHumanDensity", names(samples)) ] = "LogHumanDensity"
  samples = samples[ which(names(samples) %in% c("Intercept", "Primary_Substantial", "Secondary_Minimal",
                                                 "Secondary_Substantial", "Managed_Minimal", "Managed_Substantial",
                                                 "Managed", "Urban_Minimal", "Urban_Substantial", "Urban", 
                                                 "logTotalSR", "logTotalAbund", "LogHumanDensity", "id", "spp_imputed"))]
  
  # summarise for 250, 500 and 1000 iterations
  iterations = c(250, 500, 1000)
  result = data.frame()
  for(it in iterations){
    print(paste("Summarising for", it, "models...", sep=" "))
    samplesx = samples[ samples$id %in% sample(unique(samples$id), it, replace=F), ] # n random models
    resx = data.frame(
      median = apply(samplesx, 2, median),
      lower_95 = apply(samplesx, 2, quantile, 0.025), # 95% interval
      lower_90 = apply(samplesx, 2, quantile, 0.05), # 90% interval
      lower_67 = apply(samplesx, 2, quantile, 0.167), # 67% interval
      upper_67 = apply(samplesx, 2, quantile, 0.833),
      upper_90 = apply(samplesx, 2, quantile, 0.95),
      upper_95 = apply(samplesx, 2, quantile, 0.975)
    )
    resx$param = row.names(resx)
    resx$model = model
    resx$tier = substr(model, 1, 1)
    resx$num_it = it
    resx = resx[ !resx$param %in% c("id", "spp_imputed"), ] # remove non param rows
    resx$mean_spp_imputed = mean(samplesx$spp_imputed[ !duplicated(samplesx$id) ])
    result = rbind(result, resx)
    row.names(result) = c()
  }
  
  # return df of result
  print("Completed")
  return(result)
}

# get and summarise samples for all models
mods = as.vector(unique(ff_doc$tier_model))
res1 = do.call(rbind.data.frame, lapply(mods, getSamplesBootstrap))

# save result
write.csv(res1, paste(save_dir, "summary_allbootstrapmodels_20200109.csv", sep=""))



### ------- 2. Visualise results -------

## data
#df = res1
df = read.csv(paste(save_dir, "summary_allbootstrapmodels_20200109.csv", sep=""), stringsAsFactors = FALSE)


## setup for plotting:

# plot for 1000 models per run
num_viz = 1000
df = df[ df$num_it == num_viz, ]

# sort params 
df = df[ , c("lower_95", "lower_90", "lower_67", "median", "upper_67", "upper_90", "upper_95", "param", "model", "tier")]
df$param[ df$param == "Intercept" ] = "Primary_Minimal"
df = df[ ! df$param %in% c("MeanEffort", "CWMeanEffort", "LogHumanDensity", "logTotalAbund"), ]
df[ df$param == "Primary_Minimal", 1:7] = 0 # make intercept 0
df[ , 1:7 ] = 100 * (exp(df[ 1:7]) - 1) # rescale, centre around 0, make %

# sort param and model names
df$model = unlist(lapply(strsplit(df$model, "_"), "[[", 2))
df$div_metric = ifelse(grepl("Abund", df$model), "Abundance", "SR")
#df$div_metric = factor(df$div_metric, levels=c("SR", "Abundance"), ordered=TRUE)
df$model[ grep("Prop", df$model) ] = "Prop"
df$model[ grep("Host", df$model) ] = ifelse(grepl("Non", df$model[ grep("Host", df$model) ]), "Non-host", "Host")
dm2 = data.frame(model = c("Host", "Host", "Non-host", "Non-host", "Prop", "Prop"), 
                 div_metric=rep(c("SR", "Abundance"),3), 
                 div_metric2 = c(rep(c("Species richness", "Total abundance"), 2), c("Host proportion of richness", "Host proportion of total abundance")))
df = left_join(df, dm2)
df$div_metric2 = factor(df$div_metric2, levels=dm2$div_metric2[c(1, 2, 5, 6)], ordered=T)

# sort param names
df$param[ df$param %in% c("Managed", "Urban")] = paste(df$param[ df$param %in% c("Managed", "Urban")], "_Both", sep="")
df$LU = unlist(lapply(strsplit(df$param, "_"), "[[", 1))
df$Intensity = unlist(lapply(strsplit(df$param, "_"), "[[", 2))

# order factors
df$LU = factor(df$LU, levels=c("Primary", "Secondary", "Managed", "Urban"), ordered=T)
df$foo = paste(df$Intensity, df$model, sep="_")

# colours for plotting
nonhostcol = "#018571"
hostcol = "#8c510a"

# legend text size
legend_text_size = 7
point_size = 2



# ------- Plot for Tier 1 (all hosts) -------

# 1. Host-non host
dfx = df[ df$tier == 1 & df$model %in% c("Host", "Non-host"), ]
p1 = ggplot(dfx, aes(x=LU, y=median, split=Intensity, col=model)) + 
  geom_point(position=position_dodge(width=0.6), aes(pch=Intensity), size=point_size) +
  #geom_errorbar(aes(ymin=lower_95, ymax=upper_95), position=position_dodge(width=0.6), width=0.2) + 
 # geom_errorbar(aes(ymin=lower_90, ymax=upper_90), position=position_dodge(width=0.6), width=0.2, alpha=0.7) + 
  geom_errorbar(aes(ymin=lower_95, ymax=upper_95), position=position_dodge(width=0.6), width=0.2, lwd=0.3, alpha=1, show.legend = FALSE) + 
  geom_linerange(aes(ymin=lower_67, ymax=upper_67), position=position_dodge(width=0.6), lwd=0.8, show.legend = FALSE) + 
  theme_bw() + 
  xlab("") + 
  ylab("% difference from primary minimal") +
  geom_hline(yintercept=0, lty=2) +
  theme(legend.background = element_rect(size=0.2, linetype="solid", colour=NA, fill=NA),
        # axis.text = element_text(size=15),
        panel.grid.minor = element_line(colour="grey93", size=0.2), 
        panel.grid.major = element_line(colour="grey88", size=0.3),
        # axis.title.y = element_text(size=15),
        legend.position=c(0.4, 0.95),
        legend.text = element_text(size=legend_text_size),
        legend.title=element_blank(),
        legend.box="horizontal",
        strip.background = element_rect(fill="grey90")) + 
  scale_color_manual(values=c("Host"=hostcol, 
                              "Non-host"=nonhostcol),
                     name="Model") + 
  scale_shape_manual(name="Use intensity", values=c("Minimal"=16, "Substantial"=17), labels=c("Minimal use", "Substantial use")) +
  facet_wrap(~div_metric2, nrow=2, scales="free_y")

# proportion
dfy = df[ df$tier == 1 & df$model == "Prop", ]
p2 = ggplot(dfy, aes(x=LU, y=median, split=Intensity)) + 
  geom_point(position=position_dodge(width=0.6), aes(pch=Intensity), size=point_size, col=hostcol) +
  #geom_errorbar(aes(ymin=lower_95, ymax=upper_95), position=position_dodge(width=0.6), width=0.2) + 
  #geom_errorbar(aes(ymin=lower_90, ymax=upper_90), position=position_dodge(width=0.6), width=0.2, alpha=0.7, col=hostcol) + 
  geom_errorbar(aes(ymin=lower_95, ymax=upper_95), position=position_dodge(width=0.6), width=0.1, lwd=0.3, alpha=1, col=hostcol, show.legend = FALSE) + 
  geom_linerange(aes(ymin=lower_67, ymax=upper_67), position=position_dodge(width=0.6), lwd=0.8, col=hostcol, show.legend = FALSE) + 
  theme_bw() + 
  xlab("") + 
  ylab("% difference from primary minimal") +
  geom_hline(yintercept=0, lty=2) +
  theme(legend.background = element_rect(size=0.2, linetype="solid", colour=NA, fill=NA),
        # axis.text = element_text(size=15),
        panel.grid.minor = element_line(colour="grey93", size=0.2), 
        panel.grid.major = element_line(colour="grey88", size=0.3),
        # axis.title.y = element_text(size=15),
        #legend.position=c(0.42, 0.91),
        legend.position="none",
        # legend.text = element_text(size=12),
        legend.title=element_blank(),
        legend.box="horizontal",
        axis.title.y = element_blank(),
        strip.background = element_rect(fill="grey90")) + 
  scale_shape_manual(name="Use intensity", values=c("Minimal"=16, "Substantial"=17), labels=c("Minimal use", "Substantial use")) +
  facet_wrap(~div_metric2, nrow=2, scales="free_y")

# combine
pp = gridExtra::arrangeGrob(p1, p2, ncol=2, widths=c(1.1, 1))
pp = ggpubr::as_ggplot(pp) +
  cowplot::draw_plot_label(label = c("A", "B", "C", "D"), fontface = "bold", size = 18, x = c(0.035, 0.035, 0.53, 0.53), y = c(1, 0.535, 1, 0.535))
ggsave(pp, filename="./output/figures/nature_ms3_jan2019/Figure2_CommunityTier1_bootstrap.png",
       device="png", width=7, height=7, units="in", dpi=1200, scale=0.9)



# ------- Plot for Tier 2 (mammal reservoirs) -------


# 1. Host-non host
dfx = df[ df$tier == 2 & df$model %in% c("Host", "Non-host"), ]
dfx$Intensity = factor(dfx$Intensity, levels=c("Minimal", "Substantial", "Both"), ordered=T)

# exclude urban effects for visualising because highly infated CIs (can include)
dfx = dfx[ dfx$LU != "Urban", ] 

p1 = ggplot(dfx, aes(x=LU, y=median, split=Intensity, col=model)) + 
  geom_point(position=position_dodge(width=0.6), aes(pch=Intensity), size=point_size, stroke=1.2) +
  #geom_errorbar(aes(ymin=lower_95, ymax=upper_95), position=position_dodge(width=0.6), width=0.2) + 
  geom_errorbar(aes(ymin=lower_95, ymax=upper_95), position=position_dodge(width=0.6), width=0.2, lwd=0.3, alpha=1, show.legend = FALSE) + 
  geom_linerange(aes(ymin=lower_67, ymax=upper_67), position=position_dodge(width=0.6), lwd=0.8, show.legend = FALSE) + 
  theme_bw() + 
  xlab("") + 
  ylab("% difference from primary minimal") +
  geom_hline(yintercept=0, lty=2) +
  theme(legend.background = element_rect(size=0.2, linetype="solid", colour=NA, fill=NA),
        # axis.text = element_text(size=15),
        panel.grid.minor = element_line(colour="grey93", size=0.2), 
        panel.grid.major = element_line(colour="grey88", size=0.3),
        # axis.title.y = element_text(size=15),
        legend.position=c(0.4, 0.93),
        legend.text = element_text(size=legend_text_size),
        legend.title=element_blank(),
        legend.box="horizontal",
        strip.background = element_rect(fill="grey90")) + 
  scale_color_manual(values=c("Host"=hostcol, 
                              "Non-host"=nonhostcol),
                     name="Model") + 
  scale_shape_manual(name="Use intensity", values=c("Minimal"=16, "Substantial"=17, "Both"=4), labels=c("Minimal use", "Substantial use", "Both")) +
  facet_wrap(~div_metric2, nrow=2) +
  ylim(-50, 100)

# proportion
dfy = df[ df$tier == 2 & df$model == "Prop", ]
dfy$Intensity = factor(dfy$Intensity, levels=c("Minimal", "Substantial", "Both"), ordered=T)
dfy = dfy[ dfy$LU != "Urban", ] 

p2 = ggplot(dfy, aes(x=LU, y=median, split=Intensity)) + 
  geom_point(position=position_dodge(width=0.6), aes(pch=Intensity), size=point_size, col=hostcol, stroke=1.2) +
  #geom_errorbar(aes(ymin=lower_95, ymax=upper_95), position=position_dodge(width=0.6), width=0.15, col=hostcol) + 
  geom_errorbar(aes(ymin=lower_95, ymax=upper_95), position=position_dodge(width=0.6), width=0.1, lwd=0.3, alpha=1, col=hostcol, show.legend = FALSE) + 
  geom_linerange(aes(ymin=lower_67, ymax=upper_67), position=position_dodge(width=0.6), lwd=0.8, col=hostcol, show.legend = FALSE) + 
  theme_bw() + 
  xlab("") + 
  ylab("% difference from primary minimal") +
  geom_hline(yintercept=0, lty=2) +
  theme(legend.background = element_rect(size=0.2, linetype="solid", colour=NA, fill=NA),
        # axis.text = element_text(size=15),
        panel.grid.minor = element_line(colour="grey93", size=0.2), 
        panel.grid.major = element_line(colour="grey88", size=0.3),
        # axis.title.y = element_text(size=15),
        #legend.position=c(0.42, 0.91),
        legend.position="none",
        # legend.text = element_text(size=12),
        legend.title=element_blank(),
        legend.box="horizontal",
        axis.title.y = element_blank(),
        strip.background = element_rect(fill="grey90")) + 
  scale_shape_manual(name="Use intensity", values=c("Minimal"=16, "Substantial"=17, "Both"=4), labels=c("Minimal use", "Substantial use", "Both")) +
  facet_wrap(~div_metric2, nrow=2) +
  ylim(-50, 100)

# combine
pp = gridExtra::arrangeGrob(p1, p2, ncol=2, widths=c(1.1, 1))
pp = ggpubr::as_ggplot(pp) +
  cowplot::draw_plot_label(label = c("A", "B", "C", "D"), fontface = "bold", size = 18, x = c(0.035, 0.035, 0.53, 0.53), y = c(1, 0.535, 1, 0.535))
ggsave(pp, filename="./output/figures/nature_ms3_jan2019/ExtendedData_CommunityTier2Mammals_bootstrap_urbexcl.png",
       device="png", width=7, height=7, units="in", dpi=1200, scale=0.9)



# -------------- Create summary table of numeric estimates for SuppMat -------------

# 
createSummaryTable = function(tier){
  dfx = df[ df$tier == tier & df$num_it == 1000, ]
  dfx$Median = round(dfx$median, 3)
  dfx$`Quantile_67` = paste(round(dfx$lower_67, 3), round(dfx$upper_67, 3), sep=", ")
  dfx$`Quantile_95` = paste(round(dfx$lower_95, 3), round(dfx$upper_95, 3), sep=", ")
  dfx$Parameter = dfx$param
  dfx$Model = unlist(lapply(strsplit(dfx$model, "_"), "[[", 2))
  dfx$Dataset = ifelse(tier == 1, "Full dataset", "Mammal reservoirs")
  dfx = dfx[ , c("Dataset", "Model", "Parameter", "Median", "Quantile_67", "Quantile_95") ]
  dfx$Model = factor(dfx$Model, levels=c("HostSR", "NonHostSR", "PropHostSR", "HostAbundance", "NonHostAbundance", "PropHostAbundance"), ordered=TRUE)
  if(tier == 1){
    dfx$Parameter = factor(dfx$Parameter, levels=c("Intercept", "Primary_Substantial", 
                                                   "Secondary_Minimal", "Secondary_Substantial",
                                                   "Managed_Minimal", "Managed_Substantial",
                                                   "Urban_Minimal", "Urban_Substantial", 
                                                   "LogHumanDensity", "logTotalAbund"), ordered=TRUE)
  } else{
    dfx$Parameter = factor(dfx$Parameter, levels=c("Intercept", "Primary_Substantial", 
                                                   "Secondary_Minimal", "Secondary_Substantial",
                                                   "Managed","Urban", "logTotalAbund"), ordered=TRUE)
    
  }
  dfx = dfx[ order(dfx$Model, dfx$Parameter), ]
  dfx
}
sumtable = do.call(rbind.data.frame, lapply(1:2, createSummaryTable))
write.csv(sumtable, "./output/tables/SI_CommunityModels_parameterdistributions_bootstrap.csv", row.names=FALSE)
