

### Gibb et al, 'Land use has global and systematic effects on local zoonotic host communities"
# Script 17: Reads, processes and plots output from random and geographical cross-validation of community models
# Each holdout group has n bootstrapped models with host imputation based on research effort
# Script reads and summarises quantiles across all bootstrapped models within each hold-out group, and across all-hold out groups
# Plots sensitivity of land use parameters to cross-validation

# --- working directory
#setwd("/Users/rorygibb/Desktop")
# setwd("/Volumes/ritd-ag-project-rd00nx-rjgib11/chapter2_predicts/predicts_analysis_201901/")
# setwd("Y:/chapter2_predicts/predicts_analysis_201901/")


## --- dependencies
library(ggplot2)


## --- housekeeping

# directory where bootstrap outputs are stored 
files_dir = "./output/model_outputs/dec_2019/crossval_bootstrap/crossvalidation_tier1/"
ff = list.files(files_dir)

# directory for saving
save_dir = "./output/model_outputs/dec_2019/crossval_bootstrap/"


### ---- 1. Process and summarise bootstrap outputs ----

# get info about bootstraps
ff_doc = do.call(rbind.data.frame, strsplit(ff, "_"))
names(ff_doc) = c("type", "model", "id", "num_it", "ff")
ff_doc$type_model = paste(ff_doc$type, ff_doc$model, sep="_")
ff_doc$loc = paste(files_dir, ff, sep="")

# function to get all bootstrapped samples from all models
getSamplesCrossval = function(model){
  
  # start time
  starttime = Sys.time()
  
  # subset data frame
  print(paste("Summarising crossvalidation for model:", model, sep=" "))
  mods = ff_doc[ ff_doc$type_model == model, ]

  # get samples
  print("Getting samples:")
  samples = data.frame()
  for(i in mods$loc){
    cat(" loading files...")
    load(i)
    getSamp = function(x){ 
      resx = result[[x]]$samples_fixed 
      resx$id = sample(1:10^9, 1)
      resx$spp_imputed = result[[x]]$species_imputed
      resx$geo_excl = result[[x]]$geo_excl
      resx$random_excl = result[[x]]$k_excl
      resx$sites_excluded = result[[x]]$sites_excluded
      resx
    }
    ss = do.call(rbind.data.frame, lapply(1:length(result), getSamp))
    samples = rbind(ss, samples)
  }
  
  # rename and remove params
  names(samples)[ grep("Intercept", names(samples)) ] = "Intercept"
  names(samples)[ grep("Primary_Substantial", names(samples)) ] = "Primary_Substantial"
  names(samples)[ grep("Secondary_Minimal", names(samples)) ] = "Secondary_Minimal"
  names(samples)[ grep("Secondary_Substantial", names(samples)) ] = "Secondary_Substantial"
  names(samples)[ grep("Managed_Minimal", names(samples)) ] = "Managed_Minimal"
  names(samples)[ grep("Managed_Substantial", names(samples)) ] = "Managed_Substantial"
  names(samples)[ grep("Managed:", names(samples)) ] = "Managed"
  names(samples)[ grep("Urban_Minimal", names(samples)) ] = "Urban_Minimal"
  names(samples)[ grep("Urban_Substantial", names(samples)) ] = "Urban_Substantial"
  names(samples)[ grep("Urban:", names(samples)) ] = "Urban"
  names(samples)[ grep("logTotalSR", names(samples)) ] = "logTotalSR"
  names(samples)[ grep("logTotalAbund", names(samples)) ] = "logTotalAbund"
  names(samples)[ grep("LogHumanDensity", names(samples)) ] = "LogHumanDensity"
  samples = samples[ which(names(samples) %in% c("Intercept", "Primary_Substantial", "Secondary_Minimal",
                                                 "Secondary_Substantial", "Managed_Minimal", "Managed_Substantial",
                                                 "Managed", "Urban_Minimal", "Urban_Substantial", "Urban", 
                                                 "logTotalSR", "logTotalAbund", "LogHumanDensity", "id", "spp_imputed",
                                                 "geo_excl", "random_excl", "sites_excluded"))]
  
  # create summary table of models
  all_models = samples[ !duplicated(samples$id), ]
  
  # set up cols so generalisable for both geo-random
  if("geo_excl" %in% names(all_models)){ 
    all_models$excl = all_models$geo_excl
    samples$excl = samples$geo_excl }
  if("random_excl" %in% names(all_models)){ 
    all_models$excl = all_models$random_excl
    samples$excl = samples$random_excl }
  
  # num bootstrapped models for each holdout group
  num_bootstraps = 500
  
  # holdout groups
  holdout = unique(all_models$excl)
  
  # 1. Summarise for each individual holdout model type
  result = data.frame()
  for(h in holdout){
    print(paste("Summarising for", h, "...", sep=" "))
    amh = all_models[ all_models$excl == h, ]
    samplesx = samples[ samples$id %in% sample(unique(amh$id), num_bootstraps, replace=F), ] # n random models
    samplesx = samplesx[ , -which(names(samplesx) %in% c("geo_excl", "id", "random_excl", "excl")) ]
    resx = data.frame(
      median = apply(samplesx, 2, median),
      lower_95 = apply(samplesx, 2, quantile, 0.025), # 95% interval
      lower_90 = apply(samplesx, 2, quantile, 0.05), # 90% interval
      lower_67 = apply(samplesx, 2, quantile, 0.167), # 67% interval
      upper_67 = apply(samplesx, 2, quantile, 0.833),
      upper_90 = apply(samplesx, 2, quantile, 0.95),
      upper_95 = apply(samplesx, 2, quantile, 0.975)
    )
    resx$param = row.names(resx)
    resx$model = model
    resx$type = substr(model, 1, 3)
    resx$holdout = h
    resx$num_it = num_bootstraps
    resx$sites_excluded = mean(samplesx$sites_excluded)
    resx$mean_spp_imputed = mean(samplesx$spp_imputed)
    resx = resx[ !resx$param %in% c("id", "spp_imputed", "sites_excluded"), ] # remove non param rows
    result = rbind(result, resx)
    row.names(result) = c()
  }
  rm(samplesx)
  
  ## 2. Summarise across all holdout types
  print("Summarising across all heldout models...")
  
  # sample n models within each holdout variable
  randomModels_excl = function(mt){
    rm1 = all_models[ all_models$excl == mt, ]
    rm1 = rm1[ rm1$id %in% sample(unique(rm1$id), num_bootstraps, replace=F), ]
    return(rm1[ , c("id", "excl") ])
  }
  ama = do.call(rbind.data.frame, lapply(unique(all_models$excl), randomModels_excl))
  
  # subset samples to just what's needed
  samples = samples[ samples$id %in% ama$id, ]
  
  # summarise (1m samples from across all models; test shows this approximates to 3dp so sufficiently accurate)
  samplesx = samples[ , -which(names(samples) %in% c("geo_excl", "id", "random_excl", "excl")) ]
  samplesx = samplesx[ sample(1:nrow(samplesx), 10^6, replace=FALSE), ]
  resx = data.frame(
    median = apply(samplesx, 2, median),
    lower_95 = apply(samplesx, 2, quantile, 0.025), # 95% interval
    lower_90 = apply(samplesx, 2, quantile, 0.05), # 90% interval
    lower_67 = apply(samplesx, 2, quantile, 0.167), # 67% interval
    upper_67 = apply(samplesx, 2, quantile, 0.833),
    upper_90 = apply(samplesx, 2, quantile, 0.95),
    upper_95 = apply(samplesx, 2, quantile, 0.975)
  )
  resx$param = row.names(resx)
  resx$model = model
  resx$type = substr(model, 1, 3)
  resx$holdout = "all_holdout"
  resx$num_it = num_bootstraps
  resx$sites_excluded = mean(samplesx$sites_excluded)
  resx$mean_spp_imputed = mean(samplesx$spp_imputed)
  resx = resx[ !resx$param %in% c("id", "spp_imputed", "sites_excluded"), ] # remove non param rows
  result = rbind(result, resx)
  row.names(result) = c()
  
  # report processing time
  endtime = Sys.time()
  timediff = endtime - starttime
  print(timediff)
  
  # return df of result
  print("Saving and returning...")
  write.csv(result, paste(save_dir, result$model[1], ".csv", sep=""), row.names=FALSE)
  return(result)
}

#test
# test = getSamplesCrossval(unique(ff_doc$type_model)[5])

# get and summarise samples for all models
type_models = as.vector(unique(ff_doc$type_model))
res1 = do.call(rbind.data.frame, lapply(type_models, getSamplesCrossval))
write.csv(res1, paste(save_dir, "processedcrossval_all_re2_20200120.csv", sep=""), row.names=FALSE)




# ------------ visualise across all models for each cross-validation approach --------------

# data
res1 = read.csv(paste(save_dir, "processedcrossval_all_20200113.csv", sep=""), stringsAsFactors = FALSE)

# sort out columns
viz = res1[ res1$holdout != "all_holdout", ]
viz$holdout[ viz$type=="ran" ] = paste("Group", viz$holdout[ viz$type=="ran" ], sep=" ")
viz$holdout2 =  paste(viz$holdout, " (", viz$sites_excluded, ")", sep="")
viz$type = ifelse(viz$type == "geo", "Biome", "Study kfold")
viz$model = unlist(lapply(strsplit(viz$model, "_"), "[[", 2))
viz$model = factor(viz$model, levels=c("HostSR", "NonHostSR", "PropHostSR", "HostAbundance", "NonHostAbundance", "PropHostAbundance"), ordered=T)

# rename and restructure params
viz = viz[ !viz$param %in% c("Intercept", "LogHumanDensity", "logTotalAbund"), ]
viz$LU = unlist(lapply(strsplit(viz$param, "_"), "[", 1))
viz$Intensity = unlist(lapply(strsplit(viz$param, "_"), "[", 2))
viz$LU2 = paste(substr(viz$LU, 1, 3), "\n(", substr(viz$Intensity, 1, 3), ")", sep="")

# order factors and create dummy row for plotting
viz$LU = factor(viz$LU, levels=c("Primary", "Secondary", "Managed", "Urban"), ordered=T)
viz$LU2 = factor(viz$LU2, levels=unique(viz$LU2)[ c(3:5, 1:2, 6:7) ], ordered=T)
viz$type = factor(viz$type, levels=c("Study kfold", "Biome"), ordered=T)
viz$foo = paste(viz$Intensity, viz$type, sep="_")

# transform param estimates
viz[ , 1:7 ] = 100* (exp(viz[ , 1:7 ]) - 1)

# create param2
param2func = function(x){
  sss = strsplit(x, "_")
  return(paste(sss[[1]][1], "\n (", sss[[1]][2], ")", sep=""))
}
# param2func = function(x){ 
#   sss = strsplit(x, "_")
#   return(paste(sss[[1]][1], " (", sss[[1]][2], ")", sep=""))
# }
viz$param2 = unlist(lapply(viz$param, param2func))
viz$param2 = factor(viz$param2, levels=unique(viz$param2)[c(3, 4, 5, 1, 2, 6, 7)], ordered=T)

## remove urban effects for abundance models for Temperate Broadleaf for visualisation
viz = viz[ -which(viz$param %in% c("Urban_Minimal", "Urban_Substantial") & viz$holdout == "Temperate Broadleaf & Mixed Forests" & viz$model %in% c("HostAbundance",  "NonHostAbundance", "PropHostAbundance")), ]

# sizes for relevant text/legends
strip_text_size = 12
legend_text_size = 9.5
xaxis_text_size = 10

# plot geographical crossval
vx = viz[ viz$type == "Biome", ]
geo_plot = ggplot(vx, aes(x=LU2, y=median, col=holdout2)) +
  geom_linerange(aes(ymin=lower_95, ymax=upper_95), position=position_dodge(width=0.0), alpha=0.6) +
  geom_point(position=position_dodge(width=0.0), size=2, stroke=1.1, alpha=0.6) +
  theme_bw() +
  geom_hline(yintercept=1, lty=2) +
  ylab("Parameter estimate") +
  facet_wrap(~model, nrow=2, ncol=3, scales="free_y") +
  theme(legend.position = "bottom",
        legend.background = element_rect(fill=NA, color="grey25"),
        axis.title.x = element_blank(),
        axis.title.y = element_text(size=strip_text_size),
        axis.text = element_text(size = xaxis_text_size),
        legend.title = element_blank(),
        strip.background = element_rect(fill="grey90"),
        strip.text = element_text(size=strip_text_size),
        legend.text = element_text(size=legend_text_size)) +
  guides(col = guide_legend(nrow = 5)) +
  ylab("Difference from primary minimal (%)")

# plot random crossval
vy = viz[ viz$type == "Study kfold", ]
ran_plot = ggplot(vy, aes(x=LU2, y=median, col=holdout2)) +
  geom_linerange(aes(ymin=lower_95, ymax=upper_95), position=position_dodge(width=0.0), alpha=0.6) +
  geom_point(position=position_dodge(width=0.0), size=2, stroke=1.1, alpha=0.6) +
  theme_bw() +
  geom_hline(yintercept=1, lty=2) +
  ylab("Parameter estimate") +
  facet_wrap(~model, nrow=2, ncol=3, scales="free_y") +
  theme(legend.position = "bottom",
        legend.background = element_rect(fill=NA, color="grey25"),
        axis.title.x = element_blank(),
        axis.title.y = element_text(size=strip_text_size),
        axis.text = element_text(size = xaxis_text_size),
        legend.title = element_blank(),
        strip.background = element_rect(fill="grey90"),
        strip.text = element_text(size=strip_text_size),
        legend.text = element_text(size=legend_text_size)) +
  ylab("Difference from primary minimal (%)")

# combine and save
dp = gridExtra::arrangeGrob(ran_plot, geo_plot, ncol=1, nrow=2)
dp = ggpubr::as_ggplot(dp) +
  cowplot::draw_plot_label(label = c("A", "B"), fontface = "bold", size = 26, x = c(0, 0), y = c(1, 0.505))
ggsave(dp, filename="./output/figures/nature_ms3_jan2019/ExtendedData_Tier1CrossValidation_20200113.png",
       device="png", width=10, height=13, units="in", dpi=600)

