
### Gibb et al, 'Land use has global and systematic effects on local zoonotic host communities"
# Script 18: Fits n bootstrap iterations of species-level models fitted to different taxonomic subsets of mammals and birds
# For each iteration, transitions non-hosts to host status with p = FNR
# then fits separate occurrence and zero-truncted abundance models with same hosts, extracts posterior samples and saves
# Outputs analysed in later scripts


#setwd("/Volumes/ritd-ag-project-rd00nx-rjgib11/chapter2_predicts/predicts_analysis_201901/")
#setwd("Y:/chapter2_predicts/predicts_analysis_201901/")
#setwd("/scratch/scratch/ucbtgib/predicts")

# dependencies
library(lme4)
library(ggplot2)
library(nlme)
library(plyr)
library(dismo)
library(geosphere)
library(spdep)
library(rgdal)
library(INLA)
library(viridis)
library(raster)
library(RColorBrewer)
library(KernSmooth)
library(dplyr)
library(reshape2)

# colours for plotting
nonhostcol = "#018571"
hostcol = "#8c510a"

# directory for saving outputs
save_dir = "./output/model_outputs/dec_2019/order_hurdle/"
#save_dir = "./species_hurdle/"
unique_id = sample(1:10^8, 1)



# ------ Aggregate land use categories (updated 16/02/18) ------
#' @param df PREDICTS dataframe

prepareLandUse = function(df){
  
  # land use mappings
  plu = data.frame(Predominant_land_use = c("Primary vegetation","Young secondary vegetation","Intermediate secondary vegetation","Mature secondary vegetation", "Secondary vegetation (indeterminate age)", "Plantation forest","Pasture","Cropland","Urban"),
                   PLU2 = c("1_Primary", "Secondary","Secondary","Secondary", "Secondary", "Plantation forest", "Pasture", "Cropland", "Urban"),
                   LandUseType = c("1_Primary","Secondary","Secondary","Secondary","Secondary","Managed","Managed","Managed","Urban"),
                   stringsAsFactors = F)
  
  # merge and reshuffle columns
  df$foo = 1:nrow(df)
  df1 = merge(df, plu, by="Predominant_land_use", all.x=T)
  df1 = df1[ , c(2:ncol(df1), 1) ]
  df1 = df1[ order(df1$foo), ]
  
  # remove 'use' from intensity
  df1$Use_intensity = gsub(" use","", as.vector(df1$Use_intensity) )
  
  # combine use intensities into three states (minimal, substantial, cannot decide)
  df1$UseIntensity = ifelse(df1$Use_intensity == "Minimal", "Minimal", "Substantial")
  df1$UseIntensity[ df1$Use_intensity == "Cannot decide" ] = "Cannot decide"
  
  # combine with land use type for composite variable for modelling
  df1$LU_UI = paste(df1$LandUseType, df1$UseIntensity, sep="_")
  df1$LU_UI[ df1$LU_UI == "1_Primary_Substantial" ] = "Primary_Substantial"
  
  return( as.data.frame(df1, stringsAsFactors=F) )
}





# ------------ prepare and summarise data --------

# load PREDICTS data
load(file="./data/predicts_processed/predicts_pathogens_researcheffort.R")
#load(file="./predicts_pathogens_researcheffort.R")

# tier 1 strictness: cargo carrier for non-mammals, mammals serological and any human infectivity
# dd1$HumanInfective_Any[ dd1$Class == "Mammalia" ] = dd1$Sero_HumanInfective_Any[ dd1$Class == "Mammalia" ]
# dd1$IsZoonotic_Strict[ dd1$Class == "Mammalia" ] = dd1$Sero_IsZoonotic_Strict[ dd1$Class == "Mammalia" ]

# tier 2 strictness: mammals pathogen isolation/direct detection
dd1$HumanInfective_Any[ dd1$Class == "Mammalia" ] = dd1$Isolation_HumanInfective_Any[ dd1$Class == "Mammalia" ]
dd1$IsZoonotic_Strict[ dd1$Class == "Mammalia" ] = dd1$Isolation_IsZoonotic_Strict[ dd1$Class == "Mammalia" ]

# prepare land use categories
dd1 = prepareLandUse(dd1)

# print details (zoonotic hosts + strict mammal reservoir)
print(paste("Zoonotic hosts:", length(unique(dd1$bgb[ which(dd1$Class == "Mammalia" & dd1$IsZoonotic_Strict == 1)])), "mammals, ", 
            length(unique(dd1$bgb[ which(dd1$Class == "Aves" & dd1$IsZoonotic_Strict == 1)])), "birds, ", 
            length(unique(dd1$bgb[ which(dd1$Class == "Insecta" & dd1$IsZoonotic_Strict == 1)])), "insects", sep=" "))

# only keep abundance studies, and create presence (1/0) and log abundance columns for modelling
dd1 = dd1[ dd1$Diversity_metric_type == "Abundance", ]
dd1$logAbund = log(dd1$Effort_corrected_measurement+1)
dd1$Presence = ifelse( dd1$Effort_corrected_measurement > 0, 1, 0 )

# host status: zoonotic strict
dd1$IsHost = dd1$IsZoonotic_Strict




# ------- set up order for model fitting

# select Order for model (except Primates, fitted in separate script)
taxa = c("Rodentia", "Chiroptera", "Cetartiodactyla", "Carnivora", "Primates", "Passeriformes", "Psittaciformes", "Mammalia")
tt = sample(taxa[ -5 ], 1)

# subset to records of interest
if(tt == "Mammalia"){
  ddx = dd1[ dd1$Class == "Mammalia", ]
} else{
  ddx = dd1[ dd1$Order == tt, ]
}

# if mammalian, exclude urban due to low sampling
if(tt %in% taxa[1:5] | tt == "Mammalia"){ ddx = ddx[ ddx$LandUseType != "Urban", ] }

# num_iterations of bootstrap
# slow to fit for passerines/mammals so fewer
num_it = 250
if(tt == "Passeriformes"){ num_it = 1 }
if(tt == "Mammalia"){ num_it = 50 }

# save tracking output
write.csv(c(1, 2, 3), paste("./output_tracker/", unique_id, tt, num_it, "runningmodels.csv", sep="_"))




# ------ run bootstrap of hurdle model: occurrence and zero-truncated abundance ---------

# fit models separately 

# list to save results
result = vector("list", num_it)

# run each iteration
for(i in 1:num_it){
  
  # print i if running locally
  print(i)
  err = simpleError("Error in model fitting")
  
  # run to work around server GLIBC issue
  INLA:::inla.dynload.workaround()
  
  
  # -------- transfer 'non-hosts' into host category ----------
  
  # impute n non-host species 
  spp = ddx[ !duplicated(ddx$bgb), c("bgb", "FNR", "IsHost") ]
  spp$ImputeRate = spp$FNR
  spp$ImputeRate[ is.na(spp$ImputeRate) ] = 0.001
  ran_impute = function(n){ rbinom(1, 1, prob=spp$ImputeRate[n]) }
  spp$IsHost_imp = unlist(lapply(1:nrow(spp), ran_impute))
  number_species_imputed = sum(spp$IsHost_imp[ spp$IsHost == 0 ]) # calculate number imputed for saving
  spp$IsHost_imp[ spp$IsHost == 1 ] = 1
  
  # combine with PREDICTS for ith iteration
  ddi = left_join(ddx, spp)
  ddi$IsHost = ddi$IsHost_imp
  
  # prepare for modelling: nested random effect
  ddi$bgb_SS = paste(ddi$bgb, ddi$SS, sep="_")
  

  
  
  # --------- fit models --------
  
  ### 1. Occurrence model (binomial logit link) across all records
  form_occ = "Presence ~ LandUseType*IsHost + f(SS, model='iid', hyper=hyper1.iid) + f(bgb_SS, model='iid', hyper=hyper1.iid) + f(Family, model='iid', hyper=hyper1.iid)"
  if(tt == "Mammalia"){ form_occ = "Presence ~ LandUseType*IsHost + f(SS, model='iid', hyper=hyper1.iid) + f(bgb_SS, model='iid', hyper=hyper1.iid) + f(Order, model='iid', hyper=hyper1.iid) + f(Family, model='iid', hyper=hyper1.iid)" }

  formx = formula(form_occ)
  hyper1.iid = list(prec = list(prior='loggamma', param=c(0.1,0.1)))
  control.fixed1 = list(mean.intercept=0, mean = 0,
                        prec = 0.01, prec.intercept = 0.01)
  m1 = tryCatch(inla(formx, data=ddi,
                     family="binomial", 
                     control.predictor=list(compute=T),
                     control.compute=list(cpo=T, waic=T, config=TRUE),
                     control.fixed=control.fixed1), error=function(err) err)
  
  if(class(m1) != "inla"){ # skip to next iteration if model breaks
    result[[i]] = "Model fit error"
    next 
  } 
  
  ### 2. Zero-truncated abundance model (Gaussian with log transformed abund) for non-zero records
  form_ab = "logAbund ~ LandUseType*IsHost + f(SS, model='iid', hyper=hyper1.iid) + f(bgb_SS, model='iid', hyper=hyper1.iid) + f(Family, model='iid', hyper=hyper1.iid)"
  if(tt == "Mammalia"){ form_ab = "logAbund ~ LandUseType*IsHost + f(SS, model='iid', hyper=hyper1.iid) + f(bgb_SS, model='iid', hyper=hyper1.iid) + f(Order, model='iid', hyper=hyper1.iid) + f(Family, model='iid', hyper=hyper1.iid)" }
  if(tt == "Primates"){ form_ab = "logAbund ~ LU_Host + f(SS, model='iid', hyper=hyper1.iid) + f(bgb_SS, model='iid', hyper=hyper1.iid) + f(Family, model='iid', hyper=hyper1.iid)" }
  
  formy = formula(form_ab)
  hyper1.iid = list(prec = list(prior='loggamma', param=c(0.1,0.1)))
  control.fixed1 = list(mean.intercept=0, mean = 0,
                        prec = 0.01, prec.intercept = 0.01)
  control.family.gauss = list(hyper=list(prec=list(prior='loggamma', param=c(0.1, 0.1))))
  m2 = tryCatch(inla(formy, data=ddi[ ddi$Presence > 0, ],
                     family="gaussian", 
                     control.predictor=list(compute=T),
                     control.compute=list(cpo=T, waic=T, config=TRUE),
                     control.fixed=control.fixed1, 
                     control.family=control.family.gauss), error=function(err) err)
  
  if(class(m2) != "inla"){ # skip to next iteration if model breaks
    result[[i]] = "Model fit error"
    next 
  } 
  
  
  # -------- extract results from models ---------
  
  # extract posterior samples from both models
  samp1 = inla.posterior.sample(2500, m1)
  samp2 = inla.posterior.sample(2500, m2)
  
  # extract from samples
  getSampFixed = function(x, samp){ 
    ff = as.data.frame(samp[[x]]$latent)
    tomatch = paste(c("Intercept", "LandUseType", "IsHost", "LU_Host"), collapse="|")
    ff = as.data.frame(t(ff[ grep(tomatch, row.names(ff)), , drop=FALSE]))
    return(ff)
  }
  fixed1 = do.call(rbind.data.frame, lapply(1:length(samp1), getSampFixed, samp=samp1))
  fixed2 = do.call(rbind.data.frame, lapply(1:length(samp2), getSampFixed, samp=samp2))
  row.names(fixed1) = 1:nrow(fixed1)
  row.names(fixed2) = 1:nrow(fixed2)
  
  
  # -------- create summary object ----------
  
  # create summary object
  resi = list()
  resi$order = tt
  resi$nobs = nrow(ddi)
  resi$nhosts = sum(spp$IsHost_imp)
  resi$nnonhosts = sum(spp$IsHost_imp == 0)
  resi$nstudies = length(unique(ddi$SS))
  resi$nsites = length(unique(ddi$SSS))
  resi$species_imputed = number_species_imputed
  resi$occurrence_fixed = m1$summary.fixed
  resi$occurrence_hyper = m1$summary.hyperpar
  resi$occurrence_samples = fixed1
  resi$ztabund_fixed = m2$summary.fixed
  resi$ztabund_hyper = m2$summary.hyperpar
  resi$ztabund_samples = fixed2
  
  
  # -------- append to results list ----------

  result[[i]] = resi
}

filename = paste("hurdle", tt, unique_id, num_it, "bootstrapx1.R", sep="_")
save(result, file=paste(save_dir, filename, sep=""))




