
### Gibb et al, 'Land use has global and systematic effects on local zoonotic host communities"
# Script 19.1: Processes outputs of hurdle bootstrap at species-level
# Reads in sampled model objects, subsets to each bootstrap iteration
# Within each iteration, rescales models to natural (prob/linear) scale, and combines occ/ztabund models to calculate hurdle using samples
# Then lastly summarises across all iterations of bootstrap using posterior samples

setwd("Y:/chapter2_predicts/predicts_analysis_201901/")
#setwd("/scratch/scratch/ucbtgib/predicts")

# dependencies
library(lme4)
library(ggplot2)
library(nlme)
library(plyr)
library(dismo)
library(geosphere)
library(spdep)
library(rgdal)
library(INLA)
library(viridis)
library(raster)
library(RColorBrewer)
library(KernSmooth)
library(dplyr)
library(reshape2)

# colours for plotting
nonhostcol = "#018571"
hostcol = "#8c510a"


# ------------- data info --------------

# directory where bootstrap outputs are stored 
files_dir = "./output/model_outputs/dec_2019/order_hurdle/species_hurdle/"
ff = list.files(files_dir, pattern=".R")

# directory for saving
save_dir = "./output/model_outputs/dec_2019/order_hurdle/"

# get info about bootstraps
ff_doc = do.call(rbind.data.frame, strsplit(ff, "_"))
names(ff_doc) = c("mtype", "order", "id", "num_it", "ff")
ff_doc$loc = paste(files_dir, ff, sep="")



# ------------ get distribution of species imputed for each Order -----------

getSpImputedOrder = function(order, num_models=500){
  
  print(paste("Getting species impute data for:", order, sep=" "))
  mods = ff_doc[ ff_doc$order == order, ]
  if(order == "Mammalia"){ num_models = 750 }
  foo = ceiling( num_models / as.numeric(as.vector(mods$num_it[1])) ) # for number of models to load
  mods = mods[ sample(1:nrow(mods), foo), ] 
  
  spp_imp = data.frame()
  for(i in mods$loc){
    load(i)
    si = unlist(lapply(result, function(x) x$species_imputed))
    nhosts = unlist(lapply(result, function(x) x$nhosts))
    dat = data.frame(order = order, spp_imp = si, nhosts = nhosts)
    spp_imp = rbind(spp_imp, dat)
  }
  return(spp_imp)
}
oo = as.vector(unique(ff_doc$order))
impr = lapply(oo, getSpImputedOrder, num_models=500)
impr = do.call(rbind.data.frame, impr)
save(impr, file=paste(save_dir, "results_singlemodel/numimputed_allorders_bootstrap.R", sep=""))




# ----------- process and summarise for each Order -----------

# function to get all samples for n models for a particular Order
getSamplesOrder = function(order, num_models = 500){
  
  # details
  print(paste("Summarising bootstrap for:", order, sep=" "))
  mods = ff_doc[ ff_doc$order == order, ]
  if(order == "Mammalia"){ num_models = 750 }
  foo = ceiling( num_models / as.numeric(as.vector(mods$num_it[1])) ) # for number of models to load
  mods = mods[ sample(1:nrow(mods), foo), ] 
  
  # get samples
  print("Getting samples...")
  samples = data.frame()
  for(i in mods$loc){
    
    load(i)
    getSamp = function(x){ 
      
      # if 
      if(result[[x]]$nnonhosts == 0){ next } 
      
      # occurrence and abundance samples
      ox = result[[x]]$occurrence_samples
      ox$modeltype = "Occurrence"
      ztx = result[[x]]$ztabund_samples
      ztx$modeltype = "ZTAbund"
      resx = rbind(ox, ztx)
      
      # label with additional info
      resx$id = sample(1:10^9, 1)
      resx$nobs = result[[x]]$nobs
      resx$nhosts = result[[x]]$nhosts
      resx$nnonhosts = result[[x]]$nnonhosts
      resx$nnonhosts = result[[x]]$nnonhosts
      resx$nstudies = result[[x]]$nstudies
      resx$nsites = result[[x]]$nsites
      resx$spp_imputed = result[[x]]$species_imputed
      resx
    }
    ss = do.call(rbind.data.frame, lapply(1:length(result), getSamp))
    samples = rbind(ss, samples)
  }
  
  # rename and remove params
  names(samples)[ grep("Intercept", names(samples)) ] = "Intercept"
  names(samples)[ names(samples) == "LandUseTypeManaged:1" ] = "LandUseTypeManaged"
  names(samples)[ names(samples) == "LandUseTypeSecondary:1" ] = "LandUseTypeSecondary"
  names(samples)[ names(samples) == "LandUseTypeUrban:1" ] = "LandUseTypeUrban"
  names(samples)[ names(samples) == "IsHost:1" ] = "IsHost"
  names(samples)[ names(samples) == "LandUseTypeManaged:IsHost:1" ] = "LandUseTypeManaged:IsHost"
  names(samples)[ names(samples) == "LandUseTypeSecondary:IsHost:1" ] = "LandUseTypeSecondary:IsHost"
  names(samples)[ names(samples) == "LandUseTypeUrban:IsHost:1" ] = "LandUseTypeUrban:IsHost"
  
  # return samples
  samples$Order = order
  return(samples)
}

# test
# s1 = getSamplesOrder("Carnivora", 500)

# run for all orders
oo = as.vector(unique(ff_doc$order))
sampr = lapply(oo, getSamplesOrder, num_models=550)
save(sampr, file=paste(save_dir, "results_singlemodel/samples_allbootstrap_04022020.R", sep=""))

# load
load(paste(save_dir, "results_singlemodel/samples_allbootstrap_04022020.R", sep=""))

# # plot histograms
# plotHists = function(x){
#   dat = x[ x$modeltype == "Occurrence" & !duplicated(x$id), ]
#   p1 = ggplot(dat, aes(spp_imputed)) + geom_histogram(binwidth=1, color="black", fill="grey90") + ggtitle(dat$Order[1])
#   p2 = ggplot(dat, aes(nhosts)) + geom_histogram(binwidth=1, color="black", fill="grey90")
#   p3 = ggplot(dat, aes(nnonhosts)) + geom_histogram(binwidth=1, color="black", fill="grey90")
#   print(gridExtra::grid.arrange(p1, p2, p3, ncol=2, nrow=2))
#   }
# plotHists(sampr[[7]])


# function to summarise all model outputs using posterior samples
summariseOrderBoot = function(x, num_models=500){
  
  # order
  mods = sampr[[x]]
  print(paste("Summarising for Order:", mods$Order[1], sep=" "))
  if(mods$Order[1] == "Mammalia"){ num_models = 750 } # more models for mammalia
  mods = mods[ mods$id %in% sample(unique(mods$id), num_models, replace=F), ]
  
  ## 1. summarise occurrence
  print("Summarising occurrence logit")
  mo = mods[ mods$modeltype == "Occurrence", grep("Intercept|LandUseType|IsHost", names(mods)) ] # params
  reso = data.frame(
    median = apply(mo, 2, median),
    lower_95 = apply(mo, 2, quantile, 0.025), # 95% interval
    lower_90 = apply(mo, 2, quantile, 0.05), # 90% interval
    lower_67 = apply(mo, 2, quantile, 0.167), # 67% interval
    upper_67 = apply(mo, 2, quantile, 0.833),
    upper_90 = apply(mo, 2, quantile, 0.95),
    upper_95 = apply(mo, 2, quantile, 0.975)
  )
  reso$modeltype = "Occurrence_logit"
  reso$param = row.names(reso)

  
  ## 2. summarise z.t. abundance
  print("Summarising ZTabund log")
  mzt = mods[ mods$modeltype == "ZTAbund", grep("Intercept|LandUseType|IsHost", names(mods)) ] # params
  reszt = data.frame(
    median = apply(mzt, 2, median),
    lower_95 = apply(mzt, 2, quantile, 0.025), # 95% interval
    lower_90 = apply(mzt, 2, quantile, 0.05), # 90% interval
    lower_67 = apply(mzt, 2, quantile, 0.167), # 67% interval
    upper_67 = apply(mzt, 2, quantile, 0.833),
    upper_90 = apply(mzt, 2, quantile, 0.95),
    upper_95 = apply(mzt, 2, quantile, 0.975)
  )
  reszt$modeltype = "ZTAbund_log"
  reszt$param = row.names(reszt)
  
  
  ## 3. rescale models to natural scale and calculate hurdle (given independence assumption)
  print("Rescaling all parameters to natural scale:")
  rescaleModels = function(i){
    
    # report
    if(i %in% c(1, seq(0, 10^5, by=5))){ cat(paste(i, "...", sep="")) }
    
    # model 
    mi = mods[ mods$id == unique(mods$id)[i], ]
    
    ## a. occcurence (to probability change)
    mio = mi[ mi$modeltype == "Occurrence", grep("Intercept|LandUseType|IsHost", names(mi)) ]

    # transform samples to probability difference from intercept within host and non-host class
    nh_sec = inla.link.invlogit(mio$Intercept + mio$LandUseTypeSecondary) / inla.link.invlogit(mio$Intercept)
    nh_man = inla.link.invlogit(mio$Intercept + mio$LandUseTypeManaged) / inla.link.invlogit(mio$Intercept)
    h_sec = inla.link.invlogit(mio$Intercept + mio$IsHost + mio$LandUseTypeSecondary + mio$`LandUseTypeSecondary:IsHost`) / inla.link.invlogit(mio$Intercept + mio$IsHost)
    h_man = inla.link.invlogit(mio$Intercept + mio$IsHost + mio$LandUseTypeManaged + mio$`LandUseTypeManaged:IsHost`) / inla.link.invlogit(mio$Intercept + mio$IsHost)
    df0 = data.frame(NonHost_Secondary = nh_sec, NonHost_Managed = nh_man, Host_Secondary = h_sec, Host_Managed = h_man)
    
    # for birds where urban is included
    if(any(grepl("Urban", names(mi)))){
      nh_urb = inla.link.invlogit(mio$Intercept + mio$LandUseTypeUrban) / inla.link.invlogit(mio$Intercept)
      h_urb = inla.link.invlogit(mio$Intercept + mio$IsHost + mio$LandUseTypeUrban + mio$`LandUseTypeUrban:IsHost`) / inla.link.invlogit(mio$Intercept + mio$IsHost)
      df0 = data.frame(df0, NonHost_Urban = nh_urb, Host_Urban = h_urb)
    }
    df0$id =  unique(mods$id)[i]
    df0$modeltype = "Occurrence_prob"
    
    
    ## b. z.t. abundance (to % change abund)
    mizt = mi[ mi$modeltype == "ZTAbund", grep("Intercept|LandUseType|IsHost", names(mi)) ]

    # transform
    nh_sec = exp(mizt$Intercept + mizt$LandUseTypeSecondary) / exp(mizt$Intercept)
    nh_man = exp(mizt$Intercept + mizt$LandUseTypeManaged) / exp(mizt$Intercept)
    h_sec = exp(mizt$Intercept + mizt$IsHost + mizt$LandUseTypeSecondary + mizt$`LandUseTypeSecondary:IsHost`) / exp(mizt$Intercept + mizt$IsHost)
    h_man = exp(mizt$Intercept + mizt$IsHost + mizt$LandUseTypeManaged + mizt$`LandUseTypeManaged:IsHost`) / exp(mizt$Intercept + mizt$IsHost)
    df1 = data.frame(NonHost_Secondary = nh_sec, NonHost_Managed = nh_man, Host_Secondary = h_sec, Host_Managed = h_man)
    
    # for birds
    if(any(grepl("Urban", names(mi)))){
      nh_urb = exp(mizt$Intercept + mizt$LandUseTypeUrban) / exp(mizt$Intercept)
      h_urb = exp(mizt$Intercept + mizt$IsHost + mizt$LandUseTypeUrban + mizt$`LandUseTypeUrban:IsHost`) / exp(mizt$Intercept + mizt$IsHost)
      df1 = data.frame(df1, NonHost_Urban = nh_urb, Host_Urban = h_urb)
    }
    df1$id =  unique(mods$id)[i]
    df1$modeltype = "ZTAbund_natural"
    
    
    ## calculate hurdle model predictions
    ## 5000 pairs of samples for each (use outer then downsample)
    hurd1 = as.vector(outer(df0$NonHost_Secondary, df1$NonHost_Secondary))
    hurd2 = as.vector(outer(df0$NonHost_Managed, df1$NonHost_Managed))
    hurd3 = as.vector(outer(df0$Host_Secondary, df1$Host_Secondary))
    hurd4 = as.vector(outer(df0$Host_Managed, df1$Host_Managed))
    
    df2 = data.frame(NonHost_Secondary = hurd1, NonHost_Managed = hurd2, Host_Secondary = hurd3, Host_Managed = hurd4)
    if(any(grepl("Urban", names(mio)))){
      hurd5 = as.vector(outer(df0$NonHost_Urban, df1$NonHost_Urban))
      hurd6 = as.vector(outer(df0$Host_Urban, df1$Host_Urban))
      df2 = data.frame(df2, NonHost_Urban = hurd5, Host_Urban = hurd6)
    }
    df2 = df2[ sample(1:nrow(df2), 5000, replace=F), ] # 5000 combinations
    df2$id =  unique(mods$id)[i]
    df2$modeltype = "Abundance_hurdle"
    
    # return
    return(rbind(df0, df1, df2))
  }
  
  # rescale all models
  rs = do.call(rbind.data.frame, lapply(1:length(unique(mods$id)), rescaleModels))
  
  # 3. summarise results from occurrence
  print("Summarising occurrence prob")
  mop = rs[ rs$modeltype == "Occurrence_prob", grep("Host|NonHost", names(rs)) ]
  resop = data.frame(
    median = apply(mop, 2, median),
    lower_95 = apply(mop, 2, quantile, 0.025), # 95% interval
    lower_90 = apply(mop, 2, quantile, 0.05), # 90% interval
    lower_67 = apply(mop, 2, quantile, 0.167), # 67% interval
    upper_67 = apply(mop, 2, quantile, 0.833),
    upper_90 = apply(mop, 2, quantile, 0.95),
    upper_95 = apply(mop, 2, quantile, 0.975))
  resop$modeltype = "Occurrence_prob"
  resop$param = row.names(resop)
  
  # 4. summarise results from ZT abundance
  print("Summarising ZTabund natural")
  mzn = rs[ rs$modeltype == "ZTAbund_natural", grep("Host|NonHost", names(rs)) ]
  reszn = data.frame(
    median = apply(mzn, 2, median),
    lower_95 = apply(mzn, 2, quantile, 0.025), # 95% interval
    lower_90 = apply(mzn, 2, quantile, 0.05), # 90% interval
    lower_67 = apply(mzn, 2, quantile, 0.167), # 67% interval
    upper_67 = apply(mzn, 2, quantile, 0.833),
    upper_90 = apply(mzn, 2, quantile, 0.95),
    upper_95 = apply(mzn, 2, quantile, 0.975))
  reszn$modeltype = "ZTAbund_natural"
  reszn$param = row.names(reszn)
  
  # 5. summarise results from Abundance hurdle
  print("Summarising Abundance hurdle")
  mh = rs[ rs$modeltype == "Abundance_hurdle", grep("Host|NonHost", names(rs)) ]
  resh = data.frame(
    median = apply(mh, 2, median),
    lower_95 = apply(mh, 2, quantile, 0.025), # 95% interval
    lower_90 = apply(mh, 2, quantile, 0.05), # 90% interval
    lower_67 = apply(mh, 2, quantile, 0.167), # 67% interval
    upper_67 = apply(mh, 2, quantile, 0.833),
    upper_90 = apply(mh, 2, quantile, 0.95),
    upper_95 = apply(mh, 2, quantile, 0.975))
  resh$modeltype = "Abundance_hurdle"
  resh$param = row.names(resh)
  
  
  # save output
  dfi = do.call(rbind.data.frame, list(reso, reszt, resop, reszn, resh))
  dfi$num_it = num_models
  dfi$Order = mods$Order[1]
  dfi$nstudies = mods$nstudies[1]
  dfi$nsites = mods$nsites[1]
  dfi$mean_sp_imputed = mean(mods$spp_imputed)
  dfi$nhosts_mean = mean(mods$nhosts)
  dfi$nnonhosts_mean = mean(mods$nnonhosts)
  row.names(dfi) = c()
  write.csv(dfi, paste(save_dir, "results_singlemodel/", dfi$Order[1], "_ordermodel_summaries1.csv", sep=""), row.names=F)
  
  # return
  return(dfi)
}

# run function and save outputs
hurdle_result = lapply(1:length(sampr), summariseOrderBoot)
hurdle_res2 = do.call(rbind.data.frame, hurdle_result)
write.csv(hurdle_res2, paste(save_dir, "results_singlemodel/_ordermodels_alloutputs.csv", sep=""), row.names=FALSE)



# ----------- save in tables and visualise ----------

# result
result = read.csv(paste(save_dir, "_ordermodels_alloutputs.csv", sep=""), stringsAsFactors = FALSE)




# --- 1. Calculate model summaries on link scale for Supp. Mat ----

# split out
reso = result[ result$modeltype == "Occurrence_logit", ]
reszt = result[ result$modeltype == "ZTAbund_log", ]

# calculate summary column
rounding_digit = 2
reso$Occurrence_logit = paste(round(reso$median, rounding_digit), " (", round(reso$lower_95, rounding_digit), ", ", round(reso$upper_95,rounding_digit), ")", sep="")
reso = reso[ , c("param", "Order", "Occurrence_logit", "nstudies", "nsites", "nhosts_mean", "nnonhosts_mean", "mean_sp_imputed")]
reszt$ZTAbund_log = paste(round(reszt$median, rounding_digit), " (", round(reszt$lower_95, rounding_digit), ", ", round(reszt$upper_95,rounding_digit), ")", sep="")
reszt = reszt[ , c("param", "Order", "ZTAbund_log")]
resdf = left_join(reso, reszt)

# summarise by order
result_table = data.frame()
for(i in unique(resdf$Order)){
  x = resdf[ resdf$Order == i, ]
  xx = as.data.frame(rbind(x$Occurrence_logit, x$ZTAbund_log))
  names(xx) = x$param
  xx$Order = x$Order[1]
  xx$Model = c("Occurrence (log odds)", "Zero-truncated abundance (log)")
  if(i %in% c("Carnivora", "Cetartiodactyla", "Mammalia", "Rodentia", "Chiroptera", "Primates")){ 
    xx$LandUseTypeUrban = ""; xx$`LandUseTypeUrban:IsHost` = ""; xx$Class = "Mammalia" 
  } else{
      xx$Class = "Aves"
    }
  xx = xx[ , c("Class", "Order", "Model", "Intercept", "LandUseTypeSecondary", "LandUseTypeManaged", "LandUseTypeUrban", "IsHost", "LandUseTypeSecondary:IsHost", "LandUseTypeManaged:IsHost", "LandUseTypeUrban:IsHost")]
  xx$Sites = x$nsites[1]
  xx$Studies = x$nstudies[1]
  xx$NumHosts_mean = x$nhosts_mean[1]
  xx$NumNonHosts_mean = x$nnonhosts_mean[1]
  xx$SppImputed_mean = x$mean_sp_imputed[1]
  result_table = rbind(result_table, xx)
}

write.csv(result_table, "./output/tables/SI_HurdleBootstrap_summarytable.csv", row.names=FALSE)

