
### Gibb et al, 'Land use has global and systematic effects on local zoonotic host communities"
# Script 19.2: Processes outputs of hurdle bootstrap at species-level for Primates (separate host/non-host models)
# Reads in sampled model objects, subsets to each bootstrap iteration
# Within each iteration, rescales models to natural (prob/linear) scale, and combines occ/ztabund models to calculate hurdle using samples
# Then lastly summarises across all iterations of bootstrap using posterior samples

# dependencies
library(lme4)
library(ggplot2)
library(nlme)
library(plyr)
library(dismo)
library(geosphere)
library(spdep)
library(rgdal)
library(INLA)
library(viridis)
library(raster)
library(RColorBrewer)
library(KernSmooth)
library(dplyr)
library(reshape2)

# colours for plotting
nonhostcol = "#018571"
hostcol = "#8c510a"


# ------------- data info --------------

# directory where bootstrap outputs are stored 
files_dir = "./output/model_outputs/dec_2019/order_hurdle/species_hurdle_separate/"
ff = list.files(files_dir, pattern=".R")

# directory for saving
save_dir = "./output/model_outputs/dec_2019/order_hurdle/results_separate/"

# get info about bootstraps
ff_doc = do.call(rbind.data.frame, strsplit(ff, "_"))
names(ff_doc) = c("mtype", "", "order", "id", "num_it", "ff")
ff_doc$loc = paste(files_dir, ff, sep="")




# ----------- process and summarise for each Order -----------

# function to get all samples for n models for a particular Order
getSamplesOrder = function(order, num_models = 500){
  
  # details
  print(paste("Summarising bootstrap for:", order, sep=" "))
  mods = ff_doc[ ff_doc$order == order, ]
  if(order == "Mammalia"){ num_models = 750 }
  foo = ceiling( num_models / as.numeric(as.vector(mods$num_it[1])) ) # for number of models to load
  mods = mods[ sample(1:nrow(mods), foo), ] 
  
  # get samples
  print("Getting samples...")
  samples = data.frame()
  for(i in mods$loc){
    
    load(i)
    getSamp = function(x){ 
      
      # if 
      if(result[[x]]$nnonhosts == 0){ next } # when all transitioned (v rare; affects model specification)
      
      # occurrence and abundance samples for host
      ox = result[[x]]$occurrence_samples_host
      ox$modeltype = "Occurrence"
      ox$class = "Host"
      ztx = result[[x]]$ztabund_samples_host
      ztx$modeltype = "ZTAbund"
      ztx$class = "Host"
      resx = rbind(ox, ztx)
      
      # occurrence and abundance samples for nonhost
      ox = result[[x]]$occurrence_samples_nh
      ox$modeltype = "Occurrence"
      ox$class = "Non-host"
      ztx = result[[x]]$ztabund_samples_nh
      ztx$modeltype = "ZTAbund"
      ztx$class = "Non-host"
      if(order == "Primates"){ ox$`LandUseTypeManaged:1` = NA; ztx$`LandUseTypeManaged:1` = NA } # no non-hosts in managed for Primates
      resx = do.call(rbind.data.frame, list(resx, ox, ztx))
      
      # label with additional info
      resx$id = sample(1:10^9, 1)
      resx$nobs = result[[x]]$nobs
      resx$nhosts = result[[x]]$nhosts
      resx$nnonhosts = result[[x]]$nnonhosts
      resx$nnonhosts = result[[x]]$nnonhosts
      resx$nstudies = result[[x]]$nstudies
      resx$nsites = result[[x]]$nsites
      resx$spp_imputed = result[[x]]$species_imputed
      return(resx)
    }
    ss = do.call(rbind.data.frame, lapply(1:length(result), getSamp))
    samples = rbind(ss, samples)
  }
  
  # rename and remove params
  names(samples)[ grep("Intercept", names(samples)) ] = "Intercept"
  names(samples)[ names(samples) == "LandUseTypeManaged:1" ] = "LandUseTypeManaged"
  names(samples)[ names(samples) == "LandUseTypeSecondary:1" ] = "LandUseTypeSecondary"
  names(samples)[ names(samples) == "LandUseTypeUrban:1" ] = "LandUseTypeUrban"

  # return samples
  samples$Order = order
  return(samples)
}

# run for primates
sampr = getSamplesOrder("Primates", num_models = 500)
sampr = list(sampr)

# big function to summarise all model outputs using posterior samples
summariseOrderBoot = function(x){
  
  # order
  mods = sampr[[x]]
  print(paste("Summarising for Order:", mods$Order[1], sep=" "))
  #mods = mods[ mods$id %in% sample(unique(mods$id), 250, replace=F), ]
  
  ## for each separate model subset (2xhost, 2xnonhost) rescale models to natural scale and calculate hurdle (given independence assumption)
  print("Rescaling all parameters to natural scale")
  rescaleModels = function(i){
    
    # model 
    cat(i)
    mi = mods[ mods$id == unique(mods$id)[i], ]
    
    # function rescale occurrences
    rescaleOcc = function(samp, class){
      
      # transform samples to probability difference from intercept (invlogit)
      sec = inla.link.invlogit(samp$Intercept + samp$LandUseTypeSecondary) / inla.link.invlogit(samp$Intercept)
      man = inla.link.invlogit(samp$Intercept + samp$LandUseTypeManaged) / inla.link.invlogit(samp$Intercept)
      df0 = data.frame(Secondary = sec, Managed = man)
      
      # for birds where urban is included
      if(any(grepl("Urban", names(samp)))){
        urb = inla.link.invlogit(samp$Intercept + samp$LandUseTypeUrban) / inla.link.invlogit(samp$Intercept)
        df0 = data.frame(df0, Urban = urb)
      }
      df0$class = class
      df0$id = unique(mods$id)[i]
      df0$modeltype = "Occurrence_prob"
      return(df0)
    }
    
    # host occcurence (to probability change)
    mioh = mi[ mi$modeltype == "Occurrence" & mi$class == "Host", grep("Intercept|LandUseType|IsHost", names(mi)) ]
    resoh = rescaleOcc(mioh, class="Host")
    mionh = mi[ mi$modeltype == "Occurrence" & mi$class == "Non-host", grep("Intercept|LandUseType|IsHost", names(mi)) ]
    resonh = rescaleOcc(mionh, class="Non-host")
    df1 = rbind(resoh, resonh)

    ## b. z.t. abundance (to % change abund) - just need to exponentiate as log link
    mizt = mi[ mi$modeltype == "ZTAbund", grep("Intercept|LandUseType|class", names(mi)) ]
    df2 = mizt[ , c("class", "LandUseTypeManaged", "LandUseTypeSecondary")]
    df2[,2:3] = exp(df2[,2:3])
    names(df2)[2:3] = c("Managed", "Secondary")
    if(any(grepl("Urban", names(mi)))){ df2 = data.frame(df2, Urban = exp(mizt$LandUseTypeUrban)) } # for birds
    df2$id =unique(mods$id)[i]
    df2$modeltype = "ZTAbund_natural"
    dfi = rbind(df1, df2)
    
    # c. Calculate hurdle predictions for each class assuming independence of occurrence/zt-abundance components
    for(c in unique(dfi$class)){
      
      aa = dfi[ dfi$class == c, ]
      
      # propagate variance; multiply distributions together
      sec_hurd = as.vector(outer(aa$Secondary[ aa$modeltype == "Occurrence_prob" ], aa$Secondary[ aa$modeltype == "ZTAbund_natural"]))
      man_hurd = as.vector(outer(aa$Managed[ aa$modeltype == "Occurrence_prob" ], aa$Managed[ aa$modeltype == "ZTAbund_natural"]))

      # subsample to 5000 samples
      sec_hurd = sample(sec_hurd, 5000, replace=FALSE)
      man_hurd = sample(man_hurd, 5000, replace=FALSE)
      dfhurd = data.frame(Secondary=sec_hurd, Managed=man_hurd)
      
      # birds
      if(any(grepl("Urban", names(aa)))){
        urb_hurd = as.vector(outer(aa$Urban[ aa$modeltype == "Occurrence_prob" ], aa$Urban[ aa$modeltype == "ZTAbund_natural"]))
        urb_hurd = sample(urb_hurd, 5000, replace=FALSE)
        dfhurd = data.frame(dfhurd, Urban=urb_hurd)
      }
      
      # add meta
      dfhurd$class = c
      dfhurd$modeltype = "Abundance_hurdle"
      dfhurd$id = unique(mods$id)[i]
      dfi = rbind(dfi, dfhurd)
    }
    return(dfi)
  }
  
  # return samples of all rescaled models
  rs = do.call(rbind.data.frame, lapply(1:length(unique(mods$id)), rescaleModels))
  
  # combine result with unscaled models
  if(any(grepl("Urban", names(mods)))){  
    mods2 = mods[ , c("Intercept", "LandUseTypeManaged", "LandUseTypeSecondary", "LandUseTypeUrban", "modeltype", "class", "id") ]
    names(mods2)[1:4] = c("Intercept", "Managed", "Secondary", "Urban")
  } else{
    mods2 = mods[ , c("Intercept", "LandUseTypeManaged", "LandUseTypeSecondary", "modeltype", "class", "id") ]
    names(mods2)[1:3] = c("Intercept", "Managed", "Secondary")
  }
  mods2$modeltype[ mods2$modeltype == "Occurrence" ] = "Occurrence_logit"
  mods2$modeltype[ mods2$modeltype == "ZTAbund" ] = "ZTAbund_log"
  rs$Intercept = NA
  rs2 = rbind(rs, mods2)
  
  # summarise results from all models
  print("Summarising all models")
  if(any(grepl("Urban", names(rs)))){  
    rs2 = melt(rs2[ , -which(names(rs2)=="id")], measure.vars = c("Intercept", "Secondary", "Managed", "Urban"))
  } else{
    rs2 = melt(rs2[ , -which(names(rs2)=="id")], measure.vars = c("Intercept", "Secondary", "Managed"))
    }
  summary = plyr::ddply(rs2, .(class, modeltype, variable), plyr::summarise,
                        median = median(value),
                        lower_95 = quantile(value, 0.025, na.rm=T),
                        lower_90 = quantile(value, 0.05, na.rm=T),
                        lower_67 = quantile(value, 0.167, na.rm=T),
                        upper_67 = quantile(value, 0.833, na.rm=T),
                        upper_90 = quantile(value, 0.95, na.rm=T),
                        upper_95 = quantile(value, 0.975, na.rm=T))
  names(summary)[ which(names(summary) == "variable") ] = "param"
  
  # save output
  summary$num_it = 500
  summary$Order = mods$Order[1]
  summary$nstudies = mods$nstudies[1]
  summary$nsites = mods$nsites[1]
  summary$mean_sp_imputed = mean(mods$spp_imputed)
  summary$nhosts_mean = mean(mods$nhosts)
  summary$nnonhosts_mean = mean(mods$nnonhosts)
  summary$totalspp = mods$nhosts[1] + mods$nnonhosts[1]
  write.csv(summary, paste(save_dir, summary$Order[1], "_ordermodel_summaries1.csv", sep=""), row.names=F)
  
  # return
  return(summary)
}

# run big function and save outputs
hurdle_result = lapply(1:length(sampr), summariseOrderBoot)
hurdle_res2 = do.call(rbind.data.frame, hurdle_result)
write.csv(hurdle_res2, paste(save_dir, "_ordermodels_alloutputs.csv", sep=""), row.names=FALSE)


