
### Gibb et al, 'Land use has global and systematic effects on local zoonotic host communities"
# Script 20: Reads in results from hurdle bootstrap, summarises and visualises in figures 

# setwd("/Volumes/ritd-ag-project-rd00nx-rjgib11/chapter2_predicts/predicts_analysis_201901/")
# setwd("Y:/chapter2_predicts/predicts_analysis_201901/")
#setwd("/scratch/scratch/ucbtgib/predicts")

# dependencies
library(lme4)
library(ggplot2)
library(nlme)
library(plyr)
library(dismo)
library(geosphere)
library(spdep)
library(rgdal)
library(INLA)
library(viridis)
library(raster)
library(RColorBrewer)
library(KernSmooth)
library(dplyr)
library(reshape2)
library(gridExtra)
library(lattice)
library(grid)

# colours for plotting
nonhostcol = "#018571"
hostcol = "#8c510a"

# result_dir
result_dir = "./output/model_outputs/dec_2019/order_hurdle/"



# -------------- read in results for summarising ----------

# result for all orders except Primates, and primates separately due to different specification during fitting
r1 = read.csv(paste(result_dir, "/results_singlemodel/_ordermodels_alloutputs.csv", sep=""), stringsAsFactors = FALSE)
r1 = r1[ r1$modeltype %in% c("Occurrence_prob", "ZTAbund_natural", "Abundance_hurdle"), ]
r2 = read.csv(paste(result_dir, "/results_separate/Primates_ordermodel_summaries1.csv", sep=""), stringsAsFactors = FALSE)
r2 = r2[ r2$param != "Intercept", ]

# harmonise formatting
r1$class = ifelse(grepl("NonHost", r1$param), "Non-host", "Host")
r1$param = unlist(lapply(strsplit(r1$param, "_"), "[[", 2))
r1$totalspp = r1$nhosts_mean + r1$nnonhosts_mean

# combine
res = rbind(r1, r2)

# histograms of n. spp. imputed for each Order
load(paste(result_dir, "results_singlemodel/numimputed_allorders_bootstrap.R", sep=""))




# ------ 1. visualise separate components of hurdle model: occurrence/ztabund ----------

# visualise in 2 batches
orders1 = c("Mammalia", "Chiroptera", "Rodentia", "Cetartiodactyla")
orders2 = c("Carnivora", "Primates", "Passeriformes", "Psittaciformes")

visualiseModels = function(orders){
  
 # subset to specific orders
  df = res[ res$Order %in% orders, ]
  
  # set up land use type etc
  df$LandUse = df$param
  df$IsHost = df$class
  df[ df$Order == "Psittaciformes" & df$modeltype == "Occurrence_prob" & df$LandUse == "Urban", c("lower_95", "upper_95") ] = NA
  df$LandUse[ df$LandUse == "Secondary" ] = "Sec"
  df$LandUse[ df$LandUse == "Managed" ] = "Man"
  df$LandUse[ df$LandUse == "Urban" ] = "Urb"
  
  # rescale around 0 and covert to % change
  df[ , 1:7] = (df[ , 1:7]-1)*100
  #df$LandUse = factor(df$LandUse, levels=c("Sec.", "Managed", "Urban"), ordered=T)
  df$LandUse = factor(df$LandUse, levels=c("Sec", "Man", "Urb"), ordered=T)
  df$IsHost = factor(df$IsHost, levels=c("Non-host", "Host"), ordered=T)
  df$Order = factor(df$Order, levels=orders, ordered=T)
  
  axis_title_size = 11
  axis_text_size = 9
  strip_text_size = 9.5
  
  # plot occurrence
  yaxis_label = "Difference in probability of occurrence (%)"
  p1 = ggplot(df[ df$modeltype == "Occurrence_prob", ], aes(LandUse, median, fill=IsHost, col=IsHost)) + 
    geom_point(position=position_dodge(width=0.5), size=2.6) +
    geom_errorbar(aes(ymin=lower_95, ymax=upper_95), position=position_dodge(width=0.5), width=0.25, lwd=0.3, alpha=1, show.legend = FALSE) + 
    geom_linerange(aes(ymin=lower_67, ymax=upper_67), position=position_dodge(width=0.5), lwd=0.8, show.legend = FALSE) + 
    facet_wrap(~Order, ncol=1, scales="free_y") + 
    geom_hline(yintercept=0, lty=2) + 
    ylab(yaxis_label) + 
    theme_bw() + 
    xlab("Land use type") + 
    theme(panel.grid.minor = element_blank(),
          panel.grid.major = element_line(size=0.2, color='grey90'),
          legend.title=element_blank(),
          axis.text = element_text(size=axis_text_size),
          #axis.text.x = element_text(size=10.5),
          strip.text.x = element_text(size=strip_text_size, color="black"), 
          #strip.text.x = element_blank(),
          strip.background = element_rect(fill="grey92"),
          #strip.background = element_blank(),
          legend.text = element_text(size=12),
          axis.title = element_text(size=axis_title_size),
          legend.background = element_rect(fill=NA, color=NA, size=0.5), 
          #legend.position=c(0.93, 0.98),
          #legend.position="bottom",
          legend.position="none",
          plot.title = element_text(hjust=0.5, size=18)) + 
    scale_fill_manual(values=c("Non-host"=nonhostcol, "Host"=hostcol), name="") + 
    scale_color_manual(values=c("Non-host"=nonhostcol, "Host"=hostcol), name="")
  
  # plot occurrence
  yaxis_label = "Difference in abundance given presence (%)"
  p2 = ggplot(df[ df$modeltype == "ZTAbund_natural", ], aes(LandUse, median, fill=IsHost, col=IsHost)) + 
    geom_point(position=position_dodge(width=0.5), size=2.6) +
    geom_errorbar(aes(ymin=lower_95, ymax=upper_95), position=position_dodge(width=0.5), width=0.25, lwd=0.3, alpha=1, show.legend = FALSE) + 
    geom_linerange(aes(ymin=lower_67, ymax=upper_67), position=position_dodge(width=0.5), lwd=0.8, show.legend = FALSE) + 
    facet_wrap(~Order, ncol=1, scales="free_y") + 
    geom_hline(yintercept=0, lty=2) + 
    ylab(yaxis_label) + 
    theme_bw() + xlab("Land use type") + 
    theme(panel.grid.minor = element_blank(),
          panel.grid.major = element_line(size=0.2, color='grey90'),
          legend.title=element_blank(),
          axis.text = element_text(size=axis_text_size),
          #axis.text.x = element_text(size=10.5),
          strip.text.x = element_text(size=strip_text_size, color="black"), 
          #strip.text.x = element_blank(),
          strip.background = element_rect(fill="grey92"),
          #strip.background = element_blank(),
          legend.text = element_text(size=10),
          axis.title = element_text(size=axis_title_size),
          legend.background = element_rect(fill="white", color="white", size=0.5), 
          legend.position="none",
          plot.title = element_text(hjust=0.5, size=18)) + 
    scale_fill_manual(values=c("Non-host"=nonhostcol, "Host"=hostcol), name="") + 
    scale_color_manual(values=c("Non-host"=nonhostcol, "Host"=hostcol), name="")
  
  # plot histograms
  # oo = unlist(lapply(impr, function(x) x$order))
  # hist_data = data.frame()
  # for(i in 1:length(orders)){
  #   i1 = impr[[ which(oo == orders[i]) ]]$spp_imp
  #   i2 = data.frame(order = orders[i], spp_imp = i1)
  #   hist_data = rbind(hist_data, i2)
  # }
  hist_data = impr[ impr$order %in% orders, ]
  hist_data$numhosts_orig = hist_data$nhosts - hist_data$spp_imp
  hist_data$order = factor(hist_data$order, levels=orders, ordered=T)
  
  # # bracket the impr
  # hist_data$xmin = hist_dat$numhosts_orig - 5
  # xmax = ddply(hist_data, .(order), plyr::summarise, xmax = max(nhosts)+5)
  # hist_data = left_join(hist_data, xmax)
  
  # create species count facet
  spp_count = df[ !duplicated(df$Order), c("Order", "totalspp")]
  #spp_count$facetlabel = paste(spp_count$Order, " (", spp_count$totalspp, " spp.)", sep="")
  spp_count$facetlabel = paste(spp_count$Order, " (", spp_count$totalspp, ")", sep="")
  names(spp_count)[1] = "order"
  hist_data = left_join(hist_data, spp_count)
  factor_order = spp_count$facetlabel[ match(orders, spp_count$order) ]
  hist_data$facetlabel = factor(hist_data$facetlabel, levels=factor_order, ordered=T)
  
  p3 = ggplot(hist_data, aes(nhosts)) + 
    geom_histogram(binwidth=1, color="black", size=0.3, fill=hostcol, alpha=0.6) + 
    #geom_density(color="black", size=0.3, fill=hostcol, alpha=0.6) + 
    geom_vline(aes(xintercept = numhosts_orig), col="red", size=0.5) +
    facet_wrap(~facetlabel, ncol=1, scales="free_x") +
    theme_bw() +
    ylab("Number of bootstrap models") +
    #ylab("Proportion of bootstrap models") +
    xlab("Num. host species") +
    theme(panel.grid.minor = element_blank(),
          panel.grid.major = element_line(size=0.2, color='grey90'),
          legend.title=element_blank(),
          axis.text = element_text(size=axis_text_size),
          axis.text.x = element_text(size=8),
          strip.text.x = element_text(size=strip_text_size, color="black"), 
          #strip.text.x = element_blank(),
          strip.background = element_rect(fill="grey92"),
          #strip.background = element_blank(),
          legend.text = element_text(size=10),
          axis.title = element_text(size=axis_title_size),
          legend.background = element_rect(fill="white", color="white", size=0.5), 
          legend.position="none",
          plot.title = element_text(hjust=0.5, size=18))
  
    
  plot_final = gridExtra::grid.arrange(grobs = list(p1, p2, p3), ncol=3, nrow=1, widths=c(1, 1, 0.9))
  plot_final
}

# plot and save
v1 = visualiseModels(orders1)
v2 = visualiseModels(orders2)
r = rectGrob(gp=gpar(fill="white", col="white"))
v_final = gridExtra::grid.arrange(grobs = list(v1, r, v2), nrow=1, ncol=3, widths=c(1, 0.05, 1))
ggsave(v_final, file="./output/figures/nature_ms3_jan2019/ExtendedData_HurdleModels_separatemodels.png", device = "png", dpi=900, height=8, width=14, units="in")




# ---------- 2. Plot combined hurdle model predictions ---------

# to plot hurdle predictions
plotHurdlePreds = function(orders){

  df = res[ res$Order %in% orders, ]
  df = df[ df$modeltype == "Abundance_hurdle", ]

  # set up land use type etc
  df$LandUse = df$param
  df$IsHost = df$class
  df[ df$Order == "Psittaciformes" & df$LandUse == "Urban", c("lower_95", "lower_90", "upper_90", "upper_95") ] = NA
  df$LandUse[ df$LandUse == "Secondary" ] = "Sec."
  # df$LandUse[ df$LandUse == "Managed" ] = "Man"
  # df$LandUse[ df$LandUse == "Urban" ] = "Urb"
  
  # rescale around 0 and covert to % change
  df[ , 1:7] = (df[ , 1:7]-1)*100
  df$LandUse = factor(df$LandUse, levels=c("Sec.", "Managed", "Urban"), ordered=T)
  #df$LandUse = factor(df$LandUse, levels=c("Sec", "Man", "Urb"), ordered=T)
  df$IsHost = factor(df$IsHost, levels=c("Non-host", "Host"), ordered=T)
  df$Order = factor(df$Order, levels=orders, ordered=T)
  
  # add barplot
  pp = ggplot(df, aes(LandUse, median, fill=IsHost, col=IsHost)) + 
    geom_point(position=position_dodge(width=0.5), size=2.6) +
    geom_errorbar(aes(ymin=lower_95, ymax=upper_95), position=position_dodge(width=0.5), width=0.2, lwd=0.325, alpha=1, show.legend = FALSE) + 
    geom_linerange(aes(ymin=lower_67, ymax=upper_67), position=position_dodge(width=0.5), lwd=0.85, show.legend = FALSE) + 
    facet_wrap(~Order, nrow=2) + 
    geom_hline(yintercept=0, lty=2) + 
    ylab("Predicted abundance difference (%)") + 
    theme_bw() + xlab("Land use") + 
    theme(panel.grid.minor = element_line(color="grey93", size=0.3),
          legend.title=element_blank(),
          axis.text = element_text(size=11),
          #axis.text.x = element_text(size=10.5),
          strip.text.x = element_text(size=11), 
          legend.text = element_text(size=11),
          axis.title = element_text(size=14),
          #legend.background = element_rect(fill="white", color="black", size=0.3), 
          legend.background = element_rect(fill="white", color="white", size=0.3), 
          legend.position=c(0.065, 0.94), # without mammalia
          #legend.position=c(0.565, 0.415), 
          plot.title = element_text(hjust=0.5, size=18)) + 
    scale_fill_manual(values=c("Non-host"=nonhostcol, "Host"=hostcol), name="") + 
    scale_color_manual(values=c("Non-host"=nonhostcol, "Host"=hostcol), name="")
  pp
}

# plot
orders = c("Passeriformes", "Chiroptera", "Rodentia", "Cetartiodactyla",  "Psittaciformes", "Carnivora", "Primates")
figure = plotHurdlePreds(orders)
ggsave(figure, file="./output/figures/nature_ms3_jan2019/Figure3_OrderModels_withoutmammalia.png", device = "png", dpi=900, height=6.8, width=9, units="in")






# --- 1. Calculate model summaries on link scale for Supp. Mat ----

# result for all orders except Primates, and primates separately due to different specification during fitting
r1 = read.csv(paste(result_dir, "/results_singlemodel/_ordermodels_alloutputs.csv", sep=""), stringsAsFactors = FALSE)
r1 = r1[ r1$modeltype %in% c("Occurrence_logit", "ZTAbund_log"), ]
r2 = read.csv(paste(result_dir, "/results_separate/Primates_ordermodel_summaries1.csv", sep=""), stringsAsFactors = FALSE)
r2 = r2[ r2$modeltype %in% c("Occurrence_logit", "ZTAbund_log"), ]
r2$param = paste(r2$class, r2$param, sep="_")

# harmonise formatting
r1$class = ifelse(grepl("NonHost", r1$param), "Non-host", "Host")
r1$totalspp = r1$nhosts_mean + r1$nnonhosts_mean

# result
res = rbind(r1, r2)

# round
rounding_digit = 3
res[ , 1:7 ] = round(res[ , 1:7], rounding_digit)
res$Estimate = paste(res$median, " (", res$lower_95, ", ", res$upper_95, ")", sep="")

res1 = res[ res$modeltype == "Occurrence_logit", c("param", "Order", "Estimate", "nstudies", "nsites", "nhosts_mean", "nnonhosts_mean", "mean_sp_imputed")]
names(res1)[3] = "Occurrence_logit"
res2 = res[ res$modeltype == "ZTAbund_log", c("param", "Order", "Estimate", "nstudies", "nsites", "nhosts_mean", "nnonhosts_mean", "mean_sp_imputed")]
names(res2)[3] = "ZTAbund_log"
resdf = left_join(res1, res2)

# add class info
resdf$Class = ifelse(resdf$Order %in% c("Passeriformes", "Psittaciformes"), "Aves", "Mammalia")
resdf$Order[ resdf$Order == "Mammalia" ] = "All"
resdf$NumHosts = resdf$nhosts_mean - resdf$mean_sp_imputed[1]
resdf$TotalSpecies = resdf$nhosts_mean + resdf$nnonhosts_mean

# save
resdf = resdf[ , c("Class", "Order", "param", "Occurrence_logit", "ZTAbund_log", "nstudies", "nsites", "TotalSpecies", "NumHosts", "mean_sp_imputed", "nhosts_mean") ]
resdf = resdf[ order(resdf$Class, resdf$Order),]
write.csv(resdf, "./output/tables/SI_HurdleBootstrap_summarytable.csv", row.names=FALSE)


# summarise by order
result_table = data.frame()
result_prim = data.frame()
for(i in unique(resdf$Order)){
  x = resdf[ resdf$Order == i, ]
  xx = as.data.frame(rbind(x$Occurrence_logit, x$ZTAbund_log))
  names(xx) = x$param
  xx$Order = x$Order[1]
  xx$Model = c("Occurrence (log odds)", "Zero-truncated abundance (log)")
  if(i %in% c("Carnivora", "Cetartiodactyla", "Mammalia", "Rodentia", "Chiroptera")){ 
    xx$LandUseTypeUrban = ""; xx$`LandUseTypeUrban:IsHost` = ""; xx$Class = "Mammalia" 
  } else{
    xx$Class = "Aves"
  }
  if(i == "Mammalia"){ xx$Order = "All" }
  if(i == "Primates"){ 
    xx$Class = "Mammalia"
    xx = xx[ , c("Class", "Order", "Model", "Non-host_Intercept", "Non-host_Secondary", "Non-host_Managed", "Host_Intercept", "Host_Secondary", "Host_Managed")]
  } else{
    xx = xx[ , c("Class", "Order", "Model", "Intercept", "LandUseTypeSecondary", "LandUseTypeManaged", "LandUseTypeUrban", "IsHost", "LandUseTypeSecondary:IsHost", "LandUseTypeManaged:IsHost", "LandUseTypeUrban:IsHost")]
    }
  xx$Sites = x$nsites[1]
  xx$Studies = x$nstudies[1]
  xx$TotalSpecies = x$nhosts_mean[1]+ x$nnonhosts_mean[1]
  xx$NumHosts = x$nhosts_mean[1] - x$mean_sp_imputed[1]
  xx$SppImputed_mean = x$mean_sp_imputed[1]
  xx$NumHosts_bootstrapmean = x$nhosts_mean[1]
  if(i == "Primates"){ 
    result_prim = xx
  } else{
    result_table = rbind(result_table, xx)
  }
}

result_table = result_table[ order(result_table$Class, result_table$Order, result_table$Model),]
result_prim

# save for MS
write.csv(result_table, "./output/tables/SI_HurdleBootstrap_summarytable.csv", row.names=FALSE)
write.csv(result_prim, "./output/tables/SI_HurdleBootstrap_summarytable_primates.csv", row.names=FALSE)

