

# Gibb et al, 'Land use has global and systematic effects on zoonotic host communities'
# Code 24: Summary mapping of datasets for summary figures in MS and SI

# dependencies
library(ggplot2)
library(rgdal)
library(countrycode)
library(RColorBrewer)

# functions
source("00_functions_general.R")

# colours for plotting
nonhostcol = "#018571"
hostcol = "#8c510a"




# ------------------ full dataset ------------------

# site data
dd1_site = read.csv("./output/predicts_sites_formodelling/Tier1_HIAMB_predicts_sites_SR0_20190514.csv", stringsAsFactors=F)

# mammal studies
mam = read.csv("./output/predicts_sites_formodelling/Tier2_mammals_predicts_sites_SR0_20190514.csv", stringsAsFactors=F)
dd1_site$All_Mam = ifelse(dd1_site$SS %in% mam$SS, "Mam", "Other")

# either mammal sampling or 'cannot decide' use intensity
models_df = dd1_site[ dd1_site$UseIntensity != "Cannot decide" | dd1_site$All_Mam == "Mam", ]
subr = read.csv("./data/worldborders/UN_regions_crossref.csv", stringsAsFactors = F)
models_df = full_join(models_df, subr)



# ------------------ map data globally --------------------

# shapefile data
cont = readOGR("./data/worldborders/continents/continent.shp", "continent")
cont1 = fortify(cont[ cont$CONTINENT != "Antarctica", ])
countries = readOGR("./data/worldborders/TM_WORLD_BORDERS-0.3/TM_WORLD_BORDERS-0.3.shp", "TM_WORLD_BORDERS-0.3")

# mark countries as 1/0 for whether PREDICTS studies are there or not
countries$InPREDICTS = ifelse(countries$ISO2 %in% countrycode(models_df$Country, "country.name", "iso2c"), 1, 0)

# split into two sections
countries1 = fortify(countries[ countries$InPREDICTS == 1, ])
countries2 = fortify(countries[ countries$InPREDICTS == 0 & countries$NAME != "Antarctica", ])

# also palegreen3
global_map = ggplot() + coord_fixed() +
  geom_polygon(data=countries1, aes(x=long, y=lat, group=group), fill="skyblue3", lwd=0.2, col="grey60") + 
  geom_polygon(data=countries2, aes(x=long, y=lat, group=group), fill="grey90", lwd=0.2, col="grey60") + 
  geom_polygon(data=cont1, aes(x=long, y=lat, group=group), fill=NA, lwd=0.2, col="grey10") + theme_classic() +
  geom_point(data=models_df, aes(Longitude, Latitude, col=All_Mam), size=2, pch=16, stroke=0.2) +
  scale_colour_manual(values = c("Other"="red3", "Mam"="black")) +
  theme(panel.background = element_rect(fill=NA, color="black", size=1.2),
        axis.text = element_blank(), 
        axis.ticks = element_blank(), 
        axis.title = element_blank(),
        legend.position="none")

ggsave(global_map, file="./output/figures/MS_Figure1_geomap_allstudies.png", width=12, height=5, units="in", device="png", dpi=1200)




# ---------------- plot taxonomic distribution of hosts ------------

# combined preds data
load(file="./output/predicts_processed/predicts_pathogens_researcheffort.R")

# color ramp palette
spec = colorRampPalette(brewer.pal(11, "Spectral"))

# create data for pie chart
dd1$HumanInfective_Any[ dd1$Class == "Mammalia" ] = dd1$Sero_HumanInfective_Any[ dd1$Class == "Mammalia" ]
dd1$IsZoonotic_Strict[ dd1$Class == "Mammalia" ] = dd1$Sero_IsZoonotic_Strict[ dd1$Class == "Mammalia" ]
hosts = dd1[ !duplicated(dd1$bgb) & dd1$HumanInfective_Any == 1 & dd1$IsDomestic == 0, ]
hosts = hosts[ , c("bgb", "Order", "Family", "Class")]
hosts_ply = ddply(hosts, .(Class), summarize, sum=length(bgb))
hosts_ply$class2 = as.vector(hosts_ply$Class)
hosts_ply$class2[ hosts_ply$class2 %in% c("Mammalia") ] = "Mammals"
hosts_ply$class2[ hosts_ply$class2 %in% c("Aves") ] = "Birds"
hosts_ply$class2[ hosts_ply$class2 %in% c("Arachnida", "Gastropoda", "Insecta") ] = "Invertebrates"
hosts_ply$class2[ hosts_ply$class2 %in% c("Reptilia", "Amphibia") ] = "Rep/Amphibs"
hosts_ply = ddply(hosts_ply, .(class2), summarize, sum=sum(sum))
hosts_ply$fraction = hosts_ply$sum / sum(hosts_ply$sum)
hosts_ply = hosts_ply[order(hosts_ply$fraction), ]
hosts_ply$ymax = cumsum(hosts_ply$fraction)
hosts_ply$ymin = c(0, head(hosts_ply$ymax, n=-1))
hosts_ply$class3 = paste(hosts_ply$class2, " (", hosts_ply$sum, ")", sep="")

hosts_ply = as.data.frame(hosts_ply)

# plot
(allhosts_pie = ggplot(hosts_ply, aes(fill=class2, ymax=ymax, ymin=ymin, xmax=4, xmin=2.5)) +
    geom_rect(colour="white", lwd=1.5) +
    coord_polar(theta="y") +
    xlim(c(0, 4)) +
    theme_bw() +
    theme(panel.grid=element_blank(),
          axis.text=element_blank(), 
          axis.ticks=element_blank(),
          legend.text = element_text(size=12),
          legend.title = element_blank(),
          legend.position = c(0.5, 0.5),
          panel.background = element_rect(fill=NA, color="black")) +
    scale_fill_manual(values=c("Mammals" = "grey10",
                               "Rep/Amphibs" = spec(11)[4],
                               "Birds" = spec(11)[11],
                               "Invertebrates" = spec(11)[9])))

ggsave(allhosts_pie, file="./output/figures/MS_Figure1_piechart_tier1hosts.png", width=3.75, height=3.75, units="in", device="png", dpi=1200)




# ----------- per-study hosts as proportion of total sampled richness --------------

# colourscale
v = viridis(250)

# prep data
ss = ddply(dd1[ dd1$SS_HIAMB == 1, ], .(SS), plyr::summarize, 
           SR = length(unique(bgb)),
           HostSR = length(unique(bgb[HumanInfective_Any == 1 ])),
           Biome = head(Biome, 1))
ss$PropHosts = ss$HostSR / ss$SR
ss$TropTemp = ifelse(ss$Biome %in% c("Temperate Broadleaf & Mixed Forests", "Mediterranean Forests, Woodlands & Scrub", "Deserts & Xeric Shrublands", "Temperate Grasslands, Savannas & Shrublands", "Temperate Conifer Forests", "Montane Grasslands & Shrublands", "Tundra", "Boreal Forests/Taiga"),
                     "Temperate", "Tropical")

# plot
(perstudy_boxplots = ggplot(ss, aes(factor(TropTemp), PropHosts*100, fill=TropTemp)) + geom_boxplot() +
    theme_bw() + xlab("") + ylab("Hosts % of per-study total species richness") + 
    theme(legend.title = element_blank(),
          axis.text.x = element_text(size=14),
          axis.text.y = element_text(size=14),
          axis.title.y = element_text(size=15),
          legend.text = element_text(size=13), 
          legend.position="none") + ylim(0, 100) +
    scale_fill_manual(values=c("Temperate" = v[120],
                               "Tropical" = v[190])))

ggsave(perstudy_boxplots, file="./output/figures/MS_Figure1_studyboxplots_tier1.png", width=3.25, height=5, units="in", device="png", dpi=1200)




# 
# ### ---------------------- Mammal and bird zoonotic agent hosts --------------
# 
# # PREDICTS/hosts data
# load("./output/predicts_processed/predicts_pathogens.R")
# 
# # avian studies and zoonotic hosts: 80 studies, 143 hosts, 4616 sites
# dd2 = dd1[ dd1$Class == "Aves", ]
# birdhosts = dd2[ !duplicated(dd2$bgb) & dd2$IsZoonotic_Strict == 1 & dd2$Class == "Aves", ]
# birdnonhosts = dd2[ !duplicated(dd2$bgb) & dd2$IsZoonotic_Strict == 0 & dd2$Class == "Aves", ] 
# birdstudies = dd2[ !duplicated(dd2$SSS), ]
# 
# # mammal studies and zoonotic hosts: 58 studies, 196 hosts, 1930 sites
# dd3 = dd1[ dd1$Class == "Mammalia" & dd1$SS_Mammalsero == 1, ]
# mamhosts = dd3[ !duplicated(dd3$bgb) & dd3$Mammalsero_IsZoonotic_Strict == 1 & dd3$Class == "Mammalia", ]
# mamnonhosts = dd3[ !duplicated(dd3$bgb) & dd3$Mammalsero_IsZoonotic_Strict == 0 & dd3$Class == "Mammalia", ]
# mamstudies = dd3[ !duplicated(dd3$SSS), ]
# 
# 
# 
# 
# # ------------------ map data globally --------------------
# 
# # shapefile data
# cont = readOGR("./data/worldborders/continents/continent.shp", "continent")
# cont1 = fortify(cont[ cont$CONTINENT != "Antarctica", ])
# countries = readOGR("./data/worldborders/TM_WORLD_BORDERS-0.3/TM_WORLD_BORDERS-0.3.shp", "TM_WORLD_BORDERS-0.3")
# 
# # mark countries as 1/0 for whether PREDICTS studies are there or not
# countries$InPREDICTS = ifelse(countries$ISO2 %in% countrycode(birdstudies$Country, "country.name", "iso2c"), 1, 0)
# countries$InPREDICTS = ifelse(countries$ISO2 %in% countrycode(mamstudies$Country, "country.name", "iso2c"), 1, 0)
# 
# # split into two sections
# countries1 = fortify(countries[ countries$InPREDICTS == 1, ])
# countries2 = fortify(countries[ countries$InPREDICTS == 0 & countries$NAME != "Antarctica", ])
# 
# # also palegreen3
# global_map = ggplot() + coord_fixed() +
#   geom_polygon(data=countries1, aes(x=long, y=lat, group=group), fill="skyblue3", lwd=0.2, col="grey60") + 
#   geom_polygon(data=countries2, aes(x=long, y=lat, group=group), fill="grey90", lwd=0.2, col="grey60") + 
#   geom_polygon(data=cont1, aes(x=long, y=lat, group=group), fill=NA, lwd=0.2, col="grey10") + theme_classic() +
#   geom_point(data=birdstudies, aes(Longitude, Latitude), size=2, pch=16, stroke=0.2, col="red3") +
#   theme(panel.background = element_rect(fill=NA, color="black", size=1.2),
#         axis.text = element_blank(), 
#         axis.ticks = element_blank(), 
#         axis.title = element_blank())
# 
# ggsave(global_map, file="./output/figures/SI_Geomap_mammal_studies.png", width=12, height=5, units="in", device="png", dpi=600)
# 
# 
# # ---------------- plot taxonomic distribution of hosts ------------
# 
# # color ramp palette
# spec = colorRampPalette(brewer.pal(11, "Spectral"))
# 
# # create data for pie chart (birds)
# hosts = birdhosts
# hosts$Order[ ! hosts$Order %in% c("Passeriformes", "Psittaciformes", "Accipitriformes", "Anseriformes", "Columbiformes") ] = "Other"
# hosts = ddply(hosts, .(Order), summarize, sum=length(Order))
# hosts$fraction = hosts$sum / sum(hosts$sum)
# hosts = hosts[order(hosts$fraction), ]
# hosts$ymax = cumsum(hosts$fraction)
# hosts$ymin = c(0, head(hosts$ymax, n=-1))
# hosts$class3 = paste(hosts$Order, " (", hosts$sum, ")", sep="")
# 
# # plot birds
# (allhosts_pie = ggplot(hosts, aes(fill=Order, ymax=ymax, ymin=ymin, xmax=4, xmin=2.5)) +
#     geom_rect(colour="white", lwd=1.5) +
#     coord_polar(theta="y") +
#     xlim(c(0, 4)) +
#     theme_bw() +
#     theme(panel.grid=element_blank(),
#           axis.text=element_blank(), 
#           axis.ticks=element_blank(),
#           legend.text = element_text(size=13),
#           legend.title = element_blank(),
#           legend.position = c(0.5, 0.5),
#           panel.background = element_rect(fill=NA, color="black")) +
#     scale_fill_manual(values=c("Anseriformes" = "black",
#                                "Accipitriformes" = spec(11)[4],
#                                "Columbiformes" = spec(11)[11],
#                                "Psittaciformes" = spec(11)[9],
#                                "Passeriformes" = "indianred2",
#                                "Other" = "grey50")))
# 
# # birds
# ggsave(allhosts_pie, file="./output/figures/SI_Piechart_hosts_avian.png", width=4.8, height=4.8, units="in", device="png", dpi=600)
# 
# # create data for pie chart (mammals)
# hosts = mamhosts
# hosts$Order[ hosts$Order %in% c("Cingulata", "Hyracoidea", "Didelphimorphia", "Diprotodontia", "Lagomorpha", "Perissodactyla", "Pilosa", "Proboscidea") ] = "Other"
# hosts = ddply(hosts, .(Order), summarize, sum=length(Order))
# hosts$fraction = hosts$sum / sum(hosts$sum)
# hosts = hosts[order(hosts$fraction), ]
# hosts$ymax = cumsum(hosts$fraction)
# hosts$ymin = c(0, head(hosts$ymax, n=-1))
# hosts$class3 = paste(hosts$Order, " (", hosts$sum, ")", sep="")
# 
# # plot mammals
# (allhosts_pie = ggplot(hosts, aes(fill=Order, ymax=ymax, ymin=ymin, xmax=4, xmin=2.5)) +
#     geom_rect(colour="white", lwd=1.5) +
#     coord_polar(theta="y") +
#     xlim(c(0, 4)) +
#     theme_bw() +
#     theme(panel.grid=element_blank(),
#           axis.text=element_blank(), 
#           axis.ticks=element_blank(),
#           legend.text = element_text(size=13),
#           legend.title = element_blank(),
#           legend.position = c(0.5, 0.5),
#           panel.background = element_rect(fill=NA, color="black")) +
#     scale_fill_manual(values=c("Chiroptera" = "black",
#                                "Cetartiodactyla" = spec(11)[4],
#                                "Carnivora" = spec(11)[11],
#                                "Primates" = spec(11)[9],
#                                "Rodentia" = "indianred2",
#                                "Other" = "grey50")))
# 
# # mammals
# ggsave(allhosts_pie, file="./output/figures/SI_Piechart_hosts_Mammalsero.png", width=4.8, height=4.8, units="in", device="png", dpi=600)
# 
# 
# 
# 
# # ----------- per-study hosts as proportion of total sampled richness --------------
# 
# # colourscale
# v = viridis(250)
# 
# # prep data (birds)
# ss = ddply(dd2[ dd2$SS_Cargocarrier_Aves == 1, ], .(SS), plyr::summarize, 
#            SR = length(unique(bgb)),
#            HostSR = length(unique(bgb[IsZoonotic_Strict == 1 ])),
#            Biome = head(Biome, 1))
# #ss2 = ddply(dd2[ dd2$SS_Cargocarrier_Aves == 1 & dd2$IsZoonotic_Strict == 1, ], .(SS), plyr::summarize,
# #            HostSR = length(unique(bgb)))
# ss$PropHosts = ss$HostSR / ss$SR
# ss$TropTemp = ifelse(ss$Biome %in% c("Temperate Broadleaf & Mixed Forests", "Mediterranean Forests, Woodlands & Scrub", "Deserts & Xeric Shrublands", "Temperate Grasslands, Savannas & Shrublands", "Temperate Conifer Forests", "Montane Grasslands & Shrublands", "Tundra", "Boreal Forests/Taiga"),
#                      "Temperate", "Tropical")
# 
# # prep data (mammals)
# ss = ddply(dd3[ dd3$SS_Mammalsero == 1, ], .(SS), plyr::summarize, 
#            SR = length(unique(bgb)),
#            HostSR = length(unique(bgb[Mammalsero_IsZoonotic_Strict == 1 ])),
#            Biome = head(Biome, 1))
# ss$PropHosts = ss$HostSR / ss$SR
# ss$TropTemp = ifelse(ss$Biome %in% c("Temperate Broadleaf & Mixed Forests", "Mediterranean Forests, Woodlands & Scrub", "Deserts & Xeric Shrublands", "Temperate Grasslands, Savannas & Shrublands", "Temperate Conifer Forests", "Montane Grasslands & Shrublands", "Tundra", "Boreal Forests/Taiga"),
#                      "Temperate", "Tropical")
# 
# # plot
# perstudy_boxplots = ggplot(ss, aes(factor(TropTemp), PropHosts*100, fill=TropTemp)) + geom_boxplot() +
#   theme_bw() + xlab("") + ylab("Hosts % of per-study total species richness") + 
#   theme(legend.title = element_blank(),
#         axis.text.x = element_text(size=14),
#         axis.text.y = element_text(size=14),
#         axis.title.y = element_text(size=15),
#         legend.text = element_text(size=13), 
#         legend.position="none") +
#   ylim(0, 100) +
#   scale_fill_manual(values=c("Temperate" = v[120],
#                              "Tropical" = v[190]))
# 
# # for birds: 29 temperate, 51 tropical
# ggsave(perstudy_boxplots, file="./output/figures/SI_StudyBoxplots_avianstudies.png", width=3.25, height=5, units="in", device="png", dpi=600)
# 
# # for mammals: 13 temperate, 45 tropical
# ggsave(perstudy_boxplots, file="./output/figures/SI_Studyboxplots_mammalstudies.png", width=3.25, height=5, units="in", device="png", dpi=600)
# 
