
### Gibb et al, 'Land use has global and systematic effects on local zoonotic host communities"
# Script 5: For mammals, derive-species level estimates of pathogen richness controlling for research effort
# From models of publication effort against pathogen richness for human/nonhuman pathogens
# Used to model against land use in pathogen richness models

setwd("/Volumes/ritd-ag-project-rd00nx-rjgib11/chapter2_predicts/predicts_analysis_201901/")
setwd("/Users/Rory/Dropbox/PhD/chapter_predicts/predicts_analysis_201901/")
setwd("Y:/chapter2_predicts/predicts_analysis_201901/")

# dependencies
library(INLA)
library(lme4)
library(sp)
library(raster)
library(rgdal)
library(ggplot2)
library(nlme)
library(dismo)
library(Hmisc)
library(plyr)
library(dplyr)
library(gridExtra)

# load functions
source("00_functions_general.R")



# ---------------- data ------------------

# load PREDICTS/pathogens data
load(file="./data/predicts_processed/predicts_pathogens_researcheffort.R")

# data on all species mammals (n=780) with serological h-p association data for modelling against publication effort
res = read.csv("./data/researcheffort_approx/MammalHosts_PathRichData_forpublicationmodels.csv", stringsAsFactors = FALSE)

# get same info from PREDICTS database to predict on species
# fitting to pathogens database, predicting to ecological data
spp2 = dd1[ dd1$Class == "Mammalia" & !duplicated(dd1$bgb), c("bgb", "Class", "Order", "Family", "NumPubs", "Sero_PathRich_HumanInfective", "Sero_PathRich_NonHumanDom")]
spp2$logPubs = log(spp2$NumPubs+1)
# spp2$Sero_PathRich_HumanInfective = NA
# spp2$Sero_PathRich_NonHumanDom = NA
spp2$pred = 1; res$pred = 0
res = rbind.data.frame(res, spp2)

# set response variables
# set responses for PREDICTS to NA for prediction via INLA
res$PR_Human = res$Sero_PathRich_HumanInfective
res$PR_NonHuman = res$Sero_PathRich_NonHumanDom
res$PR_Human[ res$pred == 1 ] = NA
res$PR_NonHuman[ res$pred == 1 ] = NA

# remove procyon lotor from fitting data (extremely large outlier in non-human model with high leverage given publication effort)
res = res[ -which(res$bgb == "procyon lotor" & res$pred == 0), ]




# ------------- specify model and run inference ---------------

# precision prior for iid effect (log-gamma, 0.1/0.1)
hyper1.iid = list(prec = list(prior='loggamma', param=c(0.1,0.1)))

# uninformative priors for mu and theta on fixed effects (mean 0, sd 100)
control.fixed1 = list(mean.intercept=0, mean = 0, prec = 0.01, prec.intercept = 0.001)

# dummy variables for random slopes
res$logPubsx = res$logPubs
res$logPubsy = res$logPubs
res$Orderx = res$Order
res$Familyx = res$Family

# fit model for human-shared pathogens
formx = formula(PR_Human ~ 1 + logPubs + f(Family, model='iid', hyper=hyper1.iid) + 
                  f(Order, model='iid', hyper=hyper1.iid) + 
                  f(Orderx, logPubsx, model='iid', hyper=hyper1.iid) + 
                  f(Familyx, logPubsy, model='iid', hyper=hyper1.iid))
m1 = inla(formx, data=res,
          family="poisson", 
          control.predictor=list(compute=TRUE, link=TRUE),
          control.compute=list(dic=T, waic=T, config=TRUE),
          control.fixed=control.fixed1)

# fit model for non human-shared pathogens
formx = formula(PR_NonHuman ~ 1 + logPubs + f(Family, model='iid', hyper=hyper1.iid) + 
                  f(Order, model='iid', hyper=hyper1.iid) + 
                  f(Orderx, logPubsx, model='iid', hyper=hyper1.iid) + 
                  f(Familyx, logPubsy, model='iid', hyper=hyper1.iid))
m2 = inla(formx, data=res,
          family="poisson", 
          control.predictor=list(compute=T, link=TRUE),
          control.compute=list(dic=T, waic=T, config=TRUE),
          control.fixed=control.fixed1)



# ----------- examine model fit -------------

# extract observed versus fitted
of = data.frame(bgb = res$bgb, obs_human = res$PR_Human, obs_nh=res$PR_NonHuman, 
                fitted_human=m1$summary.fitted.values$mean, fitted_nh=m2$summary.fitted.values$mean, 
                Order=as.character(res$Order), Family=as.character(res$Family), logPubs = res$logPubs, pred=res$pred)

# distribution of raw and standardised residuals: overdispersed towards upper end but likely due to sparse data at upper end
# i.e. few species with very many pathogens, some with very few
hist(of$obs_human - of$fitted_human, 100)
hist(of$obs_nh - of$fitted_nh, 100)
hist((of$obs_human - of$fitted_human) / sqrt(of$fitted_human), 100)
hist((of$obs_nh - of$fitted_nh) / sqrt(of$fitted_nh), 100)

# calculate residuals
of$resid_h = of$obs_human - of$fitted_human
of$resid_nh = of$obs_nh - of$fitted_nh




# ----------- extract predictions and combine with PREDICTS db ---------------

# get values for spp in PREDICTS
of2 = data.frame(bgb = res$bgb, obs_human = res$Sero_PathRich_HumanInfective, obs_nh=res$Sero_PathRich_NonHumanDom, 
                fitted_human=m1$summary.fitted.values$mean, fitted_nh=m2$summary.fitted.values$mean, 
                Order=as.character(res$Order), Family=as.character(res$Family), logPubs = res$logPubs, pred=res$pred)

# set residuals to NA for large outliers (not to be included in later models) and scale residuals
preds = of2[ of2$pred == 1, c("bgb", "obs_human", "obs_nh", "fitted_human", "fitted_nh", "Order") ]
preds[ preds$bgb %in% c("axis axis", "indicator indicator", "canis lupus", "procyon lotor"), 4:5] = NA
names(preds)[ 4:5 ] = c("PR_Human_fitted", "PR_NonHuman_fitted")
preds$ResidPathRich_Human = scale(preds$obs_human - preds$PR_Human_fitted)
preds$ResidPathRich_NonHuman = scale(preds$obs_nh - preds$PR_NonHuman_fitted)

# combine with predicts
dd1 = left_join(dd1, preds[ , c("bgb", "Order", "PR_Human_fitted", "PR_NonHuman_fitted", "ResidPathRich_Human", "ResidPathRich_NonHuman")], by=c("bgb", "Order"))

# save
save(dd1, file="./output/predicts_processed/predicts_pathogens_researcheffort.R")




# ----------- examine uncertainty in residuals ------------

# function to get fitted samples for all species
# model is INLA model
# pred specifies if using fitting or predicting data
# calc_resid: calculate residual or not?
# type - human or non human
extractPredSamples = function(model, n=3000, pred, calc_resid=FALSE, type){
  
  # get samples
  samp = inla.posterior.sample(n, model)
  
  # get samples for all species
  getSampPredictor = function(x, samp, pred=pred, calc_resid, type){
    sx = samp[[x]]
    sxx = sx$latent[ 1:nrow(res) ] # extract fitted values on the linear predictor scale
    resx = res
    resx$sampx = exp(sxx) # exp to link
    
    # if calc_resid == TRUE calculate residual from 
    if(calc_resid == TRUE & type == "human"){ 
      resx$sampx = resx$Sero_PathRich_HumanInfective - resx$sampx 
    } else if(calc_resid == TRUE & type == "nonhuman"){ 
      resx$sampx = resx$Sero_PathRich_NonHumanDom - resx$sampx 
      } else{ return("Err: specify human or nonhuman model if calculating residuals") }
    
    # return
    if(pred==0){ resx = resx[ resx$pred == 0, ] }
    if(pred==1){ resx = resx[ resx$pred == 1, ] }
    sxx = resx[ , c("bgb", "sampx")]
    names(sxx)[2] = paste("samp_", x, sep="")
    if(x == 1){ return(sxx) } else{ return(sxx[ , 2, drop=F])}
  }
  
  s1 = do.call(cbind.data.frame, lapply(1:length(samp), getSampPredictor, samp=samp, pred=pred, calc_resid=calc_resid, type=type))
  
  # rearrange and return
  row.names(s1) = s1$bgb
  s1 = t(s1[ , -1 ])
  return(s1)
}

# extract all samples for species in PREDICTS
samp_h = extractPredSamples(m1, 3000, pred=1, calc_resid=TRUE, type="human")
samp_nh = extractPredSamples(m2, 3000, pred=1, calc_resid=TRUE, type="nonhuman")

# save for testing model sensitivity to residual uncertainty
write.csv(samp_h, "./output/model_outputs/dec_2019/pathrich_mammals/residual_models_researcheffort/mammalia_human_residualPR_samples.csv", row.names=F)
write.csv(samp_nh, "./output/model_outputs/dec_2019/pathrich_mammals/residual_models_researcheffort/mammalia_nonhuman_residualPR_samples.csv", row.names=F)



### examine and visualise residual distributions

# calculate intervals 
df1 = data.frame(
  bgb = colnames(samp_h),
  lower_95 = apply(samp_h, 2, quantile, 0.025),
  lower_67 = apply(samp_h, 2, quantile, 0.167),
  median = apply(samp_h, 2, median),
  upper_67 = apply(samp_h, 2, quantile, 0.833),
  upper_95 = apply(samp_h, 2, quantile, 0.975)
)
df1 = left_join(df1, res[ res$pred == 1, c("bgb", "Order", "NumPubs")])

# classify orders
df1$Order2 = df1$Order
df1$Order2[! df1$Order2 %in% c("Chiroptera", "Carnivora", "Cetartiodactyla", "Primates", "Rodentia") ] = "Other"
df1 = df1[ order(df1$Order2, df1$median), ]
df1$bgb = factor(df1$bgb, levels=df1$bgb, ordered=T)

# scale around 0 (as for model fitting later)
df1[ , 2:6 ] = (df1[ , 2:6 ] - mean(df1$median)) / sd(df1$median)

# # plot with 95 and 67 CI
# resid_disth = ggplot(df1, aes(bgb, median, col=Order2)) +
#   geom_point() +
#   geom_hline(yintercept=5, lty=1, col="grey90") +
#   geom_hline(yintercept=-5, lty=1, col="grey90") +
#   geom_errorbar(aes(ymin=lower_95, ymax=upper_95), lwd=0.1, width=0.01, alpha=0.4, show.legend = FALSE) +
#   geom_linerange(aes(ymin=lower_67, ymax=upper_67), lwd=0.2, show.legend = FALSE) +
#   geom_hline(yintercept=0, lty=2) +
#   theme_bw() +
#   theme(axis.text.x = element_blank(),
#         axis.title = element_text(size=axis_title_size),
#         axis.text.y = element_text(size=axis_text_size),
#         axis.ticks.x = element_blank(),
#         panel.grid = element_blank(),
#         legend.title=element_blank(),
#         legend.position=c(0.85, 0.175)) +
#   ylab("Residual PR (human-shared)")

### non human-shared

# calculate intervals 
df2 = data.frame(
  bgb = colnames(samp_nh),
  lower_95 = apply(samp_nh, 2, quantile, 0.025),
  lower_67 = apply(samp_nh, 2, quantile, 0.167),
  median = apply(samp_nh, 2, median),
  upper_67 = apply(samp_nh, 2, quantile, 0.833),
  upper_95 = apply(samp_nh, 2, quantile, 0.975)
)
df2 = left_join(df2, res[ res$pred == 1, c("bgb", "Order", "NumPubs")])

# classify orders
df2$Order2 = df2$Order
df2$Order2[! df2$Order2 %in% c("Chiroptera", "Carnivora", "Cetartiodactyla", "Primates", "Rodentia") ] = "Other"
df2 = df2[ order(df2$Order2, df2$median), ]
df2$bgb = factor(df2$bgb, levels=df2$bgb, ordered=T)

# scale around 0 (as for model fitting later)
df2[ , 2:6 ] = (df2[ , 2:6 ] - mean(df2$median)) / sd(df2$median)

# plot
# resid_distnh = ggplot(df2[ -which(df2$bgb == "procyon lotor"), ], aes(bgb, median, col=Order2)) +
#   geom_point() +
#   geom_hline(yintercept=5, lty=1, col="grey90") +
#   geom_hline(yintercept=-5, lty=1, col="grey90") +
#   geom_errorbar(aes(ymin=lower_95, ymax=upper_95), lwd=0.1, width=0.01, alpha=0.4, show.legend = FALSE) +
#   geom_linerange(aes(ymin=lower_67, ymax=upper_67), lwd=0.2, show.legend = FALSE) +
#   geom_hline(yintercept=0, lty=2) +
#   theme_bw() +
#   theme(axis.text.x = element_blank(),
#         axis.title = element_text(size=axis_title_size),
#         axis.text.y = element_text(size=axis_text_size),
#         axis.ticks.x = element_blank(),
#         panel.grid = element_blank(),
#         legend.title=element_blank(),
#         legend.position=c(0.85, 0.175)) +
#   ylab("Residual PR (non-human)") +
#   xlab("Species")





# ------------- combine and visualise model outputs for MS -------------

### plot headers
axis_text_size = 12
axis_title_size = 14
legend_text_size = 14
point_cex = 1.2
alpha_cex = 0.4

# 1. Distribution of pathogen richness human vs nonhuman: correlated but lots of variation
pr_plot = ggplot(res, aes(PR_Human, PR_NonHuman)) +
  geom_point(col="black", cex=point_cex, alpha=alpha_cex) +
  theme_bw() +
  #theme_classic() +
  coord_fixed() +
  xlim(0, 80) + ylim(0, 80) +
  xlab("PR (human-shared)") + ylab("PR (non-human)") +
  theme(panel.grid.minor = element_blank(),
        axis.text=element_text(size=axis_text_size),
        axis.title=element_text(size=axis_title_size))

# 2. Pathogen richness against publication effort
res2 = res
res2$Order2 = res2$Order
res2$Order2[! res2$Order2 %in% c("Chiroptera", "Carnivora", "Cetartiodactyla", "Primates", "Rodentia") ] = "Other"

# a. humans
ph = ggplot() +
  geom_point(data=res2, aes(logPubs, PR_Human, col=Order2),  size=point_cex) +
  theme_bw() + xlab("log Publications") + ylab("PR (human-shared)") +
  theme(axis.text = element_text(size=axis_text_size),
        axis.title = element_text(size=axis_title_size),
        legend.title=element_blank(),
        legend.text=element_text(size=legend_text_size), 
        legend.background = element_rect(fill=NA),
        #legend.position=c(0.24, 0.6),
        legend.position="none")

# b. non humans
pnh = ggplot() +
  geom_point(data=res2, aes(logPubs, PR_NonHuman, col=Order2),  size=point_cex) +
  theme_bw() + xlab("log Publications") + ylab("PR (non-human)") +
  theme(axis.text = element_text(size=axis_text_size),
        axis.title = element_text(size=axis_title_size),
        legend.title=element_blank(),
        legend.text=element_text(size=axis_text_size), 
        legend.position="none")

# 3. Observed versus fitted plots from fitted model
ofh_plot = ggplot(of, aes(fitted_human, obs_human)) + 
  geom_point(size=point_cex, alpha=alpha_cex) +
  coord_fixed() +
  geom_abline(slope=1, intercept=0, col="darkred") +
  xlim(0, 80) + ylim(0, 80) +
  theme_bw() + xlab("Fitted (human-shared)") + ylab("Observed (human-shared)") +
  theme(legend.position = "none",
        panel.grid.major = element_line(),
        panel.grid.minor = element_blank(),
        axis.text = element_text(size=axis_text_size),
        axis.title = element_text(size=axis_title_size))
ofnh_plot = ggplot(of, aes(fitted_nh, obs_nh)) + 
  geom_point(size=point_cex, alpha=alpha_cex) +
  coord_fixed() +
  geom_abline(slope=1, intercept=0, col="darkred") +
  xlim(0, 80) + ylim(0, 80) +
  theme_bw() + xlab("Fitted (non-human)") + ylab("Observed (non-human)") +
  theme(legend.position = "none",
        panel.grid.major = element_line(),
        panel.grid.minor = element_blank(),
        axis.text = element_text(size=axis_text_size),
        axis.title = element_text(size=axis_title_size))

# 4. Per-species comparison of human/non-human residuals (scaled as for modelling)
resid_plot = ggplot(of, aes(scale(resid_h), scale(resid_nh))) + 
  geom_point(size=point_cex, alpha=alpha_cex) +
  coord_fixed() +
  geom_hline(yintercept=0, lty=2) + geom_vline(xintercept=0, lty=2) +
  theme_bw() + xlab("Residual PR (human-shared)") + ylab("Residual PR (non-human)") +
  xlim(-10, 10) + ylim(-10, 10) +
  theme(legend.position = "none",
        panel.grid.major = element_line(),
        panel.grid.minor = element_blank(),
        axis.text = element_text(size=axis_text_size),
        axis.title = element_text(size=axis_title_size))


# 5. Residual uncertainty at the species level for human/non-human shared pathogen models 

# combine residual summaries calculated above
dfx = df1; dfx$type = "Human-shared pathogens"; dfx$bgb2 = paste(dfx$bgb, "h", sep="_")
dfy = df2; dfy$type = "Non-human-shared pathogens"; dfy$bgb2 = paste(dfy$bgb, "nh", sep="_")
dfz = rbind(dfx, dfy)
dfz = dfz[ order(dfz$type, dfz$Order2, dfz$median), ]
dfz$bgb2 = factor(dfz$bgb2, levels=dfz$bgb2, ordered=T)
dfz = dfz[ !dfz$bgb %in% c("canis lupus", "procyon lotor"), ] # remove canis lupus and procyon lotor as with model fit: extreme outliers

# plot with facet for type
resid_dist = ggplot(dfz, aes(bgb2, median, col=Order2)) +
  geom_point(cex=0.65) +
  geom_errorbar(aes(ymin=lower_95, ymax=upper_95), lwd=0.3, width=0.01, alpha=0.4, show.legend = FALSE) +
  geom_linerange(aes(ymin=lower_67, ymax=upper_67), lwd=0.36, show.legend = FALSE) +
  geom_hline(yintercept=0, lty=2) +
  theme_bw() +
  facet_wrap(~type, nrow=2, scales="free") +
  theme(axis.text.x = element_blank(),
        axis.title = element_text(size=axis_title_size),
        axis.title.x = element_blank(),
        strip.text = element_text(size=axis_title_size),
        axis.text.y = element_text(size=axis_text_size),
        axis.ticks.x = element_blank(),
        panel.grid = element_blank(),
        panel.grid.major.y = element_line(color="grey90"),
        legend.title=element_blank(),
        legend.background = element_rect(fill=NA, color="black"),
        legend.position="bottom",
        legend.text = element_text(size=legend_text_size)) +
  guides(colour = guide_legend(nrow=1, override.aes = list(size=2.5))) +
  ylab("Residual pathogen richness")

## plot all in multi panel
dp = gridExtra::arrangeGrob(grobs = list(pr_plot, ph, pnh, ofh_plot, ofnh_plot, resid_plot, resid_dist),
                            widths=c(1, 1, 1), heights=c(1, 1, 2),
                            layout_matrix = rbind(c(1, 2, 3),
                                                  c(4, 5, 6),
                                                  c(7, 7, 7)))
dp = ggpubr::as_ggplot(dp) +
  cowplot::draw_plot_label(label = c("A", "B", "C", "D", "E", "F", "G", "H"),
                           fontface = "bold", size = 22, 
                           x = c(0.055, 0.39, 0.72, 0.055, 0.39, 0.73, 0.06, 0.06), 
                           y = c(0.985, 0.995, 0.995, 0.732, 0.732, 0.732, 0.475, 0.25))
ggsave(dp, filename="./output/figures/nature_ms3_jan2019/ExtendedData_PathRichResiduals2.png",
       device="png", width=9, height=13.5, units="in", dpi=600)








# 
# # --------------- subset to species to model effort for --------------
# 
# # unique species in PREDICTS and within taxonomic families that are relevant
# ddx = dd1[ !duplicated(dd1$bgb), c("bgb", "Kingdom", "Phylum", "Class", "Order", "Family", "Genus", 
#                                    "NumPubs", "HumanInfective_Any", "IsZoonotic_Strict", 
#                                    "PathRich_HumanInfective", "PathRich_NonHuman", "PathRich_NonHumanDom",
#                                    "Sero_PathRich_HumanInfective", "Sero_PathRich_NonHumanDom",
#                                    "DetectionMethod", "DetectionStringency", "Sources")]
# 
# # species within relevant studies and taxonomic families
# ddy = ddx[ ddx$bgb %in% as.character(unique(dd1$bgb[ dd1$SS_HIAMB == 1 ])), ]
# 
# # calculate total pathogen richness
# ddy$PathRich_Total = ddy$PathRich_HumanInfective + ddy$PathRich_NonHuman
# 
# # log-log plot of pathogen richness and publication effort
# # plot(log(ddy$NumPubs+1), log(ddy$PathRich_Total+1))
# # plot(log(ddy$PathRich_Total+1), ddy$PathRich_HumanInfective)
# # plot(log(ddy$PathRich_Total+1), ddy$HumanInfective_Any)
# 
# 
# 
# # --------- Model research effort against pathogen richness for mammals only (for pathogen richness models) ------------
# 
# # For mammalian pathogen richness models, we know that pathogen richness increases as a function of accrued research/discovery effort 
# # Across n=546 mammal species in the PREDICTS database
# # Shown for many species level macroecology studies in host-pathogen ecology
# # So we want to account for the degree to which research effort is driving the accrual of pathogen richness
# # But also measure the variability in pathogen richness that is not explained by research effort alone (e.g. species w/ high research effort and
# # lower than expected richness, or vice versa)
# # To do this we model pathogen richness as a function of log research effort, Poisson error distribution
# # then derive residuals from the fitted model, i.e. pathogen richness not explained by research effort
# 
# # mammal species only and remove canis lupus (domestic bias)
# ddy = ddx[ ddx$Class == "Mammalia"  & ddx$bgb != "canis lupus" , ]
# 
# # derive predictor and response variables (log publications, scaled pathogen richness)
# ddy$PR_Human = ddy$Sero_PathRich_HumanInfective
# ddy$PR_NonHuman = ddy$Sero_PathRich_NonHumanDom
# ddy$logPubs = log(ddy$NumPubs+1)
# ddy = ddy[ !is.na(ddy$logPubs), ]
# 
# # extra variables for random slopes
# ddy$logPubsx = ddy$logPubs
# ddy$logPubsy = ddy$logPubs
# ddy$Orderx = ddy$Order
# ddy$Familyx = ddy$Family
# 
# # plot pathogen richness against publication effort
# ddy$Order2 = ddy$Order
# ddy$Order2[! ddy$Order2 %in% c("Chiroptera", "Carnivora", "Cetartiodactyla", "Primates", "Rodentia") ] = "Other"
# 
# # humans
# ph = ggplot() +
#   geom_point(data=ddy, aes(logPubs, Sero_PathRich_HumanInfective, col=Order2),  size=0.9) +
#   theme_bw() + xlab("log Publications") + ylab("Pathogen richness (human)") +
#   theme(axis.text = element_text(size=10),
#         axis.title.y = element_text(size=10.5),
#         axis.title.x = element_text(size=10.5),
#         legend.title=element_blank(),
#         legend.text=element_text(size=7), 
#         legend.position=c(0.2, 0.6))
# 
# # non humans
# pnh = ggplot() +
#   geom_point(data=ddy, aes(logPubs, Sero_PathRich_NonHumanDom, col=Order2), size=0.9) +
#   theme_bw() + xlab("log Publications") + ylab("Pathogen richness (non-human)") +
#   theme(axis.text = element_text(size=10),
#         axis.title.y = element_text(size=10.5),
#         axis.title.x = element_text(size=10.5), 
#         legend.position="none")
# 
# # combine
# phh = grid.arrange(ph, pnh, nrow=1)
# 
# 
# 
# # specify model and run inference
# 
# # precision prior for iid effect (log-gamma, 0.1/0.1)
# hyper1.iid = list(prec = list(prior='loggamma', param=c(0.1,0.1)))
# 
# # uninformative priors for mu and theta on fixed effects (mean 0, sd 100)
# control.fixed1 = list(mean.intercept=0, mean = 0, prec = 0.01, prec.intercept = 0.001)
# 
# # fit model (human-shared)
# formx = formula(PR_Human ~ 1 + logPubs + f(Family, model='iid', hyper=hyper1.iid) + 
#                   f(Order, model='iid', hyper=hyper1.iid) + 
#                   f(Orderx, logPubsx, model='iid', hyper=hyper1.iid) + 
#                   f(Familyx, logPubsy, model='iid', hyper=hyper1.iid))
# m1x = inla(formx, data=ddy,
#            family="poisson", 
#            control.predictor=list(compute=T),
#            control.compute=list(dic=T, waic=T),
#            control.fixed=control.fixed1)
# 
# # fit model (non human-shared)
# formx = formula(PR_NonHuman ~ 1 + logPubs + f(Family, model='iid', hyper=hyper1.iid) + 
#                   f(Order, model='iid', hyper=hyper1.iid) + 
#                   f(Orderx, logPubsx, model='iid', hyper=hyper1.iid) + 
#                   f(Familyx, logPubsy, model='iid', hyper=hyper1.iid))
# m2x = inla(formx, data=ddy,
#            family="poisson", 
#            control.predictor=list(compute=T),
#            control.compute=list(dic=T, waic=T),
#            control.fixed=control.fixed1)
# 
# # view observed versus fitted (human)
# df1 = data.frame(obs = ddy$PR_Human, fitted = m1x$summary.fitted.values$mean, Order=as.character(ddy$Order), logPubs = ddy$logPubs)
# obsfit_human = ggplot(df1, aes(obs, fitted)) + 
#   geom_point(size=0.8) +
#   geom_abline(slope=1, intercept=0) +
#   theme_bw() + ylab("Predicted (human)") + xlab("Observed (human)") +
#   theme(legend.position = "none",
#         panel.grid.major = element_line(),
#         panel.grid.minor = element_blank(),
#         axis.text = element_text(size=10),
#         axis.title.y = element_text(size=10.5),
#         axis.title.x = element_text(size=10.5))
# 
# # observed versus fitted (nonhuman)
# df2 = data.frame(obs = ddy$PR_NonHuman, fitted = m2x$summary.fitted.values$mean, Order=as.character(ddy$Order), logPubs = ddy$logPubs)
# obsfit_nonhuman = ggplot(df2, aes(obs, fitted)) + 
#   geom_point(size=0.8) +
#   geom_abline(slope=1, intercept=0) +
#   theme_bw() + ylab("Predicted (non-human)") + xlab("Observed (non-human)") +
#   theme(legend.position = "none",
#         panel.grid.major = element_line(),
#         panel.grid.minor = element_blank(),
#         axis.text = element_text(size=10),
#         axis.title.y = element_text(size=10.5),
#         axis.title.x = element_text(size=10.5))
# of = grid.arrange(obsfit_human, obsfit_nonhuman, nrow=1)
# 
# # arrange all
# plot1 = grid.arrange(ph, obsfit_human, pnh, obsfit_nonhuman, nrow=2)
# 
# 
# 
# # save residual outputs for later modelling
# result = data.frame(bgb = ddy$bgb,
#                     Order = ddy$Order,
#                     Class = ddy$Class,
#                     logPubs = ddy$logPubs,
#                     ResidPathRich_Human = df1$obs - df1$fitted,
#                     ResidPathRich_NonHuman = df2$obs - df2$fitted)
# write.csv(result, "./output/model_outputs/effort_weighting/pathogenrichness_residuals_allmammals.csv", row.names=F)
# 
# resx = ggplot(result, aes(ResidPathRich_Human, ResidPathRich_NonHuman)) +
#   geom_point(size=1) + theme_bw() + 
#   geom_vline(xintercept=0, lty=2) + 
#   geom_hline(yintercept=0, lty=2) +
#   xlab("Residual pathogen richness (human)") + 
#   ylab("Residual pathogen richness (nonhuman)") +
#   theme(legend.position = "none",
#         panel.grid.major = element_line(),
#         panel.grid.minor = element_blank(),
#         axis.text = element_text(size=10),
#         axis.title.y = element_text(size=10.5),
#         axis.title.x = element_text(size=10.5))
# 
# plot2 = grid.arrange(grobs = list(ph, obsfit_human, pnh, obsfit_nonhuman, resx), widths=c(1, 1, 2),
#                      layout_matrix = rbind(c(1, 2, 5),
#                                            c(3, 4, 5)))
# ggsave(plot2, file="./output/figures/SI_PathRichness_summarystats_sero_wlegend.png",
#        device="png", dpi=1200, units="in", width=11, height=5, scale=1.1)
# 
# 
# 
# # ------------- same but for all mammals with orders represented in PREDICTS --------------
# 
# # orders
# oo = unique(dd1$Order[ dd1$Class == "Mammalia" ])
# 
# # read mammal host/pathogen list for all mammals
# res = read.csv("./output/researcheffort_approx/Publications_HostStatus_allmammalsbirds_20191218.csv", stringsAsFactors = FALSE)
# 
# # subset to mammals, remove NA rows, any synonyms and any orders that do not appear in PREDICTS
# res = res[ res$IsSynonym != TRUE, ]
# res = res[ !is.na(res$NumPubs), ]
# res = res[ !is.na(res$bgb) & !is.na(res$Order) & !is.na(res$Class), ]
# res = res[ res$Class == "Mammalia", ]
# res = res[ res$bgb != "homo sapiens", ]
# res = res[ res$Order %in% oo, ]
# 
# # relevant columns for modelling
# res = res[ , c("bgb", "Class", "Order", "Family", "NumPubs", "Sero_PathRich_HumanInfective", "Sero_PathRich_NonHumanDom")]
# res$logPubs = log(res$NumPubs+1)
# 
# # relationships
# plot(res$logPubs, res$Sero_PathRich_HumanInfective)
# plot(res$logPubs, res$Sero_PathRich_NonHumanDom)
# 
# # get same info from dd1 (to predict on species) and set responses to NA
# spp2 = dd1[ dd1$Class == "Mammalia" & !duplicated(dd1$bgb), c("bgb", "Class", "Order", "Family", "NumPubs", "Sero_PathRich_HumanInfective", "Sero_PathRich_NonHumanDom")]
# spp2$logPubs = log(spp2$NumPubs+1)
# spp2$Sero_PathRich_HumanInfective = NA
# spp2$Sero_PathRich_NonHumanDom = NA
# spp2$pred = 1; res$pred = 0
# res = rbind.data.frame(res, spp2)
# 
# # extra variables for random slopes
# res$logPubsx = res$logPubs
# res$logPubsy = res$logPubs
# res$Orderx = res$Order
# res$Familyx = res$Family
# 
# # response variables
# res$PR_Human = res$Sero_PathRich_HumanInfective
# res$PR_NonHuman = res$Sero_PathRich_NonHumanDom
#  
# 
# 
# # specify model and run inference
# 
# # precision prior for iid effect (log-gamma, 0.1/0.1)
# hyper1.iid = list(prec = list(prior='loggamma', param=c(0.1,0.1)))
# 
# # uninformative priors for mu and theta on fixed effects (mean 0, sd 100)
# control.fixed1 = list(mean.intercept=0, mean = 0, prec = 0.01, prec.intercept = 0.001)
# 
# # fit model (human-shared)
# formx = formula(PR_Human ~ 1 + logPubs + f(Family, model='iid', hyper=hyper1.iid) + 
#                   f(Order, model='iid', hyper=hyper1.iid) + 
#                   f(Orderx, logPubsx, model='iid', hyper=hyper1.iid) + 
#                   f(Familyx, logPubsy, model='iid', hyper=hyper1.iid))
# m1y = inla(formx, data=res,
#            family="poisson", 
#            control.predictor=list(compute=TRUE, link=TRUE),
#            control.compute=list(dic=T, waic=T),
#            control.fixed=control.fixed1)
# 
# # fit model (non human-shared)
# formx = formula(PR_NonHuman ~ 1 + logPubs + f(Family, model='iid', hyper=hyper1.iid) + 
#                   f(Order, model='iid', hyper=hyper1.iid) + 
#                   f(Orderx, logPubsx, model='iid', hyper=hyper1.iid) + 
#                   f(Familyx, logPubsy, model='iid', hyper=hyper1.iid))
# m2y = inla(formx, data=res,
#            family="poisson", 
#            control.predictor=list(compute=T, link=TRUE),
#            control.compute=list(dic=T, waic=T),
#            control.fixed=control.fixed1)
# 
# # view observed versus fitted (human)
# df1y = data.frame(bgb = res$bgb, obs_human = res$PR_Human, obs_nh=res$PR_NonHuman, 
#                   fitted_human=m1y$summary.fitted.values$mean, fitted_nh=m2y$summary.fitted.values$mean, 
#                   Order=as.character(res$Order), Family=as.character(res$Family), logPubs = res$logPubs, pred=res$pred)
# df1y$sresid_human = (df1y$obs_human - df1y$fitted_human) / sqrt(df1y$fitted_human)
# df1y$sresid_nh = df1y$obs_nh - df1y$fitted_nh
# 
# obsfit_human = ggplot(df1y, aes(fitted_human, obs_human)) + 
#   geom_point(size=0.8) +
#   coord_fixed() +
#   geom_abline(slope=1, intercept=0) +
#   theme_bw() + xlab("Predicted (human)") + ylab("Observed (human)") +
#   theme(legend.position = "none",
#         panel.grid.major = element_line(),
#         panel.grid.minor = element_blank(),
#         axis.text = element_text(size=10),
#         axis.title.y = element_text(size=10.5),
#         axis.title.x = element_text(size=10.5))
# 
# # observed versus fitted (nonhuman)
# obsfit_nonhuman = ggplot(df1y, aes(obs_nh, fitted_nh)) + 
#   geom_point(size=0.8) +
#   geom_abline(slope=1, intercept=0) +
#   coord_fixed() +
#   theme_bw() + ylab("Predicted (non-human)") + xlab("Observed (non-human)") +
#   theme(legend.position = "none",
#         panel.grid.major = element_line(),
#         panel.grid.minor = element_blank(),
#         axis.text = element_text(size=10),
#         axis.title.y = element_text(size=10.5),
#         axis.title.x = element_text(size=10.5))
# of = grid.arrange(obsfit_human, obsfit_nonhuman, nrow=1)
# 
# # arrange all
# plot1 = grid.arrange(ph, obsfit_human, pnh, obsfit_nonhuman, nrow=2)
# 
# ### combine preds with predicts
# 
# dff = df1y[ df1y$pred == 1, c("bgb", "fitted_human", "fitted_nh", "Order")]
# ddm = left_join(dd1[ dd1$Class == "Mammalia" & dd1$bgb != "canis lupus", ], dff)
# ddm$ResidPR_Human = ddm$Sero_PathRich_HumanInfective - ddm$fitted_human
# ddm$ResidPR_NonHuman = ddm$Sero_PathRich_NonHumanDom - ddm$fitted_nh
# 

ddm = dd1[ dd1$Class == "Mammalia" & (!dd1$bgb %in% c("canis lupus", "procyon lotor")), ]
ddm = prepareLandUse(ddm)
ddm$Presence = ifelse( ddm$Effort_corrected_measurement > 0, 1, 0 )
ddm$logAbund = log(ddm$Effort_corrected_measurement+1)

# remove urban
# 66 studies, 1950 sites, 546 species
dat = ddm[ ddm$LandUseType != "Urban", ]
printDetails(dat)


# subset to only columns of interest, set up response variable
model_dat = dat[ , c("Presence", "ResidPathRich_Human", "ResidPathRich_NonHuman", "LandUseType", "SS", "SSB", "SSBS", "bgb", "Order") ]
names(model_dat)[1] = "y"

# managed land as base factor for models
rep = data.frame(LandUseType = c("1_Primary", "Secondary", "Managed"), LandUseType2 = c("Primary", "Secondary", "1_Managed"))
model_dat = full_join(model_dat, rep)

# nest species within study for random effects
model_dat$bgb_SS = paste(model_dat$bgb, model_dat$SS, sep="_")

# inla model specifications (binomial, logit link)
family1 = "binomial"
control.family1 = list(control.link=list(model="logit"))

# hyperprior (penalised complexity or log-gamma)
hyper1 = list(prec = list(prior='loggamma', param=c(0.1,0.1)))

# model specification and inference for human shared
# random intercepts for study, spatial block, species-study and order
# considered random slopes (PathRich|Order) but parameter estimates and confidence didn't change, and delta AIC was marginal (<6) - opted for simpler model
model_dat$PathRich = model_dat$ResidPathRich_Human
form = formula(y ~ 1 + PathRich*LandUseType2 +
                 f(SS, model='iid', hyper=hyper1) +
                 f(SSB, model='iid', hyper=hyper1) +
                 f(bgb_SS, model='iid', hyper=hyper1) +
                 f(Order, model='iid', hyper=hyper1))
inla.hmod = inla(form, data=model_dat, family=family1,
                 control.predictor=list(compute=T), # predict values (not on link scale)
                 control.compute=list(cpo=T, waic=T, config=TRUE), # specify config == T so can extract samples
                 control.family=control.family1)
#save(inla.hmod, file="./output/model_outputs/dec_2019/pathrich_mammals/inla.hmod1.R")


model_dat$PathRich = model_dat$ResidPathRich_NonHuman
form = formula(y ~ 1 + PathRich*LandUseType2 +
                 f(SS, model='iid', hyper=hyper1) +
                 f(SSB, model='iid', hyper=hyper1) +
                 f(bgb_SS, model='iid', hyper=hyper1) +
                 f(Order, model='iid', hyper=hyper1))
inla.nhmod = inla(form, data=model_dat, family=family1,
                  control.predictor=list(compute=T), # predict values (not on link scale)
                  control.compute=list(cpo=T, waic=T, config=TRUE), # specify config == T so can extract samples
                  control.family=control.family1)
#save(inla.nhmod, file="./output/model_outputs/dec_2019/pathrich_mammals/inla.nhmod1.R")

# 
# 
# 
# # -------------- combine with PREDICTS data and save ----------------------
# 
# # weightings and residuals
# # effw = read.csv("./output/model_outputs/effort_weighting/modelled_effortweights_allspecies.csv", stringsAsFactors=F)
# # effw = effw[ , -which(names(effw) == "Order")]
# pathrich = read.csv("./output/model_outputs/effort_weighting/pathogenrichness_residuals_allmammals.csv", stringsAsFactors=F)
# pathrich = pathrich[ , -which(names(pathrich) == "logPubs") ]
# 
# # combine and correction for axis axis/indicator indicator
# # dd1 = left_join(dd1, effw)
# dd1 = left_join(dd1, pathrich)
# # dd1$ProbDetection[ dd1$bgb %in% c("axis axis", "indicator indicator") ] = 1
# 
# # save
# save(dd1, file="./output/predicts_processed/predicts_pathogens.R")
# 
