### Gibb et al, 'Land use has global and systematic effects on local zoonotic host communities"
# Script 21: For mammals, fit full models of residual pathogen richness against probability of occurrence 
# in different land use classes, for non-human and human-shared pathogens
# Bootstrap approach: each uses a different sample of residual pathogen richness drawn from the pathrich-publciations model (script 5)
# To test for sensitivity to uncertainty in this relationship
# Script designed to be run on a cluster n times; script 22 processes the outputs and summarises

#setwd("Y:/chapter2_predicts/predicts_analysis_201901/")
setwd("/scratch/scratch/ucbtgib/predicts")

# dependencies
library(INLA)
library(lme4)
library(sp)
library(raster)
library(rgdal)
library(ggplot2)
library(nlme)
library(dismo)
library(ggmap)
library(Hmisc)
library(plyr)
library(dplyr)
library(gridExtra)


# ------------ data and directories --------------

# pathrich
save_dir = "./pathrich_bootstrap/"
unique_id = sample(1:10^8, 1)



# ------------ functions for model fitting and bootstrap -----------

# ---- Aggregate land use categories (updated 16/02/18) ----
#' @param df PREDICTS dataframe

prepareLandUse = function(df){
  
  # land use mappings
  plu = data.frame(Predominant_land_use = c("Primary vegetation","Young secondary vegetation","Intermediate secondary vegetation","Mature secondary vegetation", "Secondary vegetation (indeterminate age)", "Plantation forest","Pasture","Cropland","Urban"),
                   PLU2 = c("1_Primary", "Secondary","Secondary","Secondary", "Secondary", "Plantation forest", "Pasture", "Cropland", "Urban"),
                   LandUseType = c("1_Primary","Secondary","Secondary","Secondary","Secondary","Managed","Managed","Managed","Urban"),
                   stringsAsFactors = F)
  
  # merge and reshuffle columns
  df$foo = 1:nrow(df)
  df1 = merge(df, plu, by="Predominant_land_use", all.x=T)
  df1 = df1[ , c(2:ncol(df1), 1) ]
  df1 = df1[ order(df1$foo), ]
  
  # remove 'use' from intensity
  df1$Use_intensity = gsub(" use","", as.vector(df1$Use_intensity) )
  
  # combine use intensities into three states (minimal, substantial, cannot decide)
  df1$UseIntensity = ifelse(df1$Use_intensity == "Minimal", "Minimal", "Substantial")
  df1$UseIntensity[ df1$Use_intensity == "Cannot decide" ] = "Cannot decide"
  
  # combine with land use type for composite variable for modelling
  df1$LU_UI = paste(df1$LandUseType, df1$UseIntensity, sep="_")
  df1$LU_UI[ df1$LU_UI == "1_Primary_Substantial" ] = "Primary_Substantial"
  
  return( as.data.frame(df1, stringsAsFactors=F) )
}



# ------------ prepare and summarise data --------

# load PREDICTS data
#load(file="./data/predicts_processed/predicts_pathogens_researcheffort.R")
load(file="./predicts_pathogens_researcheffort.R")

# read in residual samples 
#hsamp = read.csv("./output/model_outputs/dec_2019/pathrich_mammals/residual_models_researcheffort/mammalia_human_residualPR_samples.csv", stringsAsFactors = F)
hsamp = read.csv("./mammalia_human_residualPR_samples.csv", stringsAsFactors = F)
#nhsamp = read.csv("./output/model_outputs/dec_2019/pathrich_mammals/residual_models_researcheffort/mammalia_nonhuman_residualPR_samples.csv", stringsAsFactors = F)
nhsamp = read.csv("./mammalia_nonhuman_residualPR_samples.csv", stringsAsFactors = F)




# ---------- set up data for model ------------

# subset and set up variables
dd1 = dd1[ dd1$Class == "Mammalia", ]
dd1 = prepareLandUse(dd1)
dd1$Presence = ifelse( dd1$Effort_corrected_measurement > 0, 1, 0 )
dd1$logAbund = log(dd1$Effort_corrected_measurement+1)

# remove urban: 66 studies, 1950 sites, 546 species
dd1 = dd1[ dd1$LandUseType != "Urban", ]

# randomly draw set of samples from human and non-human residuals
# centre and scale for model fitting
hsx = as.data.frame(t(hsamp[ sample(1:nrow(hsamp), 1), ]))
hsx$bgb = gsub("[.]", " ", row.names(hsx))
names(hsx)[1] = "RPR_Human"
hsx$RPR_Human[ hsx$bgb %in% c("procyon lotor", "canis lupus") ] = NA
hsx$RPR_Human = scale(hsx$RPR_Human)

nhsx = as.data.frame(t(nhsamp[ sample(1:nrow(nhsamp), 1), ]))
nhsx$bgb = gsub("[.]", " ", row.names(nhsx))
names(nhsx)[1] = "RPR_NonHuman"
nhsx$RPR_NonHuman[ nhsx$bgb %in% c("procyon lotor", "canis lupus") ] = NA
nhsx$RPR_NonHuman = scale(nhsx$RPR_NonHuman)

# combine with dd1
dd1 = left_join(dd1, left_join(hsx, nhsx))

# exclude major outliers and possible confounding domestics
dd1 = dd1[ !dd1$bgb %in% c("procyon lotor", "canis lupus"), ]



# --------- set up model ----------

# subset to only columns of interest, set up response variable
model_dat = dd1[ , c("Presence", "RPR_Human", "RPR_NonHuman", "LandUseType", "SS", "SSB", "SSBS", "bgb", "Order") ]
names(model_dat)[1] = "y"

# managed land as base factor for models, and nest species within study for random effects
rep = data.frame(LandUseType = c("1_Primary", "Secondary", "Managed"), LandUseType2 = c("Primary", "Secondary", "1_Managed"))
model_dat = full_join(model_dat, rep)
model_dat$bgb_SS = paste(model_dat$bgb, model_dat$SS, sep="_")

# select predictor variable
model_type = sample(c("human", "nonhuman"), 1)
if(model_type == "human"){ model_dat$PathRich = model_dat$RPR_Human }
if(model_type == "nonhuman"){ model_dat$PathRich = model_dat$RPR_NonHuman }



# ---------- run inference -------------

# run to work around server GLIBC issue
INLA:::inla.dynload.workaround()
err = simpleError("Error!")

# inla model specifications (binomial, logit link)
family1 = "binomial"
control.family1 = list(control.link=list(model="logit"))

# hyperprior (penalised complexity or log-gamma)
hyper1 = list(prec = list(prior='loggamma', param=c(0.1,0.1)))

# model specification and inference for human shared
# random intercepts for study, spatial block, species-study and order
# considered random slopes (PathRich|Order) but parameter estimates and confidence didn't change, and delta AIC was marginal (<6) - opted for simpler model
form = formula(y ~ 1 + PathRich*LandUseType2 + 
                 f(SS, model='iid', hyper=hyper1) + 
                 f(SSB, model='iid', hyper=hyper1) + 
                 f(bgb_SS, model='iid', hyper=hyper1) + 
                 f(Order, model='iid', hyper=hyper1))
mx = tryCatch(inla(form, data=model_dat, 
                   family=family1,
                   control.predictor=list(compute=T), # predict values (not on link scale)
                   control.compute=list(cpo=T, waic=T, config=TRUE), # specify config == T so can extract samples
                   control.family=control.family1), error=function(err) err)



# ---------- extract model posterior samples -----------

# extract posterior samples
sampx = inla.posterior.sample(3000, mx)

# extract from samples
getSampFixed = function(x){ 
  l = sampx[[x]]$latent
  l = l[ grep("Intercept|PathRich|LandUseType", row.names(l)), ]
  as.data.frame(t(l))
}
fixed_samples = do.call(rbind.data.frame, lapply(1:length(sampx), getSampFixed))
row.names(fixed_samples) = 1:nrow(fixed_samples)



# --------- create and return results object -----------

result = list()
result$model_type = model_type
result$fixed = mx$summary.fixed
result$hyper = mx$summary.hyperpar
result$samples = fixed_samples

# save
filename = paste(model_type, "pathrich", unique_id, ".R", sep="_")
save(result, file=paste(save_dir, filename, sep=""))


# -------- end -----------
