

### Gibb et al, 'Land use has global and systematic effects on local zoonotic host communities"
# Script 22: For mammals, fit full models of residual pathogen richness against probability of occurrence 
# in different land use classes, for non-human and human-shared pathogens
# View diagnostics, conduct sensitivity checks, and read and process result of bootstrap to check for effects of residual distributions on result

setwd("Y:/chapter2_predicts/predicts_analysis_201901/")

# dependencies
library(INLA)
library(lme4)
library(sp)
library(raster)
library(rgdal)
library(ggplot2)
library(nlme)
library(dismo)
library(ggmap)
library(Hmisc)
library(plyr)
library(dplyr)
library(gridExtra)

# functions
source("./00_functions_general.R")


# ---- directory to save outputs to ---------

save_dir = "./output/model_outputs/dec_2019/pathrich_mammals/"



# ---- Aggregate land use categories (updated 16/02/18) ----
#' @param df PREDICTS dataframe

prepareLandUse = function(df){
  
  # land use mappings
  plu = data.frame(Predominant_land_use = c("Primary vegetation","Young secondary vegetation","Intermediate secondary vegetation","Mature secondary vegetation", "Secondary vegetation (indeterminate age)", "Plantation forest","Pasture","Cropland","Urban"),
                   PLU2 = c("1_Primary", "Secondary","Secondary","Secondary", "Secondary", "Plantation forest", "Pasture", "Cropland", "Urban"),
                   LandUseType = c("1_Primary","Secondary","Secondary","Secondary","Secondary","Managed","Managed","Managed","Urban"),
                   stringsAsFactors = F)
  
  # merge and reshuffle columns
  df$foo = 1:nrow(df)
  df1 = merge(df, plu, by="Predominant_land_use", all.x=T)
  df1 = df1[ , c(2:ncol(df1), 1) ]
  df1 = df1[ order(df1$foo), ]
  
  # remove 'use' from intensity
  df1$Use_intensity = gsub(" use","", as.vector(df1$Use_intensity) )
  
  # combine use intensities into three states (minimal, substantial, cannot decide)
  df1$UseIntensity = ifelse(df1$Use_intensity == "Minimal", "Minimal", "Substantial")
  df1$UseIntensity[ df1$Use_intensity == "Cannot decide" ] = "Cannot decide"
  
  # combine with land use type for composite variable for modelling
  df1$LU_UI = paste(df1$LandUseType, df1$UseIntensity, sep="_")
  df1$LU_UI[ df1$LU_UI == "1_Primary_Substantial" ] = "Primary_Substantial"
  
  return( as.data.frame(df1, stringsAsFactors=F) )
}



# ------------ prepare and summarise data --------

# load PREDICTS data
load(file="./data/predicts_processed/predicts_pathogens_researcheffort.R")

# subset and set up variables
dd1 = dd1[ dd1$Class == "Mammalia", ]
dd1 = prepareLandUse(dd1)
dd1$Presence = ifelse( dd1$Effort_corrected_measurement > 0, 1, 0 )

# remove urban due to poor sampling: 66 studies, 1950 sites, 546 species
dd1 = dd1[ dd1$LandUseType != "Urban", ]

# exclude major outliers and possible confounding domestics
dd1 = dd1[ !dd1$bgb %in% c("procyon lotor", "canis lupus"), ]



# --------- set up model ----------

# subset to only columns of interest, set up response variable
df = dd1[ , c("Presence", "ResidPathRich_Human", "ResidPathRich_NonHuman", "LandUseType", "SS", "SSB", "SSBS", "bgb", "Order") ]
names(df)[1] = "y"

# managed land as base factor for models, and nest species within study for random effects
rep = data.frame(LandUseType = c("1_Primary", "Secondary", "Managed"), LandUseType2 = c("Primary", "Secondary", "1_Managed"))
df = full_join(df, rep)
df$bgb_SS = paste(df$bgb, df$SS, sep="_")

# inla model specifications (binomial, logit link)
family1 = "binomial"
control.family1 = list(control.link=list(model="logit"))

# hyperprior (log-gamma)
hyper1 = list(prec = list(prior='loggamma', param=c(0.1,0.1)))

# model specification and inference
# random intercepts for study, spatial block, species-study and order
# considered random slopes (PathRich|Order) but parameter estimates and confidence didn't change, and delta AIC was marginal (<6) - opted for simpler model
form = formula(y ~ 1 + PathRich*LandUseType2 + 
                 f(SS, model='iid', hyper=hyper1) + 
                 f(SSB, model='iid', hyper=hyper1) + 
                 f(bgb_SS, model='iid', hyper=hyper1) + 
                 f(Order, model='iid', hyper=hyper1))

# human-shared pathogens
df$PathRich = df$ResidPathRich_Human
mh = tryCatch(inla(form, data=df, 
                   family=family1,
                   control.predictor=list(compute=T), # predict values (not on link scale)
                   control.compute=list(cpo=T, waic=T, config=TRUE), # specify config == T so can extract samples
                   control.family=control.family1), error=function(err) err)
save(mh, file=paste(save_dir, "PathRich_logistic_humanshared_model_full.R", sep=""))

# non human-shared pathogens
df$PathRich = df$ResidPathRich_NonHuman
mnh = tryCatch(inla(form, data=df, 
                   family=family1,
                   control.predictor=list(compute=T), # predict values (not on link scale)
                   control.compute=list(cpo=T, waic=T, config=TRUE), # specify config == T so can extract samples
                   control.family=control.family1), error=function(err) err)
save(mnh, file=paste(save_dir, "PathRich_logistic_nonhumanshared_model_full.R", sep=""))



### -------------------- model diagnostics --------------------------

# plot nth hyperparameter
# plotHyperparMarginal = function(model, n){
#   m.sigma = inla.tmarginal(fun = function(x) exp(-1/2*x), marginal = model$internal.marginals.hyperpar[[n]])
#   plot(m.sigma, type="l", xlab = expression(sigma[iid]), main=names(model$internal.marginals.hyperpar)[n], ylab = "Probability density")
# }
# 
# # plot nth random/fixed effect means distribution
# plotRanef = function(model, n){ plot(density(model$summary.random[[n]]$mean), main = names(model$summary.random)[n] ) }
# plotFixef = function(model, n){ plot(model$marginals.fixed[[n]], main=names(model$marginals.fixed)[n], type="l"); abline(v=0, col="blue") }
# 
# # plot CPO scores by order or land use type
# plotCPO2 = function(model, data, param){
#   data$cpo = model$cpo$cpo
#   order_spp = ddply(data, .(Order), plyr::summarise, n=length(unique(bgb_SS)))
#   data$Order = factor(data$Order, levels=order_spp$Order[ order(order_spp$n, decreasing=T) ], ordered=T)
#   if(param == "order"){ p = ggplot(data, aes(Order, cpo)) + geom_boxplot() + theme(axis.text.x = element_text(angle=90)) }
#   if(param == "lu"){ p = ggplot(data, aes(LandUseType, cpo)) + geom_boxplot() + theme(axis.text.x = element_text(angle=90)) }
#   p
# }


### ----------------sensitivity analysis by data source: GMPD major source so inventories likely to be much more complete ------------

# (1) exclude GMPD orders (58 studies, 1804 sites)

# human-shared
df2 = df[ !df$Order %in% c("Primates", "Carnivora", "Cetertiodactyla", "Perissodactyla"), ]
df2$PathRich = df2$ResidPathRich_Human
form = y ~ 1 + PathRich*LandUseType2 + f(SS, model='iid', hyper=hyper1) + f(SSB, model='iid', hyper=hyper1) + f(bgb_SS, model='iid', hyper=hyper1) + f(Order, model='iid', hyper=hyper1)
m1.1 = inla(form, data=df2, 
            family=family1, 
            control.predictor=list(compute=T), # predict values (not on link scale)
            control.compute=list(cpo=T, waic=T, config=TRUE), # specify config == T so can extract samples
            control.family=control.family1)
save(m1.1, file="./PathRich_logistic_humanshared_model_full_EXgmpd.R")
# plotFixed(getFixed(m1.1))
# par(mfrow=c(2, 2)); plotHyperparMarginal(m1.1, 1); plotHyperparMarginal(m1.1, 2); plotHyperparMarginal(m1.1, 3); plotHyperparMarginal(m1.1, 4)
# plotRanef(m1.1, 1); plotRanef(m1.1, 2); plotRanef(m1.1, 3); plotRanef(m1.1, 4)
# par(mfrow=c(2, 3)); plotFixef(m1.1, 1); plotFixef(m1.1, 2); plotFixef(m1.1, 3); plotFixef(m1.1, 4); plotFixef(m1.1, 5); plotFixef(m1.1, 6)
# par(mfrow=c(1,1))

# not human shared
df2$PathRich = df2$ResidPathRich_NonHuman
m1.2 = inla(form, data=df2, family=family1, 
            control.predictor=list(compute=T), # predict values (not on link scale)
            control.compute=list(cpo=T, waic=T, config=TRUE), # specify config == T so can extract samples
            control.family=control.family1)
save(m1.2, file="./PathRich_logistic_nonhumanshared_model_full_EXgmpd.R")
# plotFixed(getFixed(m1.2))
# par(mfrow=c(2, 2)); plotHyperparMarginal(m1.2, 1); plotHyperparMarginal(m1.2, 2); plotHyperparMarginal(m1.2, 3); plotHyperparMarginal(m1.2, 4)
# plotRanef(m1.2, 1); plotRanef(m1.2, 2); plotRanef(m1.2, 3); plotRanef(m1.2, 4)
# par(mfrow=c(2, 3)); plotFixef(m1.2, 1); plotFixef(m1.2, 2); plotFixef(m1.2, 3); plotFixef(m1.2, 4); plotFixef(m1.2, 5); plotFixef(m1.2, 6)
# par(mfrow=c(1,1))


# (2) only GMPD orders (25 studies, 887 sites)

# human-shared
df2 = df[ df$Order %in% c("Primates", "Carnivora", "Cetertiodactyla", "Perissodactyla"), ]
df2$PathRich = df2$ResidPathRich_Human
m2.1 = inla(form, data=df2, 
            family=family1, 
            control.predictor=list(compute=T), # predict values (not on link scale)
            control.compute=list(cpo=T, waic=T, config=TRUE), # specify config == T so can extract samples
            control.family=control.family1)
save(m2.1, file="./PathRich_logistic_humanshared_model_full_ONLYgmpd.R")
# plotFixed(getFixed(m2.1))
# plotFixed(getFixed(m1.2))
# par(mfrow=c(2, 2)); plotHyperparMarginal(m2.1, 1); plotHyperparMarginal(m2.1, 2); plotHyperparMarginal(m2.1, 3); plotHyperparMarginal(m2.1, 4)
# plotRanef(m2.1, 1); plotRanef(m2.1, 2); plotRanef(m2.1, 3); plotRanef(m2.1, 4)
# par(mfrow=c(2, 3)); plotFixef(m2.1, 1); plotFixef(m2.1, 2); plotFixef(m2.1, 3); plotFixef(m2.1, 4); plotFixef(m2.1, 5); plotFixef(m2.1, 6)
# par(mfrow=c(1,1))


# not human shared
df2$PathRich = df2$ResidPathRich_NonHuman
m2.2 = inla(form, data=df2, 
            family=family1, 
            control.predictor=list(compute=T), # predict values (not on link scale)
            control.compute=list(cpo=T, waic=T, config=TRUE), # specify config == T so can extract samples
            control.family=control.family1)
save(m2.2, file="./PathRich_logistic_nonhumanshared_model_full_ONLYgmpd.R")
# plotFixed(getFixed(m2.2))
# plotFixed(getFixed(m1.2))
# par(mfrow=c(2, 2)); plotHyperparMarginal(m2.2, 1); plotHyperparMarginal(m2.2, 2); plotHyperparMarginal(m2.2, 3); plotHyperparMarginal(m2.2, 4)
# plotRanef(m2.2, 1); plotRanef(m2.2, 2); plotRanef(m2.2, 3); plotRanef(m2.2, 4)
# par(mfrow=c(2, 3)); plotFixef(m2.2, 1); plotFixef(m2.2, 2); plotFixef(m2.2, 3); plotFixef(m2.2, 4); plotFixef(m2.2, 5); plotFixef(m2.2, 6)
# par(mfrow=c(1,1))




### --------------- sensitivity analysis by residuals (bootstrap) ----------------

# directory where bootstrap outputs are stored 
files_dir = "./output/model_outputs/dec_2019/pathrich_mammals/pathrich_bootstrap/"
ff = list.files(files_dir, pattern=".R")

# get info about bootstraps
ff_doc = do.call(rbind.data.frame, strsplit(ff, "_"))
names(ff_doc) = c("type", "m", "id", "ff")
ff_doc$loc = paste(files_dir, ff, sep="")

# read in human/non-human results and summarise across all samples
processBootstrap = function(type, num_models=400){
  
  # subset to type and subsample num models
  locs = ff_doc[ sample(which(as.vector(ff_doc$type) == type), num_models, replace=FALSE), ]
  
  # read and return samples
  readModel = function(x){
    load(locs$loc[x])
    samp = result$samples
    #samp$id = as.numeric(locs$id[x])
    samp
  }
  samp_type = do.call(rbind.data.frame, lapply(1:nrow(locs), readModel))
  
  # summarise
  results = data.frame(
    median = apply(samp_type, 2, median),
    lower_95 = apply(samp_type, 2, quantile, 0.025), # 95% interval
    lower_90 = apply(samp_type, 2, quantile, 0.05), # 90% interval
    lower_67 = apply(samp_type, 2, quantile, 0.167), # 67% interval
    upper_67 = apply(samp_type, 2, quantile, 0.833),
    upper_90 = apply(samp_type, 2, quantile, 0.95),
    upper_95 = apply(samp_type, 2, quantile, 0.975)
  )
  results$param = row.names(results)
  results$modeltype = type
  results$nboot = num_models
  results
}

# run and save as table
# results robust
boot_results = do.call(rbind.data.frame, lapply(c("human", "nonhuman"), processBootstrap, num_models=400))



# ----------- combine all results into a table showing full model and sensitivity checks ------------

# for human-shared
hs1 = mh$summary.fixed
hs_result = data.frame(Model = "Human-shared",
                       Parameter = row.names(hs1),
                       Type = c("intercept", "continuous", "categorical", "categorical", "continuous (interaction)", "continuous (interaction)"),
                       Estimate_full = paste(round(hs1$`0.5quant`, 3), " (", round(hs1$`0.025quant`, 3), ", ", round(hs1$`0.975quant`, 3), ")", sep=""))
br1 = boot_results[ boot_results$modeltype == "human", ]
hs_result$Estimate_boot400 = paste(round(br1$median, 3), " (", round(br1$lower_95, 3), ", ", round(br1$upper_95, 3), ")", sep="")
hs2 = m1.1$summary.fixed
hs_result$Estimate_exGMPD = paste(round(hs2$`0.5quant`, 3), " (", round(hs2$`0.025quant`, 3), ", ", round(hs2$`0.975quant`, 3), ")", sep="")
hs3 = m2.1$summary.fixed
hs_result$Estimate_onlyGMPD = paste(round(hs3$`0.5quant`, 3), " (", round(hs3$`0.025quant`, 3), ", ", round(hs3$`0.975quant`, 3), ")", sep="")
hs_result$Parameter = c("Intercept", "PathRich", "LU_Primary", "LU_Secondary", "LU_Primary:PathRich", "LU_Secondary:PathRich")

# for non human-shared
hs1 = mnh$summary.fixed
nhs_result = data.frame(Model = "Non-human",
                       Parameter = row.names(hs1),
                       Type = c("intercept", "continuous", "categorical", "categorical", "continuous (interaction)", "continuous (interaction)"),
                       Estimate_full = paste(round(hs1$`0.5quant`, 3), " (", round(hs1$`0.025quant`, 3), ", ", round(hs1$`0.975quant`, 3), ")", sep=""))
br1 = boot_results[ boot_results$modeltype == "nonhuman", ]
nhs_result$Estimate_boot400 = paste(round(br1$median, 3), " (", round(br1$lower_95, 3), ", ", round(br1$upper_95, 3), ")", sep="")
hs2 = m1.2$summary.fixed
nhs_result$Estimate_exGMPD = paste(round(hs2$`0.5quant`, 3), " (", round(hs2$`0.025quant`, 3), ", ", round(hs2$`0.975quant`, 3), ")", sep="")
hs3 = m2.2$summary.fixed
nhs_result$Estimate_onlyGMPD = paste(round(hs3$`0.5quant`, 3), " (", round(hs3$`0.025quant`, 3), ", ", round(hs3$`0.975quant`, 3), ")", sep="")
nhs_result$Parameter = c("Intercept", "PathRich", "LU_Primary", "LU_Secondary", "LU_Primary:PathRich", "LU_Secondary:PathRich")

# combine
result_table = rbind(hs_result, nhs_result)
write.csv(result_table, "./output/tables/SI_PathRich_models_summarytable.csv", row.names=FALSE)
