
# Gibb et al, 'Land use has global and systematic effects on zoonotic host communities'
# Script 23: Plot results of mammal species occurrence-pathogen richness models
# Uses full models fitted in previous script (22), draws samples and visualises results

# dependencies
library(lme4)
library(ggplot2)
library(gridExtra)
library(devtools)
#install_github("timnewbold/StatisticalModels")
library(StatisticalModels)
library(dplyr)
library(INLA)

# functions
source("00_functions_general.R")

# colours for plotting
# nonhostcol = "#018571"
# hostcol = "#8c510a"
hostcol = inferno(200)[30]
nonhostcol = inferno(200)[110]

# calc SSS
calcSSS = function(df){ return(data.frame(NumStudies = length(unique(df$SS)),
                                          NumSites = length(unique(df$SSS)),
                                          NumHosts = length(unique(df$bgb[df$IsZoonotic_Strict==1])),
                                          NumNonHosts = length(unique(df$bgb[df$IsZoonotic_Strict==0])),
                                          NumPrimary = sum(df$LandUseType[ !duplicated(df$SSS) ] == "1_Primary"),
                                          NumSecondary = sum(df$LandUseType[ !duplicated(df$SSS) ] == "Secondary"),
                                          NumManaged = sum(df$LandUseType[ !duplicated(df$SSS) ] == "Managed"),
                                          NumUrban = sum(df$LandUseType[ !duplicated(df$SSS) ] == "Urban") ))}

# ------------- load data -----------------

# load PREDICTS data
load(file="./data/predicts_processed/predicts_pathogens_researcheffort.R")

# subset and set up variables
dd1 = dd1[ dd1$Class == "Mammalia", ]
dd1 = prepareLandUse(dd1)
dd1$Presence = ifelse( dd1$Effort_corrected_measurement > 0, 1, 0 )

# remove urban due to poor sampling: 66 studies, 1950 sites, 546 species
dd1 = dd1[ dd1$LandUseType != "Urban", ]

# exclude major outliers and possible confounding domestics
dd1 = dd1[ !dd1$bgb %in% c("procyon lotor", "canis lupus"), ]

# subset to only columns of interest, set up response variable
df = dd1[ , c("Presence", "ResidPathRich_Human", "ResidPathRich_NonHuman", "LandUseType", "SS", "SSB", "SSBS", "bgb", "Order") ]
names(df)[1] = "y"



# ------------- load models for pathogen richness: human and non-human -----------------

# load human and non-human, non-domestic pathogen richness models
load(file="./output/model_outputs/dec_2019/pathrich_mammals/PathRich_logistic_humanshared_model_full.R")
load(file="./output/model_outputs/dec_2019/pathrich_mammals/PathRich_logistic_nonhumanshared_model_full.R")

# set variable names
mod1 = mh
mod2 = mnh
basefactor="Managed"
legend.x=TRUE



#------------ Taxonomic distribution ------------

mam = dd1[ !duplicated(dd1$bgb), ]
mam1 = as.data.frame(table(mam$Order))
mam1 = mam1[ order(mam1$Freq), ]
mam1$Var1 = factor(mam1$Var1, levels=mam1$Var1, ordered=T)
mamplot = ggplot(mam1, aes(Var1, Freq)) + 
  geom_bar(stat="identity", fill="grey50", col="black") + 
  coord_flip() + 
  scale_y_continuous(position = "right") +
  ylab("Num. species") + theme_bw() +
  theme(axis.title.y=element_blank())
  


# ------------ plot model outputs (parameter estimates) ------------

# combine parameter estimates
f = getFixed(mod1); f$model = "Human-shared"
f$param_type = c("intercept", "slope", "intercept", "intercept", "slope", "slope")
if(basefactor == "Managed"){ f$LU = c("Managed", "Managed", "Primary", "Secondary", "Primary", "Secondary") }
if(basefactor == "Primary"){ f$LU = c("Primary", "Primary", "Managed", "Secondary", "Managed", "Secondary") }
f1 = getFixed(mod2); f1$model = "Not human-shared"
f1$param_type = c("intercept", "slope", "intercept", "intercept", "slope", "slope")
if(basefactor == "Managed"){ f1$LU = c("Managed", "Managed", "Primary", "Secondary", "Primary", "Secondary") }
if(basefactor == "Primary"){ f1$LU = c("Primary", "Primary", "Managed", "Secondary", "Managed", "Secondary") }
f = rbind(f, f1)
if(basefactor == "Managed"){ f$LU = factor(f$LU, levels=c("Managed", "Primary", "Secondary"), ordered=T) }
if(basefactor == "Primary"){ f$LU = factor(f$LU, levels=c("Managed", "Primary", "Secondary"), ordered=T) }
#f$model = modelname

# plot base factor slope
#p1 = ggplot(f[ f$param == "PathRich" & f$model == "Human-shared", ], aes(LU, quant05, col=model)) + 
p1 = ggplot(f[ f$param == "PathRich", ], aes(LU, quant05, col=model)) +   
  geom_point(cex=3, position=position_dodge(width=0.5)) +
  geom_errorbar(aes(ymin=quant025, ymax=quant975, col=model), lwd=0.7, width=0.15, position=position_dodge(width=0.5)) + 
  ylab("slope (log odds)") + 
  ylim(-0.25, 0.5) +
  xlab("") + theme_classic() + geom_hline(yintercept=0, lty=2) +
  theme(legend.position = "none",
        panel.grid.major = element_line(),
        panel.grid.minor = element_line(),
        axis.text = element_text(size=11),
        axis.title = element_text(size=11)) +
  scale_color_manual(values=c("Human-shared"=hostcol, "Not human-shared"=nonhostcol))

# plot delta slope for other factors
#p2 = ggplot(f[ f$param_type == "slope" & f$LU != "Primary" & f$model == "Human-shared", ], aes(LU, quant05, col=model)) + 
p2 = ggplot(f[ f$param_type == "slope" & f$LU != basefactor, ], aes(LU, quant05, col=model)) + 
  geom_point(cex=3, position=position_dodge(width=0.5)) +
  geom_errorbar(aes(ymin=quant025, ymax=quant975, col=model), lwd=0.7, width=0.15, position=position_dodge(width=0.5)) + 
  ylab(expression(paste(Delta, " slope (log odds)"))) + 
  ylim(-0.5, 0.25) +
  xlab("") + theme_classic() + geom_hline(yintercept=0, lty=2) +
  theme(legend.position = "none",
        panel.grid.major = element_line(),
        panel.grid.minor = element_line(),
        axis.text = element_text(size=11),
        axis.title = element_text(size=11)) +
  scale_color_manual(values=c("Human-shared"=hostcol, "Not human-shared"=nonhostcol))


ylab1 = "intercept (log odds)"
ylab2 = expression(paste(Delta, " intercept (log odds)"))

# plot
p3 = ggplot(f[ f$param == "(Intercept)", ], aes(LU, quant05, col=model)) +   
  geom_point(cex=3, position=position_dodge(width=0.5)) +
  geom_errorbar(aes(ymin=quant025, ymax=quant975, col=model), lwd=0.7, width=0.15, position=position_dodge(width=0.5)) + 
  ylab(ylab1) +
  ylim(-1.5, 1.1) +
  xlab("") + theme_classic() + 
  geom_hline(yintercept=0, lty=2) +
  theme(legend.position = "none",
        legend.text = element_text(size=11),
        panel.grid.major = element_line(),
        panel.grid.minor = element_line(),
        axis.text = element_text(size=11),
        axis.title = element_text(size=11)) +
  scale_color_manual(values=c("Human-shared"=hostcol, "Not human-shared"=nonhostcol))

p4 = ggplot(f[ f$param_type == "intercept" & f$LU != basefactor, ], aes(LU, quant05, col=model)) + 
  geom_point(cex=3, position=position_dodge(width=0.5)) +
  geom_errorbar(aes(ymin=quant025, ymax=quant975, col=model), lwd=0.7, width=0.15, position=position_dodge(width=0.5)) + 
  ylab(ylab2) + 
  ylim(-0.1, 0.6) +
  xlab("") + theme_classic() + geom_hline(yintercept=0, lty=2) +
  theme(legend.position = "none",
        panel.grid.major = element_line(),
        panel.grid.minor = element_line(),
        axis.text = element_text(size=11),
        axis.title = element_text(size=11)) +
  scale_color_manual(values=c("Human-shared"=hostcol, "Not human-shared"=nonhostcol))

  
# add legends if specified
if(legend.x == TRUE){ p3 = p3 + theme(legend.position = c(0.62, 0.9), legend.title = element_blank()) }
#if(legend.y == TRUE){ p2 = p2 + theme(legend.position = c(0.35, 0.9), legend.title = element_blank()) }

#grid.arrange(p3, p4, h1, p1, p2, h2, nrow=2, ncol=3)
plot1 = grid.arrange(p3, p4, p1, p2, nrow=2, ncol=2)



# ------------ Plot model predictions (probability scale) ------------

# draw samples from posterior and calculate outcomes

n.samples = 3000
samples_hmod = inla.posterior.sample(n.samples, result = mh)
samples_nhmod = inla.posterior.sample(n.samples, result = mnh)

# extract relevant data from sample x
extractSampleData = function(x){
  
  # hyperparameters
  hyp = as.data.frame(samples[[x]]$hyperpar)
  
  # latent fixed effects
  par = tail(samples[[x]]$latent, 6)
  
  # combine and return
  sampx = as.vector(c(par[, 1], hyp[, 1]))
  names(sampx) = c(row.names(par), row.names(hyp))
  names(sampx)[ 1:6 ] = c("(Intercept)", "PathRich", "LandUseType2Primary", "LandUseType2Secondary", "PathRich:LandUseType2Primary", "PathRich:LandUseType2Secondary")
  sampx
}

# samples from posterior
samples = samples_hmod
samp_h = as.data.frame(do.call(rbind, lapply(1:length(samples), extractSampleData)))
samples = samples_nhmod
samp_nh = as.data.frame(do.call(rbind, lapply(1:length(samples), extractSampleData)))


# use samples from posterior to predict on new data

# new data
newdat = expand.grid(unique(df$LandUseType), seq(-6, 7.8, length.out=300), stringsAsFactors = F)
names(newdat) = c("LandUseType", "PathRich_Resid")
binCol = function(v, category){ ifelse(v == category, 1, 0) }
newdat$Primary = binCol(newdat$LandUseType, "1_Primary")
newdat$Secondary = binCol(newdat$LandUseType, "Secondary")
newdat$Managed = binCol(newdat$LandUseType, "Managed")

# formula
mh$.args$formula
mnh$.args$formula

# function to calculate prediction
calculatePrediction = function(n){
  
  # for nth sample
  s = samp[n, ]
  
  # calculate fixed effects
  intercept = s$`(Intercept)` # intercept
  int_prim = s$LandUseType2Primary * newdat$Primary # intercept for secondary
  int_sec = s$LandUseType2Secondary * newdat$Secondary # intercept for managed
  slope = s$PathRich * newdat$PathRich_Resid # path richness * slope
  slope_prim = s$`PathRich:LandUseType2Primary` * newdat$PathRich_Resid * newdat$Primary # add to slope depending on sec/managed
  slope_sec = s$`PathRich:LandUseType2Secondary` * newdat$PathRich_Resid * newdat$Secondary
  
  # don't include random effect
  # calculate random effects (mean 0, sd = sigma)
  # ss_ranef = rnorm(nrow(newdat), sd=s$`Precision for SS`^-0.5)
  # bgb_ranef = rnorm(nrow(newdat), sd=s$`Precision for bgb`^-0.5)
  # order_ranef = rnorm(nrow(newdat), sd=s$`Precision for Order`^-0.5)
  
  # add to get estimate (linear predictor) across all new data for that one sample - including uncertainty in the intercept
  linear_pred = intercept + int_prim + int_sec + slope + slope_prim + slope_sec
  
  # OR only uncertainty around slope - need to add intercept manually from model
  #linear_pred = slope + slope_prim + slope_sec
  
  # transform to probability scale
  #prob_pred = inla.link.invlogit(linear_pred)
  #prob_pred
  linear_pred
}

# calculate across all samples
samp = samp_h
preds_h = do.call(cbind, lapply(1:nrow(samp), calculatePrediction))
samp = samp_nh
preds_nh = do.call(cbind, lapply(1:nrow(samp), calculatePrediction))



# ------------ summarise samples to visualise predictions ------------

# preds for human-shared
nd1 = newdat
nd1$median = apply(preds_h, 1, median)
nd1$upper = apply(preds_h, 1, quantile, probs=0.975)
nd1$lower = apply(preds_h, 1, quantile, probs=0.025)
nd1$upper1 = apply(preds_h, 1, quantile, probs=0.8325)
nd1$lower1 = apply(preds_h, 1, quantile, probs=0.1675)
#nd1[ , c("median", "upper", "lower")] = inla.link.invlogit(nd1[ , c("median", "upper", "lower")])
nd1[ , c("median", "upper", "lower", "upper1", "lower1")] = inla.link.invlogit(nd1[ , c("median", "upper", "lower", "upper1", "lower1")])
nd1$LandUseType[ nd1$LandUseType == "1_Primary" ] = "Primary"
nd1$LandUseType = factor(nd1$LandUseType, levels=c("Primary", "Secondary", "Managed"), ordered=T)


# preds for non human shared
nd2 = newdat
nd2$median = apply(preds_nh, 1, median)
nd2$upper = apply(preds_nh, 1, quantile, probs=0.975) # 95% credible interval
nd2$lower = apply(preds_nh, 1, quantile, probs=0.025)
nd2$upper1 = apply(preds_nh, 1, quantile, probs=0.8325) # 67% credible interval (i.e. frequentist S.E.)
nd2$lower1 = apply(preds_nh, 1, quantile, probs=0.1675)
nd2[ , c("median", "upper", "lower", "upper1", "lower1")] = inla.link.invlogit(nd2[ , c("median", "upper", "lower", "upper1", "lower1")])
nd2$LandUseType[ nd2$LandUseType == "1_Primary" ] = "Primary"
nd2$LandUseType = factor(nd2$LandUseType, levels=c("Primary", "Secondary", "Managed"), ordered=T)

# plot
n1 = ggplot(nd1, aes(PathRich_Resid, median)) + 
  geom_ribbon(aes(ymin=lower, ymax=upper), alpha=0.3, fill=hostcol) +
  geom_ribbon(aes(ymin=lower1, ymax=upper1), alpha=0.3, fill=hostcol) +
  geom_line(size=0.8) +
  facet_wrap(~LandUseType) + geom_vline(xintercept=0, lty=2) +
  theme_bw() +
  ylim(0, 1) +
  xlab("Residual pathogen richness (human-shared)") +
  ylab("Occurrence probability") +
  theme(axis.title = element_text(size=11),
        axis.text = element_text(size=10), strip.text.x = element_text(size = 10))

# plot
n2 = ggplot(nd2, aes(PathRich_Resid, median)) + 
  geom_ribbon(aes(ymin=lower, ymax=upper), alpha=0.3, fill=nonhostcol) +
  geom_ribbon(aes(ymin=lower1, ymax=upper1), alpha=0.3, fill=nonhostcol) +
  geom_line(size=0.8) +
  facet_wrap(~LandUseType) + geom_vline(xintercept=0, lty=2) +
  theme_bw() +
  ylim(0, 1) +
  xlab("Residual pathogen richness (not human-shared)") +
  ylab("Occurrence probability") +
  theme(axis.title = element_text(size=11),
        axis.text = element_text(size=10),
        strip.text.x = element_text(size = 10))

# plot2 = grid.arrange(n1, n2, nrow=2)
# ggsave(grid.arrange(plot1, plot2, ncol=1),
#        file="./output/figures/nature_ms3_jan2019/ExtendedData7_PathRich_results.png",
#        device="png", dpi=900,
#        units="in", width=6, height=10, scale=0.9)


### combine into full plot and save

pp = gridExtra::arrangeGrob(p3, p4, p1, p2, n1, n2,
                            heights=c(1, 1, 1, 1), widths=c(0.5, 0.5),
                            layout_matrix = rbind(c(1, 2),
                                                  c(3, 4),
                                                  c(5, 5),
                                                  c(6, 6)))
pp = ggpubr::as_ggplot(pp)  +
  cowplot::draw_plot_label(label = c("A", "B", "C", "D", "E", "F"), 
                           fontface = "bold", size = 18, 
                           x = c(0.105, 0.605, 0.122, 0.625, 0.05, 0.05), y = c(1, 1, 0.75, 0.75, 0.5, 0.25))
ggsave(pp,
       file="./output/figures/nature_ms3_jan2019/ExtendedData7_PathRich_results.png",
       device="png", dpi=900,
       units="in", width=6, height=10, scale=0.9)
